@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -332,8 +332,10 @@ select_basis_set(int atomIndex,
 
 /** Returns number of shells needed to describe the electronic density
     for given molecule and basis set.
-    
-    @param molecule a molecule for which the shells are to be counted.
+
+    @param atomList list of atoms
+
+    @param noOfAtoms the length of atomList
 
     @param basissetDefault the basis set to be used for all atoms but
     those specified by rangeList.
