@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "densfromf_general.h"
 #include "densfromf_sparse.h"
 #include "densfromf_full.h"
+#include "densfromf_stochastic.h"
 #include "output.h"
 #include "utilities.h"
 
@@ -65,6 +66,8 @@ int get_dens_from_fock_general(int n,
 			       int create_m_files,
 			       int ignore_purification_failure,
 			       int use_rand_perturbation_for_alleigsint,
+			       int use_stochastic_orbitals,
+			       const std::vector< std::vector<ergo_real> > stochastic_orbitals,
 			       std::string stats_prefix,
 			       std::map<std::string, double> & puri_stats,
 			       int do_sparsity_investigation,
@@ -75,9 +78,27 @@ int get_dens_from_fock_general(int n,
 			       generalVector * eigVecHOMO
 			       )
 {
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_dens_from_fock_general, n = %i, use_diagonalization = %i, use_diag_on_error = %i", 
-	    n, use_diagonalization, use_diag_on_error);
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_dens_from_fock_general, n = %i, use_diagonalization = %i, use_diag_on_error = %i, use_stochastic_orbitals = %i", 
+	    n, use_diagonalization, use_diag_on_error, use_stochastic_orbitals);
   resultEntropyTerm = 0; // In nonzero temperature case, this will be set to nonzero value later.
+  if(use_stochastic_orbitals == 1) {
+    resultDens.readFromFile();
+    resultDens.clear();
+    if(get_dens_from_fock_stochastic(n, 
+				     noOfOccupiedOrbs, 
+				     resultDens,
+				     factor,
+				     Finput,
+				     invCholFactor,
+				     matrixSizesAndBlocks,
+				     stochastic_orbitals) != 0) {
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "get_dens_from_fock_general: error in get_dens_from_fock_stochastic; aborting.");
+      return -1;
+    }
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_dens_from_fock_stochastic finished");
+    resultDens.writeToFile();
+    return 0;
+  }
   int use_diag = 0;
   if(use_diagonalization == 1)
     use_diag = 1;
