@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,6 +66,8 @@ int get_dens_from_fock_general(int n, /**< System size. */
 			       int create_m_files, /**< Flag to create m-files with information about the purification process. */
 			       int ignore_purification_failure, /**< Continue even if purification fails to converge. */
 			       int use_rand_perturbation_for_alleigsint, /**< Apply a random perturbation to (try to) improve the convergence speed of Lanczos calculation of extremal eigenvalues.  */
+			       int use_stochastic_orbitals, /**< Use stochastic orbitals instead of standard density matrix construction.  */
+			       const std::vector< std::vector<ergo_real> > stochastic_orbitals, /**< Vector of stochastic orbitals (empty vector if stochastic orbitals not used).  */
 			       std::string stats_prefix, /**<  Prefix to be added to statistics files. */
 			       std::map<std::string, double> & puri_stats, /**< Map to store stats for purification. */
 			       int do_sparsity_investigation, /**< Flag to turn on sparsity investigation. */
