@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ static pthread_mutex_t dft_prop_mutex = PTHREAD_MUTEX_INITIALIZER;
 #include "functionals.h"
 #include "dft_common.h"
 
-#include "gblas.h"
+#include "mat_gblas.h"
 #include "output.h"
 #include "utilities.h"
 #include "matrix_utilities.h"
