@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@
 #endif
 
 #include "atom_labels.h"
-#include "density_description_file_2.h"
+#include "density_description_file.h"
 #include "ergo_scripted.h"
 #include "grid_reader.h"
 #include "dft_common.h"
@@ -80,6 +80,9 @@
 #include "electron_dynamics.h"
 #include "tdhf_dynamics.h"
 
+// ELIAS NOTE 2014-07-14: define SKIP_UNOFFICIAL_INPUT_PARAMS for "official" releases of the code, so that some testing/debugging parameters are skipped.
+#define SKIP_UNOFFICIAL_INPUT_PARAMS
+
 static void variable_free(struct variable* v);
 
 /** An object representing the state of the input processor.  A way to
@@ -553,8 +556,10 @@ jkparams_set_from_vars(JK::Params& jkp)
   jkp.threshold_J = var_get_realJK("threshold_2el_J");
   jkp.threshold_K = var_get_realJK("threshold_2el_K");
   jkp.multipole_threshold_factor = var_get_realJK("multipole_threshold_factor");
-  //  jkp.use_densfit_for_J          = var_get_intJK("use_densfit_for_J");
-  //  jkp.use_differential_density   = var_get_intJK("use_differential_density");
+#ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
+  jkp.use_densfit_for_J          = var_get_intJK("use_densfit_for_J");
+  jkp.use_differential_density   = var_get_intJK("use_differential_density");
+#endif
   jkp.use_fmm                    = var_get_intJK("use_fmm");
   jkp.fmm_box_size               = var_get_realJK("fmm_box_size");
   jkp.fmm_no_of_branches = var_get_intJK("fmm_no_of_branches");
@@ -673,21 +678,26 @@ es_run(const char *mode, int save_pot)
     const int do_basis_normalization = 1;
     const int skip_sort_shells = 0;
 
-    if(Basis_info->addBasisfuncsForMolecule(ergo.molecule, 
-                                            ergo.Basis,
-                                            Ergo::NO_OF_BASIS_SET_RANGES,
-                                            ergo.basissetRangeList,
-                                            *ergoIntegralInfo, 
-                                            output_basis,
-                                            do_basis_normalization,
-                                            skip_sort_shells) != 0)
-      {
-	do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, 
-		  "error in basisInfo->add_basisfuncs_for_molecule "
-		  "for main basis set, Basis='%s'",
-                  ergo.Basis);
-	return -1;
+    if(ergo.Basis != NULL) {
+      // We skip adding basis functions here if the special basis set string "none" is given.
+      if(strcmp(ergo.Basis, "none") != 0) {
+	if(Basis_info->addBasisfuncsForMolecule(ergo.molecule, 
+						ergo.Basis,
+						Ergo::NO_OF_BASIS_SET_RANGES,
+						ergo.basissetRangeList,
+						*ergoIntegralInfo, 
+						output_basis,
+						do_basis_normalization,
+						skip_sort_shells) != 0)
+	  {
+	    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, 
+		      "error in basisInfo->add_basisfuncs_for_molecule "
+		      "for main basis set, Basis='%s'",
+		      ergo.Basis);
+	    return -1;
+	  }
       }
+    }
 
     if(ergo.ghostMolecule.getNoOfAtoms() > 0 &&
        Basis_info->addBasisfuncsForMolecule(ergo.ghostMolecule, 
@@ -732,7 +742,6 @@ es_run(const char *mode, int save_pot)
   scf.step_length_giveup    = var_get_realSCF("step_length_giveup");
   scf.error_maxabs_for_diis = var_get_realSCF("error_maxabs_for_diis");
   scf.starting_guess_disturbance  = var_get_realSCF("starting_guess_disturbance");
-  //  scf.purification_create_m_files = var_get_intSCF("purification_create_m_files");
   scf.purification_subspace_err_limit = var_get_realSCF("purification_subspace_err_limit");
   scf.purification_eigvalue_err_limit = var_get_realSCF("purification_eigvalue_err_limit");
   scf.puri_eig_acc_factor_for_guess = var_get_realSCF("puri_eig_acc_factor_for_guess");
@@ -740,8 +749,6 @@ es_run(const char *mode, int save_pot)
   scf.shift_using_prev_density_matrix = var_get_realSCF("shift_using_prev_density_matrix");
   scf.electronic_temperature = var_get_realSCF("electronic_temperature");
   scf.purification_truncation_norm = mat::getNormType( var_get_stringSCF("purification_truncation_norm") );  
-  //  scf.purification_use_rand_perturbation_for_alleigsint =
-  //    var_get_intSCF("purification_use_rand_perturbation_for_alleigsint");
   scf.break_on_energy_increase    = var_get_intSCF("break_on_energy_increase");
   scf.create_basis_func_coord_file = var_get_intSCF("create_basis_func_coord_file");
   scf.output_homo_and_lumo_eigenvectors = var_get_intSCF("output_homo_and_lumo_eigenvectors");
@@ -757,14 +764,6 @@ es_run(const char *mode, int save_pot)
   scf.max_restart_count           = var_get_intSCF("max_restart_count");
   scf.min_number_of_iterations    = var_get_intSCF("min_number_of_iterations");
   scf.max_number_of_iterations    = var_get_intSCF("max_number_of_iterations");
-  //  scf.do_f_thresh_verification    = var_get_intSCF("do_f_thresh_verification");
-  //  scf.do_comparison_to_simple_purification   = var_get_intSCF("do_comparison_to_simple_purification");
-  //  scf.do_puri_mmul_tests          = var_get_intSCF("do_puri_mmul_tests");
-  //  scf.output_statistics_mfiles    = var_get_intSCF("output_statistics_mfiles");
-  //  scf.do_sparsity_investigation   = var_get_intSCF("do_sparsity_investigation");
-  //  scf.do_sparsity_investigation_reppuri   = var_get_intSCF("do_sparsity_investigation_reppuri");
-  //  scf.sparsity_plots_resolution_r = var_get_intSCF("sparsity_plots_resolution_r");
-  //  scf.sparsity_plots_resolution_m = var_get_intSCF("sparsity_plots_resolution_m");
   scf.no_of_careful_first_scf_steps     = var_get_intSCF("no_of_careful_first_scf_steps");
   scf.do_report_density_diff      = var_get_intSCF("do_report_density_diff");
   scf.no_of_impr_req_for_diis     = var_get_intSCF("no_of_impr_req_for_diis");
@@ -780,18 +779,36 @@ es_run(const char *mode, int save_pot)
   scf.output_density_images_only  = var_get_intSCF("output_density_images_only");
   scf.output_density_images_boxwidth = var_get_realSCF("output_density_images_boxwidth");
   scf.compute_gradient_fixeddens  = var_get_intSCF("compute_gradient_fixeddens");
-  //  scf.do_acc_scan_J               = var_get_intSCF("do_acc_scan_J");
-  //  scf.do_acc_scan_K               = var_get_intSCF("do_acc_scan_K");
-  //  scf.do_acc_scan_Vxc             = var_get_intSCF("do_acc_scan_Vxc");
-  //  scf.scan_no_of_steps            = var_get_intSCF("scan_no_of_steps");
-  //  scf.scan_start_thresh           = var_get_realSCF("scan_start_thresh");
-  //  scf.scan_step_factor            = var_get_realSCF("scan_step_factor");
+  scf.verify_gradient_fixeddens   = var_get_intSCF("verify_gradient_fixeddens");
+#ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
+  scf.do_f_thresh_verification    = var_get_intSCF("do_f_thresh_verification");
+  scf.do_comparison_to_simple_purification   = var_get_intSCF("do_comparison_to_simple_purification");
+  scf.do_puri_mmul_tests          = var_get_intSCF("do_puri_mmul_tests");
+  scf.output_statistics_mfiles    = var_get_intSCF("output_statistics_mfiles");
+  scf.do_sparsity_investigation   = var_get_intSCF("do_sparsity_investigation");
+  scf.do_sparsity_investigation_reppuri   = var_get_intSCF("do_sparsity_investigation_reppuri");
+  scf.sparsity_plots_resolution_r = var_get_intSCF("sparsity_plots_resolution_r");
+  scf.sparsity_plots_resolution_m = var_get_intSCF("sparsity_plots_resolution_m");
+  scf.do_acc_scan_J               = var_get_intSCF("do_acc_scan_J");
+  scf.do_acc_scan_K               = var_get_intSCF("do_acc_scan_K");
+  scf.do_acc_scan_Vxc             = var_get_intSCF("do_acc_scan_Vxc");
+  scf.scan_no_of_steps            = var_get_intSCF("scan_no_of_steps");
+  scf.scan_start_thresh           = var_get_realSCF("scan_start_thresh");
+  scf.scan_step_factor            = var_get_realSCF("scan_step_factor");
+  scf.write_guess_density_only    = var_get_intSCF("write_guess_density_only");
+  scf.compute_core_density        = var_get_intSCF("compute_core_density");
+  scf.no_of_core_electrons        = var_get_intSCF("no_of_core_electrons");
+  scf.skip_H_core                 = var_get_intSCF("skip_H_core");
+  scf.purification_create_m_files = var_get_intSCF("purification_create_m_files");
+  scf.purification_use_rand_perturbation_for_alleigsint = var_get_intSCF("purification_use_rand_perturbation_for_alleigsint");
+  scf.use_stochastic_orbs               = var_get_intSCF("use_stochastic_orbs");
+  scf.stochastic_orbs_no_of_vectors     = var_get_intSCF("stochastic_orbs_no_of_vectors");
+  scf.stochastic_orbs_use_unit_vectors  = var_get_intSCF("stochastic_orbs_use_unit_vectors");
+  scf.stochastic_orbs_rand_param        = var_get_intSCF("stochastic_orbs_rand_param");
+#endif
   scf.write_overlap_matrix        = var_get_intSCF("write_overlap_matrix");
-  //  scf.write_guess_density_only    = var_get_intSCF("write_guess_density_only");
-  //  scf.skip_H_core                 = var_get_intSCF("skip_H_core");
   scf.use_simple_dense_H_core     = var_get_intSCF("use_simple_dense_H_core");
   scf.use_diis_always             = var_get_intSCF("use_diis_always");
-
   scf.use_simple_starting_guess   = var_get_int("use_simple_starting_guess");
   scf.use_dft  = mode && strcmp(mode, "HF") != 0 
     ? (var_get_intXC("sparse_mode") ? 2 : 1) : 0;
@@ -928,7 +945,7 @@ es_run(const char *mode, int save_pot)
     ergo.extraChargesMolecule.replaceAtom(i, atom);
   }
 
-  std::auto_ptr<BasisInfoStruct> basisInfoDensFit(new BasisInfoStruct(0)); /* FIXME */
+  BasisInfoStruct basisInfoDensFit(0); /* FIXME */
 
   int noOfElectrons = ergo.molecule.getNumberOfElectrons();
   try {
@@ -939,7 +956,7 @@ es_run(const char *mode, int save_pot)
       SCF_unrestricted SCF(ergo.molecule, 
 			   ergo.extraChargesMolecule,
 			   *Basis_info, 
-			   *basisInfoDensFit, /* FIXME */
+			   basisInfoDensFit, /* FIXME */
 			   *ergoIntegralInfo,
 			   initial_density_fname,
 			   ergo.jkOptions,
@@ -1012,7 +1029,7 @@ es_run(const char *mode, int save_pot)
       SCF_restricted SCF(ergo.molecule, 
 			 ergo.extraChargesMolecule,
 			 *Basis_info, 
-			 *basisInfoDensFit, /* FIXME */
+			 basisInfoDensFit, /* FIXME */
 			 *ergoIntegralInfo,
 			 initial_density_fname,
 			 ergo.jkOptions,
@@ -1452,8 +1469,10 @@ Ergo::registerInputVariables()
 #define KL ergo.J_K_params
 #define KWJK(n,type,d) KW(ergo.J_K_params,n,type,jkOptions.n,(d))
   KWJK(use_fmm,                      VAR_INT,"Use multipole method for Coulomb matrix construction. This also enables linear scaling HF-exchange matrix construction.");
-  //  KWJK(use_differential_density,     VAR_INT,"Use \"differential density\" procedure to try to speed up Fock matrix construction.");
-  //  KWJK(use_densfit_for_J,            VAR_INT,"Use \"density fitting\" for Coulomb matrix construction.");
+#ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
+  KWJK(use_differential_density,     VAR_INT,"Use \"differential density\" procedure to try to speed up Fock matrix construction.");
+  KWJK(use_densfit_for_J,            VAR_INT,"Use \"density fitting\" for Coulomb matrix construction.");
+#endif
   KW(KL,threshold_2el_J,             VAR_FLOAT,jkOptions.threshold_J,"Threshold value for Coulomb matrix construction.");
   KW(KL,threshold_2el_K,             VAR_FLOAT,jkOptions.threshold_K,"Threshold value for HF exchange matrix construction.");
   KW(KL,threshold_1el,               VAR_FLOAT, 1e-12,"Threshold value for one-electron (core) Hamiltonian matrix construction.");
@@ -1526,19 +1545,35 @@ Ergo::registerInputVariables()
 
 #define KL ergo.scf_params
 #define KWSCF(n,type,d) KW(ergo.scf_params,n,type,scfOptions.n,d)
-  //  KWSCF(write_guess_density_only,     VAR_INT,"Only generate starting guess density, write it to file and exit.");
-  //  KWSCF(skip_H_core,                  VAR_INT,"Skip computation of 1-electron (core) Hamiltonian matrix. This gives bogus results, only useful if looking at results of initialization part or first cycle where H_core does not matter yet.");
+#ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
+  KWSCF(do_f_thresh_verification,     VAR_INT,"When truncating Fock matrix, verify that the error matrix norm is below the requested threshold.");
+  KWSCF(do_comparison_to_simple_purification,     VAR_INT,"In each density matrix construction, do a comparison to a simple purification scheme.");
+  KWSCF(do_puri_mmul_tests,           VAR_INT,"In each density matrix construction, do some extra purification test runs comparing different matrix-matrix multiplication and truncation approaches.");
+  KWSCF(output_statistics_mfiles,     VAR_INT,"Output m-file with statistics (timings etc) each SCF iteration.");
+  KWSCF(do_sparsity_investigation,    VAR_INT,"Create distance-vs-magnitude plot files.");
+  KWSCF(do_sparsity_investigation_reppuri,    VAR_INT,"Perform purification 3 times using different norms, and create corresponding distance-vs-magnitude plot files.");
+  KWSCF(sparsity_plots_resolution_r,  VAR_INT,"Distance resolution parameter for sparsity plots.");
+  KWSCF(sparsity_plots_resolution_m,  VAR_INT,"Magnitude resolution parameter for sparsity plots.");
+  KWSCF(use_stochastic_orbs,              VAR_INT,"Use stochastic orbitals as in the Stochastic DFT paper [PRL 111, 106402 (2013)].");
+  KWSCF(stochastic_orbs_no_of_vectors,    VAR_INT,"Number of stochastic orbitals to use, if use_stochastic_orbs is turned on.");
+  KWSCF(stochastic_orbs_use_unit_vectors, VAR_INT,"If use_stochastic_orbs is turned on, use unit vectors instead of random vectors.");
+  KWSCF(stochastic_orbs_rand_param,       VAR_INT,"If use_stochastic_orbs is turned on, this value affects the way random numbers are generated. Setting this parameter to 2 means that a more elaborate, hopefully better, method is used to generate random numbers.");
+  KWSCF(write_guess_density_only,     VAR_INT,"Only generate starting guess density, write it to file and exit.");
+  KWSCF(compute_core_density,     VAR_INT,"Each time the density matrix is computed, also compute a 'core density matrix' using only the core electrons.");
+  KWSCF(no_of_core_electrons,     VAR_INT,"If compute_core_density is set, use this number of core electrons when coputing the 'core density matrix'.");
+  KWSCF(skip_H_core,                  VAR_INT,"Skip computation of 1-electron (core) Hamiltonian matrix. This gives bogus results, only useful if looking at results of initialization part or first cycle where H_core does not matter yet.");
+  KWSCF(do_acc_scan_J,                VAR_INT,"Perform \"accuracy scan\" for Coulomb matrix.");
+  KWSCF(do_acc_scan_K,                VAR_INT,"Perform \"accuracy scan\" for HF exchange matrix.");
+  KWSCF(do_acc_scan_Vxc,              VAR_INT,"Perform \"accuracy scan\" for DFT exchange-correlation matrix.");
+  KWSCF(scan_no_of_steps,             VAR_INT,"Number of steps to use in \"accuracy scans\" of J, K, Vxc matrices.");
+  KWSCF(scan_start_thresh,            VAR_FLOAT,"Scan start threshold value to use for \"accuracy scans\" of J, K, Vxc matrices.");
+  KWSCF(scan_step_factor,             VAR_FLOAT,"Step factor to use for \"accuracy scans\" of J, K, Vxc matrices.");
+  KWSCF(purification_create_m_files,  VAR_INT,"Create m files with information about purification."),
+  KWSCF(purification_use_rand_perturbation_for_alleigsint, VAR_INT,"Use random perturbation to attempt getting easier Lanczos convergence when determining min/max eigenvalues of Fock matrix before purification."),
+#endif
   KWSCF(use_simple_dense_H_core,      VAR_INT,"Use simple dense matrix computation of 1-electron (core) Hamiltonian matrix.");
   KWSCF(use_diis_always,              VAR_INT,"Always use DIIS even if the energy goes up.");
   KWSCF(write_overlap_matrix,         VAR_INT,"Write overlap matrix to file.");
-  //  KWSCF(do_acc_scan_J,                VAR_INT,"Perform \"accuracy scan\" for Coulomb matrix.");
-  //  KWSCF(do_acc_scan_K,                VAR_INT,"Perform \"accuracy scan\" for HF exchange matrix.");
-  //  KWSCF(do_acc_scan_Vxc,              VAR_INT,"Perform \"accuracy scan\" for DFT exchange-correlation matrix.");
-  //  KWSCF(scan_no_of_steps,             VAR_INT,"Number of steps to use in \"accuracy scans\" of J, K, Vxc matrices.");
-  //  KWSCF(scan_start_thresh,            VAR_FLOAT,"Scan start threshold value to use for \"accuracy scans\" of J, K, Vxc matrices.");
-  //  KWSCF(scan_step_factor,             VAR_FLOAT,"Step factor to use for \"accuracy scans\" of J, K, Vxc matrices.");
-  //  KWSCF(purification_create_m_files,  VAR_INT,"Create m files with information about purification."),
-  //  KWSCF(purification_use_rand_perturbation_for_alleigsint, VAR_INT,"Use random perturbation to attempt getting easier Lanczos convergence when determining min/max eigenvalues of Fock matrix before purification."),
   KWSCF(use_diagonalization,          VAR_INT,"Use diagonalization instead of purification.");
   KWSCF(use_diag_on_error,            VAR_INT,"Use diagonalization if purification fails.");
   KWSCF(use_diag_on_error_guess,      VAR_INT,"Use diagonalization if purification fails during starting guess projection.");
@@ -1546,7 +1581,8 @@ Ergo::registerInputVariables()
   KWSCF(output_density_images,        VAR_INT,"Output density image files (including Gabedit gcube density files) after SCF finished.");
   KWSCF(output_density_images_only,   VAR_INT,"Output density image files (including Gabedit gcube density files) directly, without any SCF procedure, using the given starting guess density.");
   KWSCF(output_density_images_boxwidth, VAR_FLOAT, "Box width to use for density image files (including Gabedit gcube density files).");
-  KWSCF(compute_gradient_fixeddens,   VAR_INT,"Compute gradient of energy with respect to nuclear positions for fixed electron density.");
+  KWSCF(compute_gradient_fixeddens,   VAR_INT,"Compute gradient of energy with respect to nuclear positions for fixed electron density (Hellmann-Feynman forces).");
+  KWSCF(verify_gradient_fixeddens,    VAR_INT,"Verify gradient of energy with respect to nuclear positions (Hellmann-Feynman forces), using finite differences. Used only when compute_gradient_fixeddens is set.");
   KWSCF(step_length_start,            VAR_FLOAT,"Start value of SCF step length parameter.");
   KWSCF(step_length_giveup,           VAR_FLOAT,"When step length in SCF procedure becomes smaller than this value, give up.");
   KWSCF(starting_guess_disturbance,   VAR_FLOAT,"Magnitude of random disturbance to apply to starting guess density matrix.");
@@ -1557,14 +1593,6 @@ Ergo::registerInputVariables()
   KWSCF(min_number_of_iterations,     VAR_INT,"Minimum number of SCF iterations.");
   KWSCF(max_restart_count,            VAR_INT,"Max number of times to attempt to restart SCF procedure when it appears stuck.");
   KWSCF(max_number_of_iterations,     VAR_INT,"Maximum number of SCF iterations.");
-  //  KWSCF(do_f_thresh_verification,     VAR_INT,"When truncating Fock matrix, verify that the error matrix norm is below the requested threshold.");
-  //  KWSCF(do_comparison_to_simple_purification,     VAR_INT,"In each density matrix construction, do a comparison to a simple purification scheme.");
-  //  KWSCF(do_puri_mmul_tests,           VAR_INT,"In each density matrix construction, do some extra purification test runs comparing different matrix-matrix multiplication and truncation approaches.");
-  //  KWSCF(output_statistics_mfiles,     VAR_INT,"Output m-file with statistics (timings etc) each SCF iteration.");
-  //  KWSCF(do_sparsity_investigation,    VAR_INT,"Create distance-vs-magnitude plot files.");
-  //  KWSCF(do_sparsity_investigation_reppuri,    VAR_INT,"Perform purification 3 times using different norms, and create corresponding distance-vs-magnitude plot files.");
-  //  KWSCF(sparsity_plots_resolution_r,  VAR_INT,"Distance resolution parameter for sparsity plots.");
-  //  KWSCF(sparsity_plots_resolution_m,  VAR_INT,"Magnitude resolution parameter for sparsity plots.");
   KWSCF(no_of_careful_first_scf_steps,VAR_INT,"Number of \"careful\" steps in beginning of SCF procedure. May be useful if starting guess is very bad.");
   KWSCF(do_report_density_diff,       VAR_INT,"In each SCF cycle, output norm of difference between current and previous density matrix.");
   KWSCF(max_no_of_diis_matrices,      VAR_INT,"Max number of DIIS matrices to use. Lower value may be useful to reduce disk/memory usage.");
