@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ static const double GE = 2.0023193044;
 #include "memorymanag.h"
 #include "basisinfo.h"
 #include "aos.h"
-#include "density_description_file_2.h"
+#include "density_description_file.h"
 #include "Matrix.h"
 #include "MatrixSymmetric.h"
 
@@ -123,11 +123,11 @@ int main(int argc, char *argv[])
     return 1;
   }
 
-  std::auto_ptr<IntegralInfo> integralInfo(new IntegralInfo(true));
+  IntegralInfo integralInfo(true);
   
   ergo_real       *densMatrix[2] = { NULL, NULL };
   BasisInfoStruct *basisRead  = NULL;
-  if (ddf_load_density(argv[1], 2, *integralInfo,
+  if (ddf_load_density(argv[1], 2, integralInfo,
                        &basisRead, densMatrix)) {
     fprintf(stderr, "Loading unrestricted densities from '%s' failed. "
             "Calculation aborted.\n", argv[1]);
