@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,6 +47,7 @@ void hermite_conversion_info_struct::init(const monomial_info_struct & monomial_
 
   // Allocate work memory buffers.
   std::vector<hermite_conversion_contrib_struct> currlist(MAX_NO_OF_CONTRIBS);
+  std::vector<hermite_conversion_element_struct> currlist_elements(MAX_NO_OF_CONTRIBS);
   const int nMonMax = monomial_info.no_of_monomials_list[nmax];
   std::vector<symb_matrix_element> list(nMonMax*nMonMax);
   const int invFlag = 1;
@@ -109,6 +110,54 @@ void hermite_conversion_info_struct::init(const monomial_info_struct & monomial_
     }
   }
 
+  // Get list_right_simple
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    get_hermite_conversion_matrix_symb(&monomial_info, n1, invFlag, &list[0]);
+    int nMon1 = monomial_info.no_of_monomials_list[n1];
+    int count = 0;
+    for(int j = 0; j < nMon1; j++) {
+      for(int k = 0; k < nMon1; k++) {
+	int idx = j*nMon1+k;
+	if(std::fabs(list[idx].coeff) > 1e-5) {
+	  assert(count < MAX_NO_OF_CONTRIBS);
+	  currlist_elements[count].idx_j = j;
+	  currlist_elements[count].idx_k = k;
+	  currlist_elements[count].a_power = list[idx].ia;
+	  currlist_elements[count].dummy = 0;
+	  currlist_elements[count].coeff = list[idx].coeff;
+	  count++;
+	}
+      }
+    }
+    list_right_simple[n1] = new hermite_conversion_element_struct[count];
+    memcpy(list_right_simple[n1], &currlist_elements[0], count*sizeof(hermite_conversion_element_struct));
+    counters_right_simple[n1] = count;
+  }
+
+  // Get list_left_simple
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    get_hermite_conversion_matrix_symb(&monomial_info, n1, invFlag, &list[0]);
+    int nMon1 = monomial_info.no_of_monomials_list[n1];
+    int count = 0;
+    for(int j = 0; j < nMon1; j++) {
+      for(int k = 0; k < nMon1; k++) {
+	int idx = k*nMon1+j; // reverse meaning of j k here compared to "right" case
+	if(std::fabs(list[idx].coeff) > 1e-5) {
+	  assert(count < MAX_NO_OF_CONTRIBS);
+	  currlist_elements[count].idx_j = j;
+	  currlist_elements[count].idx_k = k;
+	  currlist_elements[count].a_power = list[idx].ia;
+	  currlist_elements[count].dummy = 0;
+	  currlist_elements[count].coeff = list[idx].coeff;
+	  count++;
+	}
+      }
+    }
+    list_left_simple[n1] = new hermite_conversion_element_struct[count];
+    memcpy(list_left_simple[n1], &currlist_elements[0], count*sizeof(hermite_conversion_element_struct));
+    counters_left_simple[n1] = count;
+  }
+
   timeMeter.print(LOG_AREA_INTEGRALS, 
 		  "hermite_conversion_info_struct constructor");
 } // end hermite_conversion_info_struct constructor
@@ -120,6 +169,10 @@ hermite_conversion_info_struct::hermite_conversion_info_struct() {
       list_right[n1][n2] = NULL;
       list_left [n1][n2] = NULL;
     }
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    list_right_simple[n1] = NULL;
+    list_left_simple [n1] = NULL;
+  }
 }
 
 hermite_conversion_info_struct::~hermite_conversion_info_struct() {
@@ -129,6 +182,10 @@ hermite_conversion_info_struct::~hermite_conversion_info_struct() {
       delete []list_right[n1][n2];
       delete []list_left [n1][n2];
     }
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    delete [] list_right_simple[n1];
+    delete [] list_left_simple [n1];
+  }
 }
 
 
@@ -190,6 +247,75 @@ int hermite_conversion_info_struct::multiply_by_hermite_conversion_matrix_from_l
 }
 
 
+int hermite_conversion_info_struct::get_hermite_conversion_matrix_right(const monomial_info_struct & monomial_info,
+									int nmax,
+									ergo_real a,
+									ergo_real* result) const {
+  int noOfContribs = counters_right_simple[nmax];
+  hermite_conversion_element_struct* list = list_right_simple[nmax];  
+  int nMon1 = monomial_info.no_of_monomials_list[nmax];
+  int Ntot = 2 * nmax;
+  ergo_real apowlist[Ntot+1];
+  apowlist[0] = 1;
+  for(int i = 1; i <= Ntot; i++)
+    apowlist[i] = apowlist[i-1] * a;  
+  for(int i = 0; i < nMon1*nMon1; i++)
+    result[i] = 0;
+  for(int i = 0; i < noOfContribs; i++) {
+    int j = list[i].idx_j;
+    int k = list[i].idx_k;
+    result[j*nMon1+k] = list[i].coeff * apowlist[-list[i].a_power];
+  }  
+  return 0;
+}
+
+int hermite_conversion_info_struct::get_hermite_conversion_matrix_right_sparse(const monomial_info_struct & monomial_info,
+									       int nmax,
+									       ergo_real a,
+									       i_j_val_struct* result) const {
+  int noOfContribs = counters_right_simple[nmax];
+  hermite_conversion_element_struct* list = list_right_simple[nmax];  
+  int Ntot = 2 * nmax;
+  ergo_real apowlist[Ntot+1];
+  apowlist[0] = 1;
+  for(int i = 1; i <= Ntot; i++)
+    apowlist[i] = apowlist[i-1] * a;
+  int count = 0;
+  for(int i = 0; i < noOfContribs; i++) {
+    int j = list[i].idx_j;
+    int k = list[i].idx_k;
+    result[count].i = j;
+    result[count].j = k;
+    result[count].same_i_count = 1;
+    result[count].value = list[i].coeff * apowlist[-list[i].a_power];
+    count++;
+  }
+  return count;
+}
+
+int hermite_conversion_info_struct::get_hermite_conversion_matrix_left(const monomial_info_struct & monomial_info,
+								       int nmax,
+								       ergo_real a,
+								       ergo_real* result) const {
+  int noOfContribs = counters_left_simple[nmax];
+  hermite_conversion_element_struct* list = list_left_simple[nmax];  
+  int nMon1 = monomial_info.no_of_monomials_list[nmax];
+  int Ntot = 2 * nmax;
+  ergo_real apowlist[Ntot+1];
+  apowlist[0] = 1;
+  for(int i = 1; i <= Ntot; i++)
+    apowlist[i] = apowlist[i-1] * a;  
+  for(int i = 0; i < nMon1*nMon1; i++)
+    result[i] = 0;
+  for(int i = 0; i < noOfContribs; i++) {
+    int j = list[i].idx_j;
+    int k = list[i].idx_k;
+    result[j*nMon1+k] = list[i].coeff * apowlist[-list[i].a_power];
+  }  
+  return 0;
+}
+
+
 /** Function needed for Chunks&Tasks usage. */
 hermite_conversion_info_struct::hermite_conversion_info_struct(const hermite_conversion_info_struct & other) {
   memcpy(counters_right, other.counters_right, sizeof(counters_right));
