@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "realtype.h"
 #include "polydegree.h"
 #include "monomial_info.h"
+#include "simple_sparse_mat.h"
 
 struct hermite_conversion_contrib_struct
 {
@@ -44,6 +45,16 @@ struct hermite_conversion_contrib_struct
 
 typedef hermite_conversion_contrib_struct * hermite_conversion_contrib_struct_ptr;
 
+struct hermite_conversion_element_struct {
+  int idx_j;
+  int idx_k;
+  int a_power;
+  int dummy;
+  ergo_real coeff;
+};
+
+typedef hermite_conversion_element_struct * hermite_conversion_element_struct_ptr;
+
 const int HERMITE_CONVERSION_MAX_N = BASIS_FUNC_POLY_MAX_DEGREE*2;
 
 class hermite_conversion_info_struct {
@@ -52,6 +63,11 @@ class hermite_conversion_info_struct {
   hermite_conversion_contrib_struct_ptr list_left [HERMITE_CONVERSION_MAX_N+1][HERMITE_CONVERSION_MAX_N+1];
   int counters_right[HERMITE_CONVERSION_MAX_N+1][HERMITE_CONVERSION_MAX_N+1];
   int counters_left [HERMITE_CONVERSION_MAX_N+1][HERMITE_CONVERSION_MAX_N+1];
+  // Simple lists used to represent conversion matrices
+  hermite_conversion_element_struct_ptr list_right_simple[HERMITE_CONVERSION_MAX_N+1];
+  hermite_conversion_element_struct_ptr list_left_simple [HERMITE_CONVERSION_MAX_N+1];
+  int counters_right_simple[HERMITE_CONVERSION_MAX_N+1];
+  int counters_left_simple [HERMITE_CONVERSION_MAX_N+1];
  public:
   void init(const monomial_info_struct & monomial_info);
   hermite_conversion_info_struct();
@@ -68,6 +84,20 @@ class hermite_conversion_info_struct {
 						      ergo_real a,      
 						      ergo_real* A,     
 						      ergo_real* result) const;
+  int get_hermite_conversion_matrix_right(const monomial_info_struct & monomial_info,
+					  int nmax,
+					  ergo_real a,
+					  ergo_real* result) const;
+  int get_hermite_conversion_matrix_left(const monomial_info_struct & monomial_info,
+					 int nmax,
+					 ergo_real a,
+					 ergo_real* result) const;
+
+  int get_hermite_conversion_matrix_right_sparse(const monomial_info_struct & monomial_info,
+						 int nmax,
+						 ergo_real a,
+						 i_j_val_struct* result) const;
+
   // Stuff needed for Chunks&Tasks usage
   hermite_conversion_info_struct(const hermite_conversion_info_struct & other);
   void write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const;
