@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -257,6 +257,24 @@ int IntegralInfo::multiply_by_hermite_conversion_matrix_from_left(int n1max,
   return hermite_conversion_info.multiply_by_hermite_conversion_matrix_from_left(monomial_info, n1max, n2max, a, A, result);
 }
 
+int IntegralInfo::get_hermite_conversion_matrix_right(int nmax,
+						      ergo_real a,
+						      ergo_real* result) const {
+  return hermite_conversion_info.get_hermite_conversion_matrix_right(monomial_info, nmax, a, result);
+}
+
+int IntegralInfo::get_hermite_conversion_matrix_left(int nmax,
+						     ergo_real a,
+						     ergo_real* result) const {
+  return hermite_conversion_info.get_hermite_conversion_matrix_left(monomial_info, nmax, a, result);
+}
+
+int IntegralInfo::get_hermite_conversion_matrix_right_sparse(int nmax,
+							     ergo_real a,
+							     i_j_val_struct* result) const {
+  return hermite_conversion_info.get_hermite_conversion_matrix_right_sparse(monomial_info, nmax, a, result);  
+}
+
 void IntegralInfo::init() {
   if(initialized)
     return;
