@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -101,6 +101,16 @@ class IntegralInfo
 						      ergo_real a,      
 						      ergo_real* A,     
 						      ergo_real* result) const;
+  int get_hermite_conversion_matrix_right(int nmax,
+					  ergo_real a,
+					  ergo_real* result) const;
+  int get_hermite_conversion_matrix_left(int nmax,
+					 ergo_real a,
+					 ergo_real* result) const;
+  int get_hermite_conversion_matrix_right_sparse(int nmax,
+                                                 ergo_real a,
+                                                 i_j_val_struct* result) const;
+
   IntegralInfo(bool initialize);
   ~IntegralInfo();
 
