@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,7 +81,7 @@ do_derivative_of_simple_prim(const DistributionSpecStruct& prim,
    prim2. Note that this function is *not* strict wrt the
    effectiveThreshold parameter, the approximation is only
    proportional to its value but it can exceed it. */
-static ergo_real 
+ergo_real 
 simplePrimTintegral(const DistributionSpecStruct& prim1,
 		    const DistributionSpecStruct& prim2,
 		    int coord,
