@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,10 @@
 #include "basisinfo.h"
 
 
+ergo_real simplePrimTintegral(const DistributionSpecStruct& prim1,
+			      const DistributionSpecStruct& prim2,
+			      int coord,
+			      ergo_real threshold);
 
 int compute_T_matrix_sparse(const BasisInfoStruct& basisInfo,
 			    ergo_real threshold,
