@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,28 @@
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
-/* Written by Elias Rudberg, KTH, Stockholm */
+/* Written by Elias Rudberg. First version was written in 2006 or even
+   earlier, at KTH. Then the code has been updated many times after
+   that.  */
+
+/* 
+   ELIAS NOTE 2014-05-31: gradient computation has been added now. The
+   code for computing the gradient uses that the nuclear coordinates
+   enter the coputation of the V matrix in two ways: 
+
+   (1) Directly in integrals of the type (ij|A) where ij is a basis
+   function pair and A is an atom index. In this case we can get the
+   derivatives directly using get_related_integrals_hermite().
+
+   (2) Via the multipole tree. For this kind of contribution we can
+   compute the derivatives indirectly, by first computing derivatives
+   of th emultipole moments w.r.t. nuclear coordinates, and
+   derivatives of the energy w.r.t. the multipole moments. Then, we
+   multiply those values to get the final derivatives of the energy
+   w.r.t. nuclear coordinates. I.e., we compute the derivative df/dx
+   using the chain rule, df/dx = (df/dy)*(dy/dx).
+*/
+
 #include <stdlib.h>
 #include <math.h>
 #include <stdio.h>
@@ -64,6 +85,8 @@ typedef struct
   box_struct_basic basicBox;
   ergo_real centerOfChargeCoords[3];
   multipole_struct_large multipole;
+  ergo_real* multipole_moment_derivatives; // If needed, list of derivatives of multipole moments w.r.t. atom coordinates for all atoms in this box.
+  ergo_real* derivatives_wrt_multipole_moments;  // If needed, list of derivatives of the energy w.r.t. the multipole moments for this box.
 } atom_box_struct;
 
 
@@ -80,13 +103,57 @@ get_distance_3d(const ergo_real* x, const ergo_real* y)
 }
 
 
+static void get_multipole_contribs_for_atom(multipole_struct_large & boxMultipole, ergo_real* multipolePointCoords, const Atom & currAtom, const MMTranslator & translator) {
+  multipole_struct_small multipoleCurrAtom;
+  memset(&multipoleCurrAtom, 0, sizeof(multipole_struct_small));
+  multipoleCurrAtom.degree = 0;
+  multipoleCurrAtom.noOfMoments = 1;
+  multipoleCurrAtom.momentList[0] = currAtom.charge;
+  ergo_real dx = currAtom.coords[0] - multipolePointCoords[0];
+  ergo_real dy = currAtom.coords[1] - multipolePointCoords[1];
+  ergo_real dz = currAtom.coords[2] - multipolePointCoords[2];
+
+  ergo_real W[MAX_NO_OF_MOMENTS_PER_MULTIPOLE*MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
+  translator.getTranslationMatrix(dx, dy, dz, MAX_MULTIPOLE_DEGREE, 0, W);
+		  
+  multipole_struct_large translatedMultipole;
+  for(int A = 0; A < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; A++)
+    {
+      ergo_real sum = 0;
+      for(int B = 0; B < 1; B++)
+	sum += W[A*1+B] * multipoleCurrAtom.momentList[B];
+      translatedMultipole.momentList[A] = sum;
+    } // END FOR A
+  for(int kk = 0; kk < 3; kk++)
+    translatedMultipole.centerCoords[kk] = multipolePointCoords[kk];
+  translatedMultipole.degree = MAX_MULTIPOLE_DEGREE;
+  translatedMultipole.noOfMoments = MAX_NO_OF_MOMENTS_PER_MULTIPOLE;
+	  
+  // add translated multipole to box multipole
+  for(int A = 0; A < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; A++)
+    boxMultipole.momentList[A] += translatedMultipole.momentList[A];  
+}
+
+
+static void init_multipole_struct_large(multipole_struct_large & boxMultipole, const ergo_real* multipolePointCoords) {
+  memset(&boxMultipole, 0, sizeof(multipole_struct_large));
+  for(int kk = 0; kk < 3; kk++)
+    boxMultipole.centerCoords[kk] = multipolePointCoords[kk];
+  boxMultipole.degree = MAX_MULTIPOLE_DEGREE;
+  boxMultipole.noOfMoments = MAX_NO_OF_MOMENTS_PER_MULTIPOLE;
+}
+
+
 static int
 create_nuclei_mm_tree(int nAtoms,
 		      const Atom* atomList,
 		      ergo_real boxSize,
+		      BoxSystem & boxSystem,
 		      atom_box_struct** return_boxList, // allocated by this routine, must be freed by caller.
 		      int *return_numberOfLevels,
-		      Atom **return_atomListReordered // allocated by this routine, must be freed by caller.
+		      Atom **return_atomListReordered, // allocated by this routine, must be freed by caller.
+		      int* return_atomPermutation, // list of int, length=nAtoms, saying how atoms have been reordered in return_atomListReordered.
+		      bool compute_gradient_also
 		      )
 {
   // Create box system based on atoms
@@ -99,8 +166,6 @@ create_nuclei_mm_tree(int nAtoms,
     } // END FOR i
   
   const ergo_real maxToplevelBoxSize = boxSize;
-
-  BoxSystem boxSystem;
   
   if(boxSystem.create_box_system(itemList,
 				 nAtoms,
@@ -110,6 +175,10 @@ create_nuclei_mm_tree(int nAtoms,
       return -1;
     }
 
+  // Now the itemList has been reordered. Store the ordering info in the return_atomPermutation list.
+  for(int i = 0; i < nAtoms; i++)
+    return_atomPermutation[i] = itemList[i].originalIndex;
+
   int numberOfLevels = boxSystem.noOfLevels;
   
   atom_box_struct* boxList = new atom_box_struct[boxSystem.totNoOfBoxes];
@@ -163,6 +232,14 @@ create_nuclei_mm_tree(int nAtoms,
 	for(int k = 0; k < 3; k++)
 	  currBox->centerOfChargeCoords[k] = currBox->basicBox.centerCoords[k];	
       }
+
+      if(compute_gradient_also) {
+	// Always use center point in this case, to simplify computation of derivatives.
+	// FIXME: CHECK IF THIS IS REALLY NEEDED, MAYBE IT WAS REALLY BUGS IN OTHER PLACES THAT CAUSED THE PROBLEMS?
+	for(int k = 0; k < 3; k++)
+	  currBox->centerOfChargeCoords[k] = currBox->basicBox.centerCoords[k];
+      }
+
     } // END FOR i Find center-of-charge for each box at top level
 
   // Create multipole for each box at top level (smallest boxes)
@@ -171,50 +248,43 @@ create_nuclei_mm_tree(int nAtoms,
     {
       atom_box_struct* currBox = &boxListTopLevel[i];
       ergo_real* multipolePointCoords = currBox->centerOfChargeCoords;
-      multipole_struct_large boxMultipole;
-      memset(&boxMultipole, 0, sizeof(multipole_struct_large));
-      for(int kk = 0; kk < 3; kk++)
-	boxMultipole.centerCoords[kk] = multipolePointCoords[kk];
-      boxMultipole.degree = MAX_MULTIPOLE_DEGREE;
-      boxMultipole.noOfMoments = MAX_NO_OF_MOMENTS_PER_MULTIPOLE;
-
-      Atom* atomListCurrBox = &atomList2[currBox->basicBox.firstItemIndex];
       int noOfAtomsCurrBox = currBox->basicBox.noOfItems;
+      if(compute_gradient_also) {
+	currBox->multipole_moment_derivatives = new ergo_real[noOfAtomsCurrBox*MAX_NO_OF_MOMENTS_PER_MULTIPOLE*3];
+	currBox->derivatives_wrt_multipole_moments = new ergo_real[MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
+	memset(currBox->derivatives_wrt_multipole_moments, 0, MAX_NO_OF_MOMENTS_PER_MULTIPOLE*sizeof(ergo_real));
+      }
+      multipole_struct_large boxMultipole;
+      init_multipole_struct_large(boxMultipole, multipolePointCoords);
+      Atom* atomListCurrBox = &atomList2[currBox->basicBox.firstItemIndex];
       // Go through all atoms in this box
       for(int j = 0; j < noOfAtomsCurrBox; j++)
 	{
-	  Atom* currAtom = &atomListCurrBox[j];
-      
-	  // take multipole for this atom, and translate it to center-of-charge point
+	  const Atom & currAtom = atomListCurrBox[j];
+      	  // take multipole for this atom, and translate it to center-of-charge point
 	  // the "multipole" for this atom is of course only a monopole.
-	  multipole_struct_small multipoleCurrAtom;
-	  memset(&multipoleCurrAtom, 0, sizeof(multipole_struct_small));
-	  multipoleCurrAtom.degree = 0;
-	  multipoleCurrAtom.noOfMoments = 1;
-	  multipoleCurrAtom.momentList[0] = currAtom->charge;
-	  ergo_real dx = currAtom->coords[0] - multipolePointCoords[0];
-	  ergo_real dy = currAtom->coords[1] - multipolePointCoords[1];
-	  ergo_real dz = currAtom->coords[2] - multipolePointCoords[2];
-
-	  ergo_real W[MAX_NO_OF_MOMENTS_PER_MULTIPOLE*MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
-	  translator.getTranslationMatrix(dx, dy, dz, MAX_MULTIPOLE_DEGREE, 0, W);
-		  
-	  multipole_struct_large translatedMultipole;
-	  for(int A = 0; A < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; A++)
-	    {
-	      ergo_real sum = 0;
-	      for(int B = 0; B < 1; B++)
-		sum += W[A*1+B] * multipoleCurrAtom.momentList[B];
-	      translatedMultipole.momentList[A] = sum;
-	    } // END FOR A
-	  for(int kk = 0; kk < 3; kk++)
-	    translatedMultipole.centerCoords[kk] = multipolePointCoords[kk];
-	  translatedMultipole.degree = MAX_MULTIPOLE_DEGREE;
-	  translatedMultipole.noOfMoments = MAX_NO_OF_MOMENTS_PER_MULTIPOLE;
-	  
-	  // add translated multipole to box multipole
-	  for(int A = 0; A < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; A++)
-	    boxMultipole.momentList[A] += translatedMultipole.momentList[A];
+	  get_multipole_contribs_for_atom(boxMultipole, multipolePointCoords, currAtom, translator);
+	  if(compute_gradient_also) {
+	    for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
+	      multipole_struct_large boxMultipoleTmp1;
+	      init_multipole_struct_large(boxMultipoleTmp1, multipolePointCoords);
+	      multipole_struct_large boxMultipoleTmp2;
+	      init_multipole_struct_large(boxMultipoleTmp2, multipolePointCoords);
+	      const ergo_real eps = 1e-5;
+	      Atom atomTmp1 = currAtom;
+	      atomTmp1.coords[coordIdx] += eps;
+	      Atom atomTmp2 = currAtom;
+	      atomTmp2.coords[coordIdx] -= eps;
+	      get_multipole_contribs_for_atom(boxMultipoleTmp1, multipolePointCoords, atomTmp1, translator);
+	      get_multipole_contribs_for_atom(boxMultipoleTmp2, multipolePointCoords, atomTmp2, translator);
+	      for(int ii = 0; ii < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; ii++) {
+		ergo_real m1 = boxMultipoleTmp1.momentList[ii];
+		ergo_real m2 = boxMultipoleTmp2.momentList[ii];
+		ergo_real value = (m1 - m2) / (2*eps);
+		currBox->multipole_moment_derivatives[j*MAX_NO_OF_MOMENTS_PER_MULTIPOLE*3 + MAX_NO_OF_MOMENTS_PER_MULTIPOLE*coordIdx + ii] = value;
+	      }
+	    }
+	  }
 	} // END FOR j Go through all atoms in this box
 
       currBox->multipole = boxMultipole;
@@ -259,6 +329,13 @@ create_nuclei_mm_tree(int nAtoms,
 	    } // END FOR childIndex	  
 	  for(kk = 0; kk < 3; kk++)
 	    newMultipole->centerCoords[kk] = avgPosList[kk] / noOfChildren;
+
+	  if(compute_gradient_also) {
+	    // Always use center point in this case, to simplify computation of derivatives.
+	    // FIXME: CHECK IF THIS IS REALLY NEEDED, MAYBE IT WAS REALLY BUGS IN OTHER PLACES THAT CAUSED THE PROBLEMS?
+	    for(kk = 0; kk < 3; kk++)
+	      newMultipole->centerCoords[kk] = currBox->basicBox.centerCoords[kk];
+	  }
 	  
 	  newMultipole->degree = MAX_MULTIPOLE_DEGREE;
 	  newMultipole->noOfMoments = MAX_NO_OF_MOMENTS_PER_MULTIPOLE;
@@ -296,6 +373,44 @@ create_nuclei_mm_tree(int nAtoms,
 		newMultipole->momentList[A] += translatedMultipole.momentList[A];
 	    } // END FOR childIndex
 
+	  if(compute_gradient_also) {
+	    // Get multipole for large box by going through all atoms instead of translating child multipoles
+	    ergo_real* multipolePointCoords = newMultipole->centerCoords;
+	    multipole_struct_large boxMultipole;
+	    init_multipole_struct_large(boxMultipole, multipolePointCoords);
+	    Atom* atomListCurrBox = &atomList2[currBox->basicBox.firstItemIndex];
+	    int noOfAtomsCurrBox = currBox->basicBox.noOfItems;
+	    currBox->multipole_moment_derivatives = new ergo_real[noOfAtomsCurrBox*MAX_NO_OF_MOMENTS_PER_MULTIPOLE*3];
+	    currBox->derivatives_wrt_multipole_moments = new ergo_real[MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
+	    memset(currBox->derivatives_wrt_multipole_moments, 0, MAX_NO_OF_MOMENTS_PER_MULTIPOLE*sizeof(ergo_real));
+	    // Go through all atoms in this box
+	    for(int j = 0; j < noOfAtomsCurrBox; j++) {
+	      const Atom & currAtom = atomListCurrBox[j];
+	      // take multipole for this atom, and translate it to center-of-charge point
+	      // the "multipole" for this atom is of course only a monopole.
+	      get_multipole_contribs_for_atom(boxMultipole, multipolePointCoords, currAtom, translator);
+	      for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
+		multipole_struct_large boxMultipoleTmp1;
+		init_multipole_struct_large(boxMultipoleTmp1, multipolePointCoords);
+		multipole_struct_large boxMultipoleTmp2;
+		init_multipole_struct_large(boxMultipoleTmp2, multipolePointCoords);
+		const ergo_real eps = 1e-5;
+		Atom atomTmp1 = currAtom;
+		atomTmp1.coords[coordIdx] += eps;
+		Atom atomTmp2 = currAtom;
+		atomTmp2.coords[coordIdx] -= eps;
+		get_multipole_contribs_for_atom(boxMultipoleTmp1, multipolePointCoords, atomTmp1, translator);
+		get_multipole_contribs_for_atom(boxMultipoleTmp2, multipolePointCoords, atomTmp2, translator);
+		for(int ii = 0; ii < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; ii++) {
+		  ergo_real m1 = boxMultipoleTmp1.momentList[ii];
+		  ergo_real m2 = boxMultipoleTmp2.momentList[ii];
+		  ergo_real value = (m1 - m2) / (2*eps);
+		  currBox->multipole_moment_derivatives[j*MAX_NO_OF_MOMENTS_PER_MULTIPOLE*3 + MAX_NO_OF_MOMENTS_PER_MULTIPOLE*coordIdx + ii] = value;
+		}
+	      }
+	    } // END FOR j Go through all atoms in this box
+	  }
+
 	} // END FOR boxIndex
     } // END FOR levelNumber  
 
@@ -321,9 +436,6 @@ create_nuclei_mm_tree(int nAtoms,
 
 
 
-
-
-
 typedef struct
 {
   DistributionSpecStruct distr;
@@ -338,6 +450,7 @@ static int
 do_interaction_recursive(const IntegralInfo & integralInfo,
 			 ergo_real* V_list,
 			 int noOfBasisFuncIndexPairs,
+			 const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
 			 const DistributionSpecStructWithPairIdx* list,
 			 int nDistrs,
 			 const multipole_struct_small* multipoleList,
@@ -346,12 +459,17 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 			 int maxDegreeForDistrs,
 			 ergo_real distrExtent,
 			 const Atom *atomListReordered,
+			 const int* atomPermutation, // list of int, length=nAtoms, saying how atoms have been reordered in return_atomListReordered.
 			 ergo_real threshold,
 			 const atom_box_struct* boxList,
 			 MMInteractor & interactor,
 			 int boxIndex,
 			 int currLevel,
-			 int numberOfLevels)
+			 int numberOfLevels,
+			 bool compute_gradient_also,
+			 const ergo_real* D_list,         // used in compute_gradient_also case, NULL otherwise
+			 ergo_real* result_gradient_list  // used in compute_gradient_also case, NULL otherwise
+			 )
 {
   const atom_box_struct* currBox = &boxList[boxIndex];
   const multipole_struct_large* boxMultipole = &currBox->multipole;
@@ -371,6 +489,7 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 								   threshold);
   if(degreeNeeded < 0)
     return -1;
+  degreeNeeded+=2; // We need a couple of extra degrees to handle gradient computation. FIXME: IS THIS REALL NEEDED? WHY? MAYBE IT WAS BUGS IN OTHER PLACES THAT CAUSED THE PROBLEMS?
 
   bool multipoleDegreeIsSafe = false;
   // Demand at least two degrees margin compared to
@@ -384,7 +503,6 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
     {
       // OK, use multipole description of atom charges.
       int boxNeededNoOfMoments = (degreeNeeded+1)*(degreeNeeded+1);
-      
       // create interaction matrix
       ergo_real T[boxNeededNoOfMoments * maxNoOfMomentsForDistrs];
       ergo_real dx = boxMultipole->centerCoords[0] - list[0].distr.centerCoords[0];
@@ -413,6 +531,42 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 	    sum += tempVector[A] * multipoleList[i].momentList[A];
 	  V_list[list[i].pairIdx] += -1 * sum;
 	}
+
+      if(compute_gradient_also) {
+	// we need to compute the derivatives of the energy with respect to the multipole moments.
+	// To reduce number of critical sections when OpenMP threading is used, use temporary derivatives_wrt_multipole_moments list first, and then update the real one.
+	ergo_real derivatives_wrt_multipole_moments_tmp[MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
+	memset(derivatives_wrt_multipole_moments_tmp, 0, MAX_NO_OF_MOMENTS_PER_MULTIPOLE*sizeof(ergo_real));
+	for(int i = 0; i < nDistrs; i++) {
+	  ergo_real tempVector2[boxNeededNoOfMoments];
+	  for(int B = 0; B < boxNeededNoOfMoments; B++) {
+	    ergo_real sum = 0;
+	    for(int A = 0; A < multipoleList[i].noOfMoments; A++) {
+	      ergo_real Telement = T[A*boxNeededNoOfMoments+B];
+	      ergo_real mom = multipoleList[i].momentList[A];
+	      sum += mom * Telement;
+	    }
+	    tempVector2[B] = sum;
+	  } // end for B
+	  int pairIdx = list[i].pairIdx;
+	  ergo_real extraFactor = 1.0;
+	  if(basisFuncIndexPairList[pairIdx].index_1 != basisFuncIndexPairList[pairIdx].index_2)
+	    extraFactor = 2.0;
+	  ergo_real densityMatrixElement = D_list[list[i].pairIdx];
+	  for(int B = 0; B < boxNeededNoOfMoments; B++)
+	    derivatives_wrt_multipole_moments_tmp[B] += -1 * extraFactor * densityMatrixElement * tempVector2[B];
+	} // end for i (loop over distrs)
+
+#ifdef _OPENMP
+#pragma omp critical
+#endif
+	{
+	  for(int B = 0; B < boxNeededNoOfMoments; B++)
+	    currBox->derivatives_wrt_multipole_moments[B] += derivatives_wrt_multipole_moments_tmp[B];
+	}
+
+      } // end if compute_gradient_also
+
       return 0;
     }
 
@@ -479,6 +633,76 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 	    }
 	  
 	} // END FOR ia
+
+      if(compute_gradient_also) {
+	for(int ia = 0; ia < noOfAtomsCurrBox; ia++) {
+	  const Atom* currAtom = &currAtomList[ia];
+	  const DistributionSpecStruct* distr1 = &list[0].distr;
+	  int n1 = Nmax;
+	  int n2 = 1;
+	  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1];
+	  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2];
+	  ergo_real primitiveIntegralList_h[noOfMonomials_1*noOfMonomials_2];
+	  // Let the distr be 1 and the pointcharge 2
+	  ergo_real alpha1 = distr1->exponent;
+	  ergo_real alpha0 = alpha1;  
+	  ergo_real dx0 = currAtom->coords[0] - distr1->centerCoords[0];
+	  ergo_real dx1 = currAtom->coords[1] - distr1->centerCoords[1];
+	  ergo_real dx2 = currAtom->coords[2] - distr1->centerCoords[2];
+	  ergo_real resultPreFactor = 2 * pi / alpha1;
+	  get_related_integrals_hermite(integralInfo,
+					CAM_params_not_used,
+					n1, noOfMonomials_1,
+					n2, noOfMonomials_2,
+					dx0, 
+					dx1, 
+					dx2, 
+					alpha0,
+					resultPreFactor,
+					primitiveIntegralList_h);
+	  for(int id = 0; id < nDistrs; id++) {
+	    const DistributionSpecStruct* distr = &list[id].distr;
+	    int n1b = n1;
+	    int n2b = 0;
+	    ergo_real primitiveIntegralList_h_components[3][noOfMonomials_1];
+	    int monomialIndex_x = integralInfo.monomial_info.monomial_index_list[1][0][0];
+	    int monomialIndex_y = integralInfo.monomial_info.monomial_index_list[0][1][0];
+	    int monomialIndex_z = integralInfo.monomial_info.monomial_index_list[0][0][1];
+	    for(int i = 0; i < noOfMonomials_1; i++) {
+	      primitiveIntegralList_h_components[0][i] = primitiveIntegralList_h[i*noOfMonomials_2+monomialIndex_x];
+	      primitiveIntegralList_h_components[1][i] = primitiveIntegralList_h[i*noOfMonomials_2+monomialIndex_y];
+	      primitiveIntegralList_h_components[2][i] = primitiveIntegralList_h[i*noOfMonomials_2+monomialIndex_z];
+	    }
+	    ergo_real primitiveIntegralList_2_components[3][noOfMonomials_1];
+	    for(int i = 0; i < 3; i++)
+	      integralInfo.multiply_by_hermite_conversion_matrix_from_right(n1b, n2b, 1.0/alpha1, primitiveIntegralList_h_components[i], primitiveIntegralList_2_components[i]);
+	    int n1x = distr->monomialInts[0];
+	    int n1y = distr->monomialInts[1];
+	    int n1z = distr->monomialInts[2];
+	    int monomialIndex = integralInfo.monomial_info.monomial_index_list[n1x][n1y][n1z];
+	    ergo_real value_x = currAtom->charge * distr->coeff * primitiveIntegralList_2_components[0][monomialIndex];
+	    ergo_real value_y = currAtom->charge * distr->coeff * primitiveIntegralList_2_components[1][monomialIndex];
+	    ergo_real value_z = currAtom->charge * distr->coeff * primitiveIntegralList_2_components[2][monomialIndex];
+	    int pairIdx = list[id].pairIdx;
+	    ergo_real extraFactor = 1.0;
+	    if(basisFuncIndexPairList[pairIdx].index_1 != basisFuncIndexPairList[pairIdx].index_2)
+	      extraFactor = 2.0;
+	    ergo_real densityMatrixElement = D_list[list[id].pairIdx];
+	    int atomIndex = atomPermutation[currBox->basicBox.firstItemIndex + ia];
+#ifdef _OPENMP
+#pragma omp critical
+#endif
+	    {
+	      result_gradient_list[atomIndex*3+0] += -1 * value_x * densityMatrixElement * extraFactor;
+	      result_gradient_list[atomIndex*3+1] += -1 * value_y * densityMatrixElement * extraFactor;
+	      result_gradient_list[atomIndex*3+2] += -1 * value_z * densityMatrixElement * extraFactor;
+	    }
+	  }
+
+	}
+
+      }
+
       return 0;
     }
 
@@ -490,6 +714,7 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
       if(do_interaction_recursive(integralInfo,
 				  V_list,
 				  noOfBasisFuncIndexPairs,
+				  basisFuncIndexPairList,
 				  list,
 				  nDistrs,
 				  multipoleList,
@@ -498,12 +723,16 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 				  maxDegreeForDistrs,
 				  distrExtent,
 				  atomListReordered,
+				  atomPermutation,
 				  threshold,
 				  boxList,
 				  interactor,
 				  childBoxIndex,
 				  currLevel + 1,
-				  numberOfLevels) != 0)
+				  numberOfLevels,
+				  compute_gradient_also,
+				  D_list,
+				  result_gradient_list) != 0)
 	return -1;
     } // END FOR i 
   return 0;
@@ -650,29 +879,33 @@ getSafeMaxDistance(const BasisInfoStruct& basisInfo, const Molecule& molecule)
   return maxDistance;
 }
 
-int 
-compute_V_linear(const BasisInfoStruct& basisInfo,
-		 const IntegralInfo& integralInfo,
-		 const Molecule& molecule,
-		 ergo_real threshold,
-		 ergo_real boxSize,
-		 const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
-		 ergo_real* V_list,
-		 int noOfBasisFuncIndexPairs)
+
+int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
+				  const IntegralInfo& integralInfo,
+				  const Molecule& molecule,
+				  ergo_real threshold,
+				  ergo_real boxSize,
+				  const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
+				  ergo_real* V_list,
+				  int noOfBasisFuncIndexPairs,
+				  bool compute_gradient_also,
+				  const ergo_real* D_list, // List of corresponding density matrix elemets; used for compute_gradient_also case, NULL otherwise
+				  ergo_real* gradient_list // list of result gradient values; used for compute_gradient_also case, NULL otherwise
+				  )
 {
   int errorCount = 0;
-  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_linear start.");
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear start, compute_gradient_also = %d.", compute_gradient_also);
 
-  Util::TimeMeter timeMeter;
+  Util::TimeMeter timeMeterTotal;
+  Util::TimeMeter timeMeterInitPart;
 
   // Get maxCharge
   ergo_real maxCharge = 0;
-  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
-    {
-      ergo_real currCharge = molecule.getAtom(i).charge;
-      if(currCharge > maxCharge)
-	maxCharge = currCharge;
-    }
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
+    ergo_real currCharge = molecule.getAtom(i).charge;
+    if(currCharge > maxCharge)
+      maxCharge = currCharge;
+  }
 
   // Create list of distributions
   int nDistrs = get_list_of_distrs_for_V(basisInfo,
@@ -684,10 +917,10 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
 					 0);
   if(nDistrs <= 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_linear: (nDistrs <= 0).");
+      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_and_gradient_linear: (nDistrs <= 0).");
       return -1;
     }
-  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_linear nDistrs    = %9i", nDistrs);
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear nDistrs    = %9i", nDistrs);
   std::vector<DistributionSpecStructWithPairIdx> list(nDistrs);
   if(get_list_of_distrs_for_V(basisInfo,
 			      basisFuncIndexPairList,
@@ -697,7 +930,7 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
 			      &list[0],
 			      nDistrs) != nDistrs)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_linear, in get_list_of_distrs_for_V.");
+      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_and_gradient_linear, in get_list_of_distrs_for_V.");
       return -1;
     }
 
@@ -729,7 +962,7 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
       // start next group
       currGroupInd = ind;
     }
-  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_linear groupCount = %9i", groupCount);
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear groupCount = %9i", groupCount);
 
   init_multipole_code();
   // Get maxDistance taking into account both molecule and basis set,
@@ -743,20 +976,25 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
 
   // Note that boxList and atomListReordered are allocated by create_nuclei_mm_tree,
   // we must remember to free them in the end.
+  BoxSystem boxSystem;
+  std::vector<int> atomPermutation(molecule.getNoOfAtoms());
   atom_box_struct* boxList = NULL;
   Atom *atomListReordered = NULL;
   int numberOfLevels = -1;
   if(create_nuclei_mm_tree(molecule.getNoOfAtoms(), molecule.getAtomListPtr(), boxSize,
+			   boxSystem,
 			   &boxList, &numberOfLevels,
-			   &atomListReordered) != 0) 
+			   &atomListReordered,
+			   &atomPermutation[0],
+			   compute_gradient_also) != 0) 
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "create_nuclei_mm_tree failed");
       return -1;
     }
 
-  timeMeter.print(LOG_AREA_INTEGRALS, "compute_V_linear init part (including create_nuclei_mm_tree)");
+  timeMeterInitPart.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear init part (including create_nuclei_mm_tree)");
 
-  // Clear result list
+  Util::TimeMeter timeMeterMainPart;
 
   const JK::ExchWeights CAM_params_not_used;
 
@@ -766,14 +1004,20 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
   {
   MMInteractor interactor;
   ergo_real *private_V_list = V_list;
+  ergo_real* private_gradient_list = gradient_list;
   multipole_struct_small* multipoleList =
     new multipole_struct_small[maxNDistrsPerGroup];
 #ifdef _OPENMP
-  if (omp_get_thread_num() != 0)
+  if (omp_get_thread_num() != 0) {
     private_V_list = new ergo_real[noOfBasisFuncIndexPairs];
+    if(compute_gradient_also)
+      private_gradient_list = new ergo_real[3*molecule.getNoOfAtoms()];
+  }
 #endif
 
   memset(private_V_list, 0, noOfBasisFuncIndexPairs*sizeof(ergo_real));
+  if(compute_gradient_also)
+    memset(private_gradient_list, 0, 3*molecule.getNoOfAtoms()*sizeof(ergo_real));
 #ifdef _OPENMP
 #pragma omp for
 #endif
@@ -829,7 +1073,7 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
       ergo_real R2 = -1 * (1/exponent) * std::log(threshold/(maxabscoeff*maxCharge));
       if(R2 < 0)
         {
-          do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_linear, (R2 < 0) when computing extent.");
+          do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_and_gradient_linear, (R2 < 0) when computing extent.");
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -841,6 +1085,7 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
       if(do_interaction_recursive(integralInfo,
 				  private_V_list, 
 				  noOfBasisFuncIndexPairs,
+				  basisFuncIndexPairList,
 				  currList,
 				  nDistrsCurrGroup,
 				  multipoleList,
@@ -849,12 +1094,16 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
 				  maxDegree,
 				  extent,
 				  atomListReordered,
+				  &atomPermutation[0],
 				  threshold,
 				  boxList,
 				  interactor,
 				  0,
 				  0,
-				  numberOfLevels) != 0)
+				  numberOfLevels,
+				  compute_gradient_also,
+				  D_list,
+				  private_gradient_list) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in do_interaction_recursive");
 #ifdef _OPENMP
@@ -871,20 +1120,65 @@ compute_V_linear(const BasisInfoStruct& basisInfo,
     {
       for(int i=0; i<noOfBasisFuncIndexPairs; i++)
         V_list[i] += private_V_list[i];
+      if(compute_gradient_also)
+	for(int i = 0; i < 3*molecule.getNoOfAtoms(); i++)
+	  gradient_list[i] += private_gradient_list[i];
     }
-    delete []private_V_list;
+    delete [] private_V_list;
+    delete [] private_gradient_list;
   }
 #endif
   delete []multipoleList;
   }
+
+  timeMeterMainPart.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear main part");
+
+  if(compute_gradient_also) {
+    Util::TimeMeter timeMeterGradientMultipolePart;
+    // Update gradient with multipole contributions
+    for(int boxIdx = 0; boxIdx < boxSystem.totNoOfBoxes; boxIdx++) {
+      atom_box_struct* currBox = &boxList[boxIdx];
+      int noOfAtomsCurrBox = currBox->basicBox.noOfItems;
+      // Go through all atoms in this box
+      for(int j = 0; j < noOfAtomsCurrBox; j++) {
+	int atomIndex = atomPermutation[currBox->basicBox.firstItemIndex + j];
+	for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
+	  for(int k = 0; k < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; k++) {
+	    ergo_real multipole_moment_derivative = currBox->multipole_moment_derivatives[j*MAX_NO_OF_MOMENTS_PER_MULTIPOLE*3 + MAX_NO_OF_MOMENTS_PER_MULTIPOLE*coordIdx + k];
+	    ergo_real derivative_wrt_multipole_moment = currBox->derivatives_wrt_multipole_moments[k];
+	    ergo_real gradient_contrib = multipole_moment_derivative * derivative_wrt_multipole_moment;
+	    gradient_list[atomIndex*3+coordIdx] += gradient_contrib;
+	  } // end for k
+	} // end for coordIdx
+      } // end for j
+    } // end for boxIdx
+    timeMeterGradientMultipolePart.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear gradient multipole contributions part");
+
+    // Update gradient with nuclear-nuclear energy contribution
+    Util::TimeMeter timeMeterNuclearRepulsionEnergyGradientContrib;
+    molecule.getNuclearRepulsionEnergyGradientContrib(gradient_list);
+    timeMeterNuclearRepulsionEnergyGradientContrib.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear getNuclearRepulsionEnergyGradientContrib");
+
+    // Cleanup: delete buffers that were allocated during creation of multipole tree
+    for(int boxIdx = 0; boxIdx < boxSystem.totNoOfBoxes; boxIdx++) {
+      atom_box_struct* currBox = &boxList[boxIdx];
+      delete [] currBox->multipole_moment_derivatives;
+      currBox->multipole_moment_derivatives = NULL;
+      delete [] currBox->derivatives_wrt_multipole_moments;
+      currBox->derivatives_wrt_multipole_moments = NULL;
+    }
+
+  } // end if compute_gradient_also    
+
   delete [] boxList;
   delete [] atomListReordered;
   
+  timeMeterTotal.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear total");
+
   return -errorCount;
 }
 
 
-
 static ergo_real 
 simplePrimVintegralSingle(DistributionSpecStruct* prim,
                           const Atom* atom,
@@ -898,7 +1192,7 @@ simplePrimVintegralSingle(DistributionSpecStruct* prim,
 }
 
 
-static ergo_real 
+ergo_real 
 simplePrimVintegral_list(DistributionSpecStruct* list,
                          int nPrims,
 			 const Atom* atom,
