@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,12 @@
 #include "basis_func_pair_list_1el.h"
 
 
+ergo_real simplePrimVintegral_list(DistributionSpecStruct* list,
+				   int nPrims,
+				   const Atom* atom,
+				   ergo_real threshold,
+				   const IntegralInfo & integralInfo);
+
 int compute_V_matrix_full(const BasisInfoStruct& basisInfo,
 			  const IntegralInfo& integralInfo,
 			  int nAtoms,
@@ -39,15 +45,18 @@ int compute_V_matrix_full(const BasisInfoStruct& basisInfo,
 			  ergo_real threshold,
 			  ergo_real* result);
 
-int compute_V_linear(const BasisInfoStruct& basisInfo,
-		     const IntegralInfo& integralInfo,
-		     const Molecule& molecule,
-		     ergo_real threshold,
-		     ergo_real boxSize,
-		     const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
-		     ergo_real* V_list,
-		     int noOfBasisFuncIndexPairs);
-
+int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
+				  const IntegralInfo& integralInfo,
+				  const Molecule& molecule,
+				  ergo_real threshold,
+				  ergo_real boxSize,
+				  const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
+				  ergo_real* V_list,
+				  int noOfBasisFuncIndexPairs,
+				  bool compute_gradient_also,
+				  const ergo_real* D_list,         // List of corresponding density matrix elemets; used for compute_gradient_also case, NULL otherwise
+				  ergo_real* result_gradient_list  // list of result gradient values; used for compute_gradient_also case, NULL otherwise
+				  );
 
 
 #endif
