@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,11 @@ ergo_real do_1e_repulsion_integral_using_symb_info(DistributionSpecStruct* psi,
 						   const ergo_real* pointChargeCoords,
 						   const IntegralInfo & integralInfo);
 
+std::vector<ergo_real> do_1e_repulsion_integral_derivatives_using_symb_info(const DistributionSpecStruct* psi,
+									    ergo_real pointCharge,
+									    const ergo_real* pointChargeCoords,
+									    const IntegralInfo & integralInfo);
+
 ergo_real do_1e_repulsion_integral_list_using_symb_info(DistributionSpecStruct* psi,
 							const ergo_real* pointChargeList,
 							const ergo_real* pointChargeCoordsList,
