@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,9 +37,11 @@
 #include "mm_limit_table.h"
 #include "basis_func_pair_list.h"
 #include "integrals_2el_repeating.h"
+#include "integrals_hermite.h"
 #include "integrals_general.h"
 #include "utilities.h"
 #include "pi.h"
+#include "integrals_2el_util_funcs.h"
 
 
 static const int HUGE_INTEGER_NUMBER = 2000000000;
@@ -62,53 +64,39 @@ typedef struct
 } job_list_multipole_entry_J_struct;
 
 
-
-typedef struct
-{
-  int a, b, c, d;
-  int poly_ab_index;
-  int poly_cd_index;
-  int idx1;
-  int idx2;
-  ergo_real densValue;
-} abcd_struct;
-
-#define set_abcd_list_item_macro(i,A,B,C,D,v,i1,i2)			\
-  list[i].a = A; list[i].b = B; list[i].c = C; list[i].d = D; list[i].densValue = v; list[i].idx1 = i1; list[i].idx2 = i2; 
-
-
-
 static int 
-get_J_contribs_from_2_interacting_boxes_2(const BasisInfoStruct & basisInfo,
-					  const IntegralInfo & integralInfo,
-					  int maxNoOfMonomials,
-					  ergo_real* result_J_list,
-					  const minimal_distr_struct* minimalDistrList_1,
-					  int noOfGroups_1,
-					  const distr_group_struct* groupList_1,
-					  const minimal_distr_struct* minimalDistrList_2,
-					  int noOfGroups_2,
-					  const distr_group_struct* groupList_2,
-					  const cluster_struct* clusterList_1,
-					  int nClusters_1,
-					  const cluster_struct* clusterList_2,
-					  int nClusters_2,
-					  const chunk_struct* chunkList_1,
-					  int nChunks_1,
-					  const chunk_struct* chunkList_2,
-					  int nChunks_2,
-					  const basis_func_pair_struct* basisFuncPairList_1,
-					  const basis_func_pair_struct* basisFuncPairList_2,
-					  int interactionWithSelf,
-					  ergo_real threshold,
-					  JK_contribs_buffer_struct* bufferStructPtr)
+get_J_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
+					const IntegralInfo & integralInfo,
+					int maxNoOfMonomials,
+					ergo_real* result_J_list,
+					const distr_org_struct & distr_org_struct_1,
+					const distr_org_struct & distr_org_struct_2,
+					int interactionWithSelf,
+					ergo_real threshold,
+					JK_contribs_buffer_struct* bufferStructPtr)
 {
   const JK::ExchWeights CAM_params_not_used;
 
   const ergo_real twoTimesPiToPow5half = 2 * pitopow52;
   ergo_real* summedIntegralList = bufferStructPtr->summedIntegralList;
   ergo_real* primitiveIntegralList = bufferStructPtr->primitiveIntegralList;
-  ergo_real* primitiveIntegralList_work = bufferStructPtr->primitiveIntegralList_work;
+
+  const distr_group_struct* groupList_1 = &distr_org_struct_1.groupList[0];
+  const distr_group_struct* groupList_2 = &distr_org_struct_2.groupList[0];
+  const cluster_struct* clusterList_1 = &distr_org_struct_1.clusterList[0];
+  const cluster_struct* clusterList_2 = &distr_org_struct_2.clusterList[0];
+  const chunk_struct* chunkList_1 = &distr_org_struct_1.chunkList[0];
+  int nChunks_1 = distr_org_struct_1.chunkCount;
+  const chunk_struct* chunkList_2 = &distr_org_struct_2.chunkList[0];
+  int nChunks_2 = distr_org_struct_2.chunkCount;
+  const basis_func_pair_struct* basisFuncPairList_1 = &distr_org_struct_1.basisFuncPairList[0];
+  const basis_func_pair_struct* basisFuncPairList_2 = &distr_org_struct_2.basisFuncPairList[0];
+  const i_j_val_struct* spMatElementList_1 = &distr_org_struct_1.spMatElementList[0];
+  const int* spMatCountList_1 = &distr_org_struct_1.spMatCountList[0];
+  const int* spMatIdxList_1 = &distr_org_struct_1.spMatIdxList[0];
+  const i_j_val_struct* spMatElementList_2 = &distr_org_struct_2.spMatElementList[0];
+  const int* spMatCountList_2 = &distr_org_struct_2.spMatCountList[0];
+  const int* spMatIdxList_2 = &distr_org_struct_2.spMatIdxList[0];
 
   for(int chunk_i = 0; chunk_i < nChunks_1; chunk_i++)
     {
@@ -192,46 +180,30 @@ get_J_contribs_from_2_interacting_boxes_2(const BasisInfoStruct & basisInfo,
 			  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1max];
 			  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2max];
 
-			  if(get_related_integrals_h(integralInfo, 
-						     CAM_params_not_used,
-						     n1max, noOfMonomials_1,
-						     n2max, noOfMonomials_2,
-						     dx, dy, dz, alpha_1, alpha_2, alpha_0,
-						     primitiveIntegralList,
-						     primitiveIntegralList_work,
-						     resultPreFactor
-						     ) != 0)
-			    {
-			      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_related_integrals");
-			      return -1;
-			    }
+			  get_related_integrals_hermite(integralInfo,
+							CAM_params_not_used,
+							n1max, noOfMonomials_1,
+							n2max, noOfMonomials_2,
+							dx, dy, dz, alpha_0,
+							resultPreFactor,
+							primitiveIntegralList);
 
-			  int i_start = groupList_1[group_i].startIndex;
-			  int i_end = i_start + groupList_1[group_i].distrCount;
-			  for(int i = i_start; i < i_end; i++)
-			    {
-			      int idx_1 = minimalDistrList_1[i].basisFuncPairIndex;
-			      int monomialIndex_1 = minimalDistrList_1[i].monomialIndex;
-			      int j_start = groupList_2[group_j].startIndex;
-			      int j_end = j_start + groupList_2[group_j].distrCount;
-			      if(interactionWithSelf == 1 && group_j == group_i && chunk_i == chunk_j && cluster_i == cluster_j)
-				{
-				  // take care of case i = j separately
-				  ergo_real integralValue = primitiveIntegralList[monomialIndex_1*noOfMonomials_2+monomialIndex_1];
-				  ergo_real integralValueCurr = minimalDistrList_1[i].coeff * minimalDistrList_1[i].coeff * integralValue;
-				  integralValueCurr *= 0.5;				  
-				  summedIntegralList[idx_1*noOfBasisFuncPairs_2 + idx_1] += integralValueCurr;
-				  j_start = i+1;
-				}
-			      for(int j = j_start; j < j_end; j++)
-				{
-				  int idx_2 = minimalDistrList_2[j].basisFuncPairIndex;
-				  int monomialIndex_2 = minimalDistrList_2[j].monomialIndex;
-				  ergo_real integralValue = primitiveIntegralList[monomialIndex_1*noOfMonomials_2+monomialIndex_2];
-				  ergo_real integralValueCurr = minimalDistrList_1[i].coeff * minimalDistrList_2[j].coeff * integralValue;
-				  summedIntegralList[idx_1*noOfBasisFuncPairs_2 + idx_2] += integralValueCurr;
-				} // END FOR j 		      			      
-			    } // END FOR i
+			  if(interactionWithSelf == 1 && group_j == group_i && chunk_i == chunk_j && cluster_i == cluster_j) {
+			    do_summedIntegralList_contribs_self(&spMatElementList_1[spMatIdxList_1[group_i]], spMatCountList_1[group_i],
+								&spMatElementList_2[spMatIdxList_2[group_j]], spMatCountList_2[group_j],
+								noOfMonomials_1, noOfMonomials_2,
+								primitiveIntegralList,
+								noOfBasisFuncPairs_1, noOfBasisFuncPairs_2,
+								summedIntegralList);
+			  }
+			  else {
+			    do_summedIntegralList_contribs_std(&spMatElementList_1[spMatIdxList_1[group_i]], spMatCountList_1[group_i],
+							       &spMatElementList_2[spMatIdxList_2[group_j]], spMatCountList_2[group_j],
+							       noOfMonomials_1, noOfMonomials_2,
+							       primitiveIntegralList,
+							       noOfBasisFuncPairs_1, noOfBasisFuncPairs_2,
+							       summedIntegralList);
+			  }
 
 			} // END FOR group_j
 		    } // END FOR group_i
@@ -259,74 +231,31 @@ get_J_contribs_from_2_interacting_boxes_2(const BasisInfoStruct & basisInfo,
 		if(std::fabs(integralValueCurr)*maxabsdmatelement < threshold)
 		  continue;
 
-		if(a != b && c != d && a != c && a != d && b != c && b != d)
-		  {
-		    result_J_list[J_list_index_ab] += 2 * D_cd * integralValueCurr;
-		    result_J_list[J_list_index_cd] += 2 * D_ab * integralValueCurr;
-		  }
-		else
-		  {		    
-		    abcd_struct list[8];
-      
-		    /* determine unique configurations */
-		    set_abcd_list_item_macro(0, a, b, c, d, 0, 0, 0);
-		    set_abcd_list_item_macro(1, a, b, d, c, 0, 0, 0);
-		    set_abcd_list_item_macro(2, b, a, c, d, 0, 0, 0);
-		    set_abcd_list_item_macro(3, b, a, d, c, 0, 0, 0);
-		    set_abcd_list_item_macro(4, c, d, a, b, 0, 0, 0);
-		    set_abcd_list_item_macro(5, d, c, a, b, 0, 0, 0);
-		    set_abcd_list_item_macro(6, c, d, b, a, 0, 0, 0);
-		    set_abcd_list_item_macro(7, d, c, b, a, 0, 0, 0);
-
-		    int ccc = 0;
-	  
-		    for(int ii = 0; ii < 8; ii++)
-		      {
-			abcd_struct* abcd = &list[ii];
-			int aa, bb, cc, dd;
-
-			/* check if this is a new unique configuration */
-			int unique = 1;
-			for(int jj = 0; jj < ii; jj++)
-			  {
-			    if(abcd->a == list[jj].a && 
-			       abcd->b == list[jj].b && 
-			       abcd->c == list[jj].c && 
-			       abcd->d == list[jj].d)
-			      unique = 0;
-			  }
-			if(unique == 0)
-			  continue;
-			/* now we know that this configuration is unique. */
-			aa = abcd->a;
-			bb = abcd->b;
-			cc = abcd->c;
-			dd = abcd->d;
-
-			ccc++;
-
-			/* add contribution to coulomb matrix */
-			ergo_real D_xx;
-			int pairIndex;
-			if((cc == c && dd == d) || (cc == d && dd == c))
-			  {
-			    pairIndex = J_list_index_ab;
-			    D_xx = D_cd;
-			  }
-			else
-			  {
-			    pairIndex = J_list_index_cd;
-			    D_xx = D_ab;
-			  }
-			if(bb >= aa)
-			  {
-			    //printf("adding to J list, pairIndex = %i, D_xx = %22.11f, integralValueCurr = %22.11f\n", pairIndex, D_xx, integralValueCurr);
-			    result_J_list[pairIndex] += D_xx * integralValueCurr;
-			    //printf("added OK\n");
-			  }
-			
-		      } /* END FOR ii go through 8 configurations */
-		  }
+		if(a != b && c != d && (a != c || b != d)) {
+		  result_J_list[J_list_index_ab] += 2 * D_cd * integralValueCurr;
+		  result_J_list[J_list_index_cd] += 2 * D_ab * integralValueCurr;
+		}
+		else if(a != b && c != d && a == c && b == d) {
+		  result_J_list[J_list_index_ab] += 2 * D_cd * integralValueCurr;
+		}
+		else if(a == b && c != d) {
+		  result_J_list[J_list_index_ab] += 2 * D_cd * integralValueCurr;
+		  result_J_list[J_list_index_cd] += 1 * D_ab * integralValueCurr;
+		}
+		else if(a != b && c == d) {
+		  result_J_list[J_list_index_ab] += 1 * D_cd * integralValueCurr;
+		  result_J_list[J_list_index_cd] += 2 * D_ab * integralValueCurr;
+		}
+		else if(a == b && c == d && a != c) {
+		  result_J_list[J_list_index_ab] += D_cd * integralValueCurr;
+		  result_J_list[J_list_index_cd] += D_ab * integralValueCurr;
+		}
+		else if(a == b && c == d && a == c) {
+		  result_J_list[J_list_index_ab] += D_cd * integralValueCurr;
+		}
+		else {
+		  return -1; // This should never happen
+		}
 
 	      } // END FOR idx_1 idx_2
 	} // END FOR chunk_j
@@ -1119,39 +1048,17 @@ execute_joblist_J_std_serial(int noOfJobs_J_standard,
       int self = 0;
       if(boxIndex_1 == boxIndex_2 && branchIndex_1 == branchIndex_2)
 	self = 1;
-      if(get_J_contribs_from_2_interacting_boxes_2(basisInfo,
-						   integralInfo,
-						   maxNoOfMonomials,
-						   result_J_list,
-						  
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.minimalDistrList[0],
-						   boxList[boxIndex_1].branchList[branchIndex_1].org.groupCount,
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.groupList[0],
-
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.minimalDistrList[0],
-						   boxList[boxIndex_2].branchList[branchIndex_2].org.groupCount,
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.groupList[0],
-
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.clusterList[0],
-						   boxList[boxIndex_1].branchList[branchIndex_1].org.clusterCount,
-						    
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.clusterList[0],
-						   boxList[boxIndex_2].branchList[branchIndex_2].org.clusterCount,
-
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.chunkList[0],
-						   boxList[boxIndex_1].branchList[branchIndex_1].org.chunkCount,
-
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.chunkList[0],
-						   boxList[boxIndex_2].branchList[branchIndex_2].org.chunkCount,
-
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.basisFuncPairList[0],
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.basisFuncPairList[0],
-
-						   self,
-						   threshold,
-						   &bufferStruct) != 0)
+      if(get_J_contribs_from_2_interacting_boxes(basisInfo,
+						 integralInfo,
+						 maxNoOfMonomials,
+						 result_J_list,
+						 boxList[boxIndex_1].branchList[branchIndex_1].org,
+						 boxList[boxIndex_2].branchList[branchIndex_2].org,
+						 self,
+						 threshold,
+						 &bufferStruct) != 0)
 	{
-	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_JK_contribs_from_2_interacting_boxes");
+	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_J_contribs_from_2_interacting_boxes");
 	  return -1;
 	}
     } // END FOR jobIndex
@@ -1205,39 +1112,17 @@ execute_joblist_J_std_thread_func(void* arg)
       int self = 0;
       if(boxIndex_1 == boxIndex_2 && branchIndex_1 == branchIndex_2)
 	self = 1;
-      if(get_J_contribs_from_2_interacting_boxes_2(params->basisInfo,
-						   *params->integralInfo,
-						   params->maxNoOfMonomials,
-						   params->result_J_list,
-						  
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.minimalDistrList[0],
-						   boxList[boxIndex_1].branchList[branchIndex_1].org.groupCount,
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.groupList[0],
-
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.minimalDistrList[0],
-						   boxList[boxIndex_2].branchList[branchIndex_2].org.groupCount,
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.groupList[0],
-
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.clusterList[0],
-						   boxList[boxIndex_1].branchList[branchIndex_1].org.clusterCount,
-						    
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.clusterList[0],
-						   boxList[boxIndex_2].branchList[branchIndex_2].org.clusterCount,
-
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.chunkList[0],
-						   boxList[boxIndex_1].branchList[branchIndex_1].org.chunkCount,
-
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.chunkList[0],
-						   boxList[boxIndex_2].branchList[branchIndex_2].org.chunkCount,
-
-						   &boxList[boxIndex_1].branchList[branchIndex_1].org.basisFuncPairList[0],
-						   &boxList[boxIndex_2].branchList[branchIndex_2].org.basisFuncPairList[0],
-
-						   self,
-						   params->threshold,
-						   &bufferStruct) != 0)
+      if(get_J_contribs_from_2_interacting_boxes(params->basisInfo,
+						 *params->integralInfo,
+						 params->maxNoOfMonomials,
+						 params->result_J_list,
+						 boxList[boxIndex_1].branchList[branchIndex_1].org,
+						 boxList[boxIndex_2].branchList[branchIndex_2].org,
+						 self,
+						 params->threshold,
+						 &bufferStruct) != 0)
 	{
-	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_JK_contribs_from_2_interacting_boxes");
+	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_J_contribs_from_2_interacting_boxes");
 	  params->resultCode = -1;
 	  return NULL;
 	}
