@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,10 +31,13 @@
 
 #include "integrals_2el_exchange.h"
 #include "integrals_2el_utils.h"
+#include "integrals_hermite.h"
 #include "mm_limit_table.h"
 #include "pi.h"
 #include "pthread.h"
 #include "utilities.h"
+#include "matrix_algebra.h"
+#include "integrals_2el_util_funcs.h"
 
 
 static const int HUGE_INTEGER_NUMBER = 2000000000;
@@ -98,8 +101,6 @@ ergo_CSR_get_element_inline(const csr_matrix_struct* csr,
 }
 
 
-
-
 typedef struct
 {
   int a, b, c, d;
@@ -126,42 +127,45 @@ get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 					const ergo_real* dens,
 					const csr_matrix_struct* dens_CSR,
 					int symmetryFlag,
-					const minimal_distr_struct* minimalDistrList_1,
-					int noOfGroups_1,
-					const distr_group_struct* groupList_1,
-					const minimal_distr_struct* minimalDistrList_2,
-					int noOfGroups_2,
-					const distr_group_struct* groupList_2,
-					const cluster_struct* clusterList_1,
-					int nClusters_1,
-					const cluster_struct* clusterList_2,
-					int nClusters_2,
-					const chunk_struct* chunkList_1,
-					int nChunks_1,
-					const chunk_struct* chunkList_2,
-					int nChunks_2,
-					const basis_func_pair_struct* basisFuncPairList_1,
-					const basis_func_pair_struct* basisFuncPairList_2,
+					const distr_org_struct & distr_org_struct_1,
+					const distr_org_struct & distr_org_struct_2,
 					int interactionWithSelf,
 					ergo_real threshold,
 					JK_contribs_buffer_struct* bufferStructPtr,
 					int use_multipole_screening_for_clusters,
-					ergo_real boxDistance,
-					const int* basisFuncListForChunks_1,
-					const int* basisFuncListForChunks_map_1,
-					const int* basisFuncListForChunks_2,
-					const int* basisFuncListForChunks_map_2,
-					const int* basisFuncList_1,
-					int basisFuncList_1_count,
-					const int* basisFuncList_2,
-					int basisFuncList_2_count)
+					ergo_real boxDistance)
 {
   int n = basisInfo.noOfBasisFuncs;
 
   const ergo_real twoTimesPiToPow5half = 2 * pitopow52;// = 2 * pow(pi, 2.5);
   ergo_real* summedIntegralList = bufferStructPtr->summedIntegralList;
   ergo_real* primitiveIntegralList = bufferStructPtr->primitiveIntegralList;
-  ergo_real* primitiveIntegralList_work = bufferStructPtr->primitiveIntegralList_work;
+
+  int nChunks_1 = distr_org_struct_1.chunkCount;
+  int nChunks_2 = distr_org_struct_2.chunkCount;
+  const chunk_struct* chunkList_1 = &distr_org_struct_1.chunkList[0];
+  const chunk_struct* chunkList_2 = &distr_org_struct_2.chunkList[0];
+  const cluster_struct* clusterList_1 = &distr_org_struct_1.clusterList[0];
+  const cluster_struct* clusterList_2 = &distr_org_struct_2.clusterList[0];
+  const distr_group_struct* groupList_1 = &distr_org_struct_1.groupList[0];
+  const distr_group_struct* groupList_2 = &distr_org_struct_2.groupList[0];
+  const basis_func_pair_struct* basisFuncPairList_1 = &distr_org_struct_1.basisFuncPairList[0];
+  const basis_func_pair_struct* basisFuncPairList_2 = &distr_org_struct_2.basisFuncPairList[0];
+  const int* basisFuncListForChunks_1 = &distr_org_struct_1.basisFuncListForChunks[0];
+  const int* basisFuncListForChunks_map_1 = &distr_org_struct_1.basisFuncListForChunks_map[0];
+  const int* basisFuncListForChunks_2 = &distr_org_struct_2.basisFuncListForChunks[0];
+  const int* basisFuncListForChunks_map_2 = &distr_org_struct_2.basisFuncListForChunks_map[0];
+  const int* basisFuncList_1 = &distr_org_struct_1.basisFuncList[0];
+  int basisFuncList_1_count = distr_org_struct_1.basisFuncListCount;
+  const int* basisFuncList_2 = &distr_org_struct_2.basisFuncList[0];
+  int basisFuncList_2_count = distr_org_struct_2.basisFuncListCount;
+
+  const i_j_val_struct* spMatElementList_1 = &distr_org_struct_1.spMatElementList[0];
+  const int* spMatCountList_1 = &distr_org_struct_1.spMatCountList[0];
+  const int* spMatIdxList_1 = &distr_org_struct_1.spMatIdxList[0];
+  const i_j_val_struct* spMatElementList_2 = &distr_org_struct_2.spMatElementList[0];
+  const int* spMatCountList_2 = &distr_org_struct_2.spMatCountList[0];
+  const int* spMatIdxList_2 = &distr_org_struct_2.spMatIdxList[0];
 
   // Set up "partial box-box density matrix"
   int nnn1 = basisFuncList_1_count;
@@ -290,6 +294,8 @@ get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 
 		  int n1max = clusterList_1[cluster_i].nmax;
 		  int n2max = clusterList_2[cluster_j].nmax;
+		  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1max];
+		  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2max];
 
 		  // Now we can precompute things that depend only on exponents
 		  ergo_real alpha_1 = groupList_1[group_i_start].exponent;
@@ -297,7 +303,6 @@ get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 		  ergo_real alphasum = alpha_1 + alpha_2;
 		  ergo_real alphaproduct = alpha_1 * alpha_2;
 		  ergo_real alpha_0 = alphaproduct / alphasum;
-
 		  ergo_real resultPreFactor = twoTimesPiToPow5half / (alphaproduct*std::sqrt(alphasum));
 
 		  for(int group_i = group_i_start; group_i < group_i_end; group_i++)
@@ -330,51 +335,31 @@ get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 			    } // END IF try multipole screening
 
 			  // now we can do all integrals needed for this pair of groups
-
 			  // now we have dx dy dz alpha0 alpha1 n1max n2max. Get all integrals for this case.
-			  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1max];
-			  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2max];
+			  get_related_integrals_hermite(integralInfo,
+							CAM_params,
+							n1max, noOfMonomials_1,
+							n2max, noOfMonomials_2,
+							dx, dy, dz, alpha_0,
+							resultPreFactor,
+							primitiveIntegralList);
 
-			  if(get_related_integrals_h(integralInfo, 
-						     CAM_params,
-						     n1max, noOfMonomials_1,
-						     n2max, noOfMonomials_2,
-						     dx, dy, dz, alpha_1, alpha_2, alpha_0,
-						     primitiveIntegralList,
-						     primitiveIntegralList_work,
-						     resultPreFactor
-						     ) != 0)
-			    {
-			      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_related_integrals");
-			      return -1;
-			    }
-
-			  int i_start = groupList_1[group_i].startIndex;
-			  int i_end = i_start + groupList_1[group_i].distrCount;
-			  for(int i = i_start; i < i_end; i++)
-			    {
-			      int idx_1 = minimalDistrList_1[i].basisFuncPairIndex;
-			      int monomialIndex_1 = minimalDistrList_1[i].monomialIndex;
-			      int j_start = groupList_2[group_j].startIndex;
-			      int j_end = j_start + groupList_2[group_j].distrCount;
-			      if(interactionWithSelf == 1 && group_j == group_i && chunk_i == chunk_j && cluster_i == cluster_j)
-				{
-				  // take care of case i = j separately
-				  ergo_real integralValue = primitiveIntegralList[monomialIndex_1*noOfMonomials_2+monomialIndex_1];
-				  ergo_real integralValueCurr = minimalDistrList_1[i].coeff * minimalDistrList_1[i].coeff * integralValue;
-				  integralValueCurr *= 0.5;				  
-				  summedIntegralList[idx_1*noOfBasisFuncPairs_2 + idx_1] += integralValueCurr;
-				  j_start = i+1;
-				}
-			      for(int j = j_start; j < j_end; j++)
-				{
-				  int idx_2 = minimalDistrList_2[j].basisFuncPairIndex;
-				  int monomialIndex_2 = minimalDistrList_2[j].monomialIndex;
-				  ergo_real integralValue = primitiveIntegralList[monomialIndex_1*noOfMonomials_2+monomialIndex_2];
-				  ergo_real integralValueCurr = minimalDistrList_1[i].coeff * minimalDistrList_2[j].coeff * integralValue;
-				  summedIntegralList[idx_1*noOfBasisFuncPairs_2 + idx_2] += integralValueCurr;
-				} // END FOR j 		      			      
-			    } // END FOR i
+			  if(interactionWithSelf == 1 && group_j == group_i && chunk_i == chunk_j && cluster_i == cluster_j) {
+			    do_summedIntegralList_contribs_self(&spMatElementList_1[spMatIdxList_1[group_i]], spMatCountList_1[group_i],
+								&spMatElementList_2[spMatIdxList_2[group_j]], spMatCountList_2[group_j],
+								noOfMonomials_1, noOfMonomials_2,
+								primitiveIntegralList,
+								noOfBasisFuncPairs_1, noOfBasisFuncPairs_2,
+								summedIntegralList);
+			  }
+			  else {
+			    do_summedIntegralList_contribs_std(&spMatElementList_1[spMatIdxList_1[group_i]], spMatCountList_1[group_i],
+							       &spMatElementList_2[spMatIdxList_2[group_j]], spMatCountList_2[group_j],
+							       noOfMonomials_1, noOfMonomials_2,
+							       primitiveIntegralList,
+							       noOfBasisFuncPairs_1, noOfBasisFuncPairs_2,
+							       summedIntegralList);
+			  }
 
 			} // END FOR group_j
 		    } // END FOR group_i
@@ -425,7 +410,86 @@ get_K_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 		  dens_db = local_dmat_2[b_mod][d_mod];
 		}
 
-		if(a != b && c != d && a != c && a != d && b != c && b != d)
+
+
+
+
+		if(symmetryFlag && !K) {
+		  
+		  if(a != b && c != d && a != c && a != d && b != c && b != d) {
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+c_mod2] += -0.5 * dens_ad * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr;
+		  }
+		  else if(a == b && c != d && a != c && a != d && b != c && b != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+c_mod2] += -0.5 * dens_ad * integralValueCurr;
+		  }
+		  else if(a != b && c == d && a != c && a != d && b != c && b != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+c_mod2] += -0.5 * dens_ad * integralValueCurr;
+		  }
+		  else if(a != b && c != d && a == c && a != d && b != c && b != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr * 2.0;
+		    partial_K_1[b_mod2*nnn2+c_mod2] += -0.5 * dens_ad * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr;
+		  }
+		  else if(a != b && c != d && a != c && a == d && b != c && b != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr * 2.0;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+c_mod2] += -0.5 * dens_ad * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr;
+		  }
+		  else if(a != b && c != d && a != c && a != d && b == c && b != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+c_mod2] += -0.5 * dens_ad * integralValueCurr * 2.0;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr;
+		  }
+		  else if(a != b && c != d && a != c && a != d && b != c && b == d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+c_mod2] += -0.5 * dens_ad * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr * 2.0;
+		  }
+		  else if(a != b && c != d && a == c && b == d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr;
+		  }
+		  else if(a == b && c == d && a != c && a != d && b != c && b != d) { // OK
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		  }
+		  else if(a == b && c == d && a == c && a == d && b == c && b == d) { // OK
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		  }
+		  else if(a == b && c != d && a == c && a != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr*2.0;
+		  }
+		  else if(a == b && c != d && a != c && a == d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr*2.0;
+		    partial_K_1[a_mod2*nnn2+c_mod2] += -0.5 * dens_bd * integralValueCurr;
+		  }
+
+		  else if(a != b && c == d && a == c && b != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr*2.0;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr;
+		  }
+		  else if(a != b && c == d && b == c && a != d) { // OK
+		    partial_K_1[a_mod2*nnn2+d_mod2] += -0.5 * dens_bc * integralValueCurr;
+		    partial_K_1[b_mod2*nnn2+d_mod2] += -0.5 * dens_ac * integralValueCurr*2.0;
+		  }
+		  else {
+		    return -1;
+		  }
+
+
+
+		}
+		else if(a != b && c != d && a != c && a != d && b != c && b != d)
 		  {
 		    if(symmetryFlag)
 		      {
@@ -783,48 +847,14 @@ execute_joblist_K_thread_func(void* arg)
 						 params->K_CSR_shared,
 						 params->dens,
 						 params->densCSR,
-						 params->symmetryFlag,
-						 
-						 &boxList[boxIndex_1].distrListForK.org.minimalDistrList[0],
-						 boxList[boxIndex_1].distrListForK.org.groupCount,
-						 &boxList[boxIndex_1].distrListForK.org.groupList[0],
-
-						 &boxList[boxIndex_2].distrListForK.org.minimalDistrList[0],
-						 boxList[boxIndex_2].distrListForK.org.groupCount,
-						 &boxList[boxIndex_2].distrListForK.org.groupList[0],
-
-						 &boxList[boxIndex_1].distrListForK.org.clusterList[0],
-						 boxList[boxIndex_1].distrListForK.org.clusterCount,
-						    
-						 &boxList[boxIndex_2].distrListForK.org.clusterList[0],
-						 boxList[boxIndex_2].distrListForK.org.clusterCount,
-
-						 &boxList[boxIndex_1].distrListForK.org.chunkList[0],
-						 boxList[boxIndex_1].distrListForK.org.chunkCount,
-
-						 &boxList[boxIndex_2].distrListForK.org.chunkList[0],
-						 boxList[boxIndex_2].distrListForK.org.chunkCount,
-
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncPairList[0],
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncPairList[0],
-
+						 params->symmetryFlag,						 
+						 boxList[boxIndex_1].distrListForK.org,
+						 boxList[boxIndex_2].distrListForK.org,
 						 self,
 						 params->threshold,
 						 &bufferStruct,
-
 						 params->jobList_K[jobIndex].useMultipole,
-						 params->jobList_K[jobIndex].distance,
-
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncListForChunks[0],
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncListForChunks_map[0],
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncListForChunks[0],
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncListForChunks_map[0],
-
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncList[0],
-						 boxList[boxIndex_1].distrListForK.org.basisFuncListCount,
-
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncList[0],
-						 boxList[boxIndex_2].distrListForK.org.basisFuncListCount
+						 params->jobList_K[jobIndex].distance
 						 ) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_K_contribs_from_2_interacting_boxes");
@@ -1059,47 +1089,13 @@ execute_joblist_K_serial(const ergo_real* dens,
 						 dens,
 						 densCSR,
 						 symmetryFlag,
-						  
-						 &boxList[boxIndex_1].distrListForK.org.minimalDistrList[0],
-						 boxList[boxIndex_1].distrListForK.org.groupCount,
-						 &boxList[boxIndex_1].distrListForK.org.groupList[0],
-
-						 &boxList[boxIndex_2].distrListForK.org.minimalDistrList[0],
-						 boxList[boxIndex_2].distrListForK.org.groupCount,
-						 &boxList[boxIndex_2].distrListForK.org.groupList[0],
-
-						 &boxList[boxIndex_1].distrListForK.org.clusterList[0],
-						 boxList[boxIndex_1].distrListForK.org.clusterCount,
-						    
-						 &boxList[boxIndex_2].distrListForK.org.clusterList[0],
-						 boxList[boxIndex_2].distrListForK.org.clusterCount,
-
-						 &boxList[boxIndex_1].distrListForK.org.chunkList[0],
-						 boxList[boxIndex_1].distrListForK.org.chunkCount,
-
-						 &boxList[boxIndex_2].distrListForK.org.chunkList[0],
-						 boxList[boxIndex_2].distrListForK.org.chunkCount,
-
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncPairList[0],
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncPairList[0],
-
+						 boxList[boxIndex_1].distrListForK.org,
+						 boxList[boxIndex_2].distrListForK.org,						
 						 self,
 						 threshold,
 						 &bufferStruct,
-
 						 jobList_K[jobIndex].useMultipole,
-						 jobList_K[jobIndex].distance,
-
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncListForChunks[0],
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncListForChunks_map[0],
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncListForChunks[0],
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncListForChunks_map[0],
-
-						 &boxList[boxIndex_1].distrListForK.org.basisFuncList[0],
-						 boxList[boxIndex_1].distrListForK.org.basisFuncListCount,
-
-						 &boxList[boxIndex_2].distrListForK.org.basisFuncList[0],
-						 boxList[boxIndex_2].distrListForK.org.basisFuncListCount
+						 jobList_K[jobIndex].distance
 						 ) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_K_contribs_from_2_interacting_boxes");
@@ -2207,7 +2203,7 @@ compute_K_by_boxes(const BasisInfoStruct & basisInfo,
   output_current_memory_usage(LOG_AREA_INTEGRALS, "after freeing stuff at end of compute_K_by_boxes");
 
   timeMeterTot.print(LOG_AREA_INTEGRALS, "compute_K_by_boxes");
-  
+
   return 0;
 }
 
