@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,11 +49,15 @@ find_int_in_sorted_list(const int* list, int listLength, int i)
 {
   if(listLength == 0)
     return 0;
-  int lo = 0;
-  int hi = listLength-1;
+  /* ELIAS NOTE 2014-03-27: chaning datatype from "int" to "unsigned
+     int" for the lo, hi, mid variables here gave a considerable
+     performance inprovement. Apparently the computation of mid is
+     more efficient for unsigned int.  */
+  unsigned int lo = 0;
+  unsigned int hi = listLength-1;
   while(lo < hi-1)
     {
-      int mid = (lo + hi) / 2;
+      unsigned int mid = (lo + hi) / 2;
       if(list[mid] > i)
 	hi = mid;
       else
@@ -161,7 +165,11 @@ identify_needed_elements_part(ergo_real threshold,
   const int n = dens_CSR->n;
   int shortListCounterList[n];
   memset(shortListCounterList, 0, n*sizeof(int));
-  const int sll = 20; // sll = "short list length"
+  /* ELIAS NOTE 2014-03-27: the "short list length" value chosen here
+     has considerable impact on performance. Increasing it from 20 to
+     100 made the identify_needed_elements_part call 10% faster. This
+     effect could be machine-dependent, of course.  */
+  const int sll = 100; // sll = "short list length"
   std::vector<int> shortList(n*sll); // shortList is n lists of sll elements each.
 
   // now go through all nonzero dmat elements.
@@ -517,9 +525,9 @@ create_CSR_for_K(const BasisInfoStruct & basisInfo,
 					      maxLimitingFactor,
 					      NULL,
 					      maxDensityMatrixElement);
-  if(distrCount <= 0)
+  if(distrCount < 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in create_CSR_for_K: (distrCount <= 0)");
+      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in create_CSR_for_K: (distrCount < 0)");
       return -1;
     }
 
