@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,7 +67,28 @@ do_2e_integral(int mu,
 	       int la, 
 	       int si, 
 	       const BasisInfoStruct & basisInfo, 
-	       const IntegralInfo & integralInfo)
+	       const IntegralInfo & integralInfo) {
+  return do_2e_integral_general(mu, 
+				nu, 
+				la, 
+				si, 
+				basisInfo, 
+				basisInfo, 
+				basisInfo, 
+				basisInfo, 
+				integralInfo);
+}
+
+ergo_real 
+do_2e_integral_general(int mu, 
+		       int nu, 
+		       int la, 
+		       int si, 
+		       const BasisInfoStruct & basisInfo_mu, 
+		       const BasisInfoStruct & basisInfo_nu, 
+		       const BasisInfoStruct & basisInfo_la, 
+		       const BasisInfoStruct & basisInfo_si, 
+		       const IntegralInfo & integralInfo)
 {
   int n_psi2, i, j;
   ergo_real sum, currIntegral;
@@ -76,8 +97,8 @@ do_2e_integral(int mu,
   DistributionSpecStruct list_psi2[maxCount];
 
   /* form product of basisfuncs mu and nu, store product in psi1 */
-  int n_psi1 = get_product_simple_primitives(basisInfo, mu,
-					     basisInfo, nu,
+  int n_psi1 = get_product_simple_primitives(basisInfo_mu, mu,
+					     basisInfo_nu, nu,
 					     list_psi1,
 					     maxCount,
 					     0);
@@ -89,8 +110,8 @@ do_2e_integral(int mu,
     }
 
   /* form product of basisfuncs la and si, store product in psi2 */
-  n_psi2 = get_product_simple_primitives(basisInfo, la,
-					 basisInfo, si,
+  n_psi2 = get_product_simple_primitives(basisInfo_la, la,
+					 basisInfo_si, si,
 					 list_psi2,
 					 maxCount,
 					 0);
