@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,16 @@ ergo_real do_2e_integral(int mu,
 			 const BasisInfoStruct & basisInfo, 
 			 const IntegralInfo & integralInfo);
 
+ergo_real do_2e_integral_general(int mu, 
+				 int nu, 
+				 int la, 
+				 int si, 
+				 const BasisInfoStruct & basisInfo_mu, 
+				 const BasisInfoStruct & basisInfo_nu, 
+				 const BasisInfoStruct & basisInfo_la, 
+				 const BasisInfoStruct & basisInfo_si, 
+				 const IntegralInfo & integralInfo);
+
 int compute_2e_matrix_list_explicit(const BasisInfoStruct & basisInfo,
 				    const IntegralInfo & integralInfo,
 				    ergo_real** resultList,
