@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -92,8 +92,9 @@ MMLimitTable::~MMLimitTable()
 {
 }
 
-void MMLimitTable::init(ergo_real maxDistance)
+void MMLimitTable::init(ergo_real maxDistance_input)
 {
+  ergo_real maxDistance = maxDistance_input + 0.1; // ELIAS NOTE 2013-11-29: add small value here to avoid problems at exactly the maxDistance_input distance.
   init_multipole_code();
   ergo_real r = 0;
   ergo_real currStep = INITIAL_STEP;
@@ -201,7 +202,7 @@ void MMLimitTable::init(ergo_real maxDistance)
 const interaction_matrix_limit_struct & MMLimitTable::get_x_from_distance(ergo_real distance) const
 {
   int rangeIndex = 0;
-  while(rangeList[rangeIndex].maxDistance < distance)
+  while(rangeList[rangeIndex].maxDistance <= distance) // ELIAS NOTE 2013-11-29: changed from "<" to "<=" here to fix problem found by Anastasia, for case when (distanceLeft / range.step) divides evenly.
     {
       rangeIndex++;
       if(rangeIndex >= noOfRangesUsed)
