@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -211,7 +211,7 @@ MMTranslator::getTranslationMatrix(ergo_real dx,
                                    ergo_real dz,
                                    int l_1,
                                    int l_2,
-                                   ergo_real* result_W)
+                                   ergo_real* result_W) const
 {
   ergo_real r2 = dx*dx + dy*dy + dz*dz;
 
