@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "organize_distrs.h"
 #include "pi.h"
 
+#include <cstdio>
 
 
 static void
@@ -51,6 +52,44 @@ do_sort_int_list(int* list, int n)
 
 
 
+static void get_conversion_matrix_for_group(
+				    const IntegralInfo & integralInfo,
+				    const distr_group_struct & group,
+				    int n1max,
+				    const minimal_distr_struct* minimalDistrList_1,
+				    int noOfBasisFuncPairs_1, 
+				    const i_j_val_struct* convMat1_sp,
+				    int convMat1_nnz,
+				    i_j_val_struct* BB1_x_Ai1_x_convMat1_sp_result, // result
+				    int & BB1_x_Ai1_x_convMat1_nnz_result)  // result
+{
+  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1max];
+  int distrCount_i = group.distrCount;
+  i_j_val_struct Ai1_sp[distrCount_i];
+  for(int i = 0; i < distrCount_i; i++) {
+    int monomialIndex = minimalDistrList_1[group.startIndex+i].monomialIndex;
+    ergo_real value = minimalDistrList_1[group.startIndex+i].coeff;
+    Ai1_sp[i].i = i;
+    Ai1_sp[i].j = monomialIndex;
+    Ai1_sp[i].value = value;
+  }
+  i_j_val_struct BB1_sp[distrCount_i];
+  for(int kk = 0; kk < distrCount_i; kk++) {
+    int idx = minimalDistrList_1[kk+group.startIndex].basisFuncPairIndex;
+    BB1_sp[kk].i = idx;
+    BB1_sp[kk].j = kk;
+    BB1_sp[kk].value = 1;
+  }
+  // Multiply Ai1 by convMat1. Dimensions: (distrCount_i*noOfMonomials_1) x (noOfMonomials_1*noOfMonomials_1)
+  i_j_val_struct* Ai1_x_convMat1_sp = new i_j_val_struct[distrCount_i*noOfMonomials_1];
+  int Ai1_x_convMat1_nnz = spmat_multiply_matrices(Ai1_sp, distrCount_i, convMat1_sp, convMat1_nnz, Ai1_x_convMat1_sp, distrCount_i, noOfMonomials_1);
+  // Multiply BB1 by Ai1_x_convMat1. Dimensions: (noOfBasisFuncPairs_1*distrCount_i) x (distrCount_i*noOfMonomials_1)
+  BB1_x_Ai1_x_convMat1_nnz_result = spmat_multiply_matrices(BB1_sp, distrCount_i, Ai1_x_convMat1_sp, Ai1_x_convMat1_nnz, BB1_x_Ai1_x_convMat1_sp_result, noOfBasisFuncPairs_1, noOfMonomials_1);
+  delete [] Ai1_x_convMat1_sp;
+}
+
+
+
 int
 organize_distributions(const IntegralInfo & integralInfo,
 		       DistributionSpecStructLabeled* distrList_in, 
@@ -980,7 +1019,68 @@ organize_distributions(const IntegralInfo & integralInfo,
 	    result->basisFuncListForChunks_map[i] = k;
 	}
     }
-      
+
+
+  // take care of integral conversion matrices
+  int noOfBasisFuncPairs_max = 0;
+  for(int i = 0; i < chunkCount; i++) {
+    int noOfBasisFuncPairs = chunkList[i].noOfBasisFuncPairs;
+    if(noOfBasisFuncPairs > noOfBasisFuncPairs_max)
+      noOfBasisFuncPairs_max = noOfBasisFuncPairs;
+  }
+  int noOfMonomials_max = 0;
+  for(int i = 0; i < chunkCount; i++)
+    for(int j = chunkList[i].clusterStartIndex; j < chunkList[i].clusterStartIndex + chunkList[i].noOfClusters; j++) {
+      int noOfMonomials = integralInfo.monomial_info.no_of_monomials_list[clusterList[j].nmax];
+      if(noOfMonomials > noOfMonomials_max)
+	noOfMonomials_max = noOfMonomials;
+    }
+  // We do not know the final size of the spMatElementList yet. We give it some size to start with, and then increase the size when needed.
+  std::vector<i_j_val_struct> spMatElementList(5*noOfBasisFuncPairs_max*noOfMonomials_max);
+  int spMatElementListCount = 0;
+  result->spMatCountList.resize(groupCount);
+  result->spMatIdxList.resize(groupCount);
+  for(int i = 0; i < chunkCount; i++)
+    for(int j = chunkList[i].clusterStartIndex; j < chunkList[i].clusterStartIndex + chunkList[i].noOfClusters; j++) {
+      // Now we are dealing with cluster j
+      int noOfBasisFuncPairs = chunkList[i].noOfBasisFuncPairs;
+      int nmax = clusterList[j].nmax;
+      int noOfMonomials = integralInfo.monomial_info.no_of_monomials_list[nmax];
+      int group_k_start = clusterList[j].groupStartIndex;
+      int group_k_end = group_k_start + clusterList[j].noOfGroups;
+      // Get conversion matrices
+      ergo_real alpha = groupList[group_k_start].exponent;
+      i_j_val_struct convMat_sp[noOfMonomials*noOfMonomials];
+      int convMat_nnz = integralInfo.get_hermite_conversion_matrix_right_sparse(nmax, 1.0/alpha, convMat_sp);
+      for(int group_k = group_k_start; group_k < group_k_end; group_k++) {
+	i_j_val_struct BB1_x_Ai1_x_convMat1_sp[noOfBasisFuncPairs*noOfMonomials];
+	int BB1_x_Ai1_x_convMat1_nnz = 0;
+	get_conversion_matrix_for_group(integralInfo,
+					groupList[group_k],
+					nmax,
+					minimalDistrList,
+					noOfBasisFuncPairs, 
+					convMat_sp,
+					convMat_nnz,
+					BB1_x_Ai1_x_convMat1_sp, // result
+					BB1_x_Ai1_x_convMat1_nnz);  // result
+	spmat_sort_elements(BB1_x_Ai1_x_convMat1_sp, BB1_x_Ai1_x_convMat1_nnz);
+	// Check if the size of spMatElementList needs to be extended.
+	while((int)(spMatElementList.size()) < spMatElementListCount + BB1_x_Ai1_x_convMat1_nnz)
+	  spMatElementList.resize(2*spMatElementList.size());
+	memcpy(&spMatElementList[spMatElementListCount], BB1_x_Ai1_x_convMat1_sp, BB1_x_Ai1_x_convMat1_nnz*sizeof(i_j_val_struct));
+	result->spMatCountList[group_k] = BB1_x_Ai1_x_convMat1_nnz;
+	result->spMatIdxList[group_k] = spMatElementListCount;
+	spMatElementListCount += BB1_x_Ai1_x_convMat1_nnz;
+	if(spMatElementListCount > groupCount*noOfBasisFuncPairs_max*noOfMonomials_max)
+	  return -1;
+      }
+    }
+
+
+
+  result->spMatElementList.resize(spMatElementListCount);
+  memcpy(&result->spMatElementList[0], &spMatElementList[0], spMatElementListCount*sizeof(i_j_val_struct));
 
   result->clusterList.resize(clusterCount);
   memcpy(&result->clusterList[0], &clusterList[0], clusterCount*sizeof(cluster_struct));
