@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 #include "output.h"
 #include "multipole.h"
+#include "simple_sparse_mat.h"
 
 #include <vector>
 
@@ -108,6 +109,9 @@ struct distr_org_struct {
   std::vector<int> basisFuncListForChunks;
   std::vector<int> basisFuncListForChunks_map;
   std::vector<int> basisFuncList;
+  std::vector<i_j_val_struct> spMatElementList;
+  std::vector<int> spMatCountList;
+  std::vector<int> spMatIdxList;
   int minimalDistrCount;
   int groupCount;
   int clusterCount;
