@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
  */
 #ifndef MAT_MATRIXTRIDIAGSYMMETRIC
 #define MAT_MATRIXTRIDIAGSYMMETRIC
-#include "gblas.h"
+#include "mat_gblas.h"
 namespace mat { /* Matrix namespace */
   namespace arn { /* Arnoldi type methods namespace */
     /** Tridiagonal symmetric matrix class template
