@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -219,8 +219,7 @@ SCF_general::SCF_general(const Molecule& molecule_,
   invCholFactor.writeToFile();
   output_current_memory_usage(LOG_AREA_SCF, "after writing invCholFactor and S_symm to file");
 
-  H_core_Matrix.resetSizesAndBlocks(matOpts.size_block_info,
-				    matOpts.size_block_info);
+  H_core_Matrix.resetSizesAndBlocks(matOpts.size_block_info, matOpts.size_block_info);
   if(scfopts.skip_H_core == 1)
     {
       do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "NOTE: skip_H_core parameter set, will skip construction of H_core matrix!");
@@ -320,6 +319,14 @@ SCF_general::SCF_general(const Molecule& molecule_,
 
   CAM_params.computeRangeSeparatedExchange =
     CAM_params.beta != ergo_real(0.0);
+
+  energy_2el = 0;
+  energy = 0;
+  energy_2el_core = 0; // only used when "core density matrix" is used
+  energy_2el_valence = 0; // only used when "core density matrix" is used
+  energy_of_valence = 0; // only used when "core density matrix" is used
+  energy_reference = 0; // only used when "core density matrix" is used
+  electronicEntropyTerm = 0;
 }
 
 
@@ -412,6 +419,9 @@ void SCF_general::do_SCF_iterations()
     throw "exiting after scfopts.output_density_images_only";
   }
 
+  if(scfopts.use_stochastic_orbs == 1)
+    prepare_stochastic_orbitals();
+
   // Use at least two iterations if use_simple_starting_guess used.
   int min_number_of_iterations = scfopts.min_number_of_iterations;
   if(scfopts.use_simple_starting_guess && min_number_of_iterations < 2)
@@ -478,11 +488,17 @@ void SCF_general::do_SCF_iterations()
       
       do_output(LOG_CAT_INFO, LOG_AREA_SCF, "nuclearEnergy, energy_2el, energy = %f, %f, %f", 
 		(double)nuclearEnergy, (double)energy_2el, (double)energy);
+      // ELIAS NOTE 2014-01-01: FIXME: consider adding another output message here showing the "electronic energy" so that the difference between "electronic energy" and "total energy" becomes more clear.
       if( step > 2)
 	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Energy %3i = %22.11f  ( diff %22.15f )", step, (double)energy, (double)(energy-best_energy_so_far));
       else
 	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Energy %3i = %22.11f", step, (double)energy);
 
+      if(scfopts.compute_core_density == 1 && step > 1) {
+	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Valence-only energy %3i  = %22.11f", step, (double)energy_of_valence);
+	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Reference energy    %3i  = %22.11f", step, (double)energy_reference);
+      }
+
       output_current_memory_usage(LOG_AREA_SCF, "After computing energy");
 
       curr_cycle_stats->start_timer("get_FDSminusSDF");
