@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -90,6 +90,11 @@ class SCF_general
   ergo_real energy_2el;
   ergo_real energy;
 
+  ergo_real energy_2el_core; // only used when "core density matrix" is used
+  ergo_real energy_2el_valence; // only used when "core density matrix" is used
+  ergo_real energy_of_valence; // only used when "core density matrix" is used
+  ergo_real energy_reference; // only used when "core density matrix" is used
+
   ergo_real electronicEntropyTerm;
 
   ergo_real errorMeasure;
@@ -131,6 +136,7 @@ class SCF_general
   virtual void save_final_potential() = 0;
   virtual void add_random_disturbance_to_starting_guess() = 0;
   virtual void output_density_images() = 0;
+  virtual void prepare_stochastic_orbitals() = 0;
   virtual void output_csr_matrices_for_gao() = 0;
   virtual void write_diag_dens_to_file() = 0;
   virtual void report_final_results() = 0;
