@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,12 +33,13 @@
 #include "diis_restricted.h"
 #include "density_projection.h"
 #include "densfromf_general.h"
-#include "density_description_file_2.h"
+#include "density_description_file.h"
 #include "matrix_utilities.h"
 #include "matrix_utilities_gao.h"
 #include "machine_epsilon.h"
 #include "units.h"
 #include "atom_labels.h"
+#include "integral_matrix_wrappers.h"
 
 
 SCF_restricted::SCF_restricted(const Molecule& molecule_,
@@ -95,6 +96,10 @@ void SCF_restricted::initialize_matrices()
   densityMatrix.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
 
+  densityMatrix_core.resetSizesAndBlocks(matOpts.size_block_info, matOpts.size_block_info);
+
+  twoel_matrix_core.resetSizesAndBlocks(matOpts.size_block_info, matOpts.size_block_info);
+
   FockMatrix.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
   Fprev.resetSizesAndBlocks(matOpts.size_block_info,
@@ -221,6 +226,8 @@ void SCF_restricted::get_starting_guess_density()
 					scfopts.purification_create_m_files,
 					scfopts.purification_ignore_failure,
 					scfopts.purification_use_rand_perturbation_for_alleigsint,
+					scfopts.use_stochastic_orbs,
+					stochastic_orbitals,
 					"",
 					puri_stats,
 					scfopts.do_sparsity_investigation,
@@ -236,6 +243,8 @@ void SCF_restricted::get_starting_guess_density()
   densityMatrix.readFromFile();
   output_sparsity_symm(n, densityMatrix, "starting guess density matrix");
   densityMatrix.writeToFile();
+
+  densityMatrix_core.writeToFile(); // densityMatrix_core (if used) also needs to be written to file
 }
 
 
@@ -297,6 +306,7 @@ static void output_diff_norm_values(symmMatrix const & F1,
 void SCF_restricted::get_2e_part_and_energy()
 {
   densityMatrix.readFromFile();
+  densityMatrix_core.readFromFile();
 
   bool scan_do_invcholfactor_transf = (bool)scfopts.scan_do_invcholfactor_transf;
 
@@ -372,7 +382,81 @@ void SCF_restricted::get_2e_part_and_energy()
       do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_2e_matrix_and_energy_sparse");
       throw "error in get_2e_matrix_and_energy_sparse";
     }
+
+  if(scfopts.compute_core_density == 1) {
+    // Get energy_2el_valence and H_matrix_core, but only if densityMatrix_core is nonzero (it is zero the first time, then we just skip).
+    if(densityMatrix_core.frob() != 0) {
+      // Get energy_2el_valence
+      {
+	symmMatrix densityMatrix_valence(densityMatrix);
+	densityMatrix_valence += (ergo_real)-1 * densityMatrix_core;
+	symmMatrix G_valence;
+	G_valence.resetSizesAndBlocks(matOpts.size_block_info, matOpts.size_block_info);  
+	symmMatrix J_matrix_valence;
+	symmMatrix K_matrix_valence;
+	symmMatrix Fxc_matrix_valence;
+	SCF_statistics curr_cycle_stats_dummy;
+	int no_of_valence_electrons = noOfElectrons - scfopts.no_of_core_electrons;
+	if(get_2e_matrix_and_energy_sparse(basisInfo, 
+					   basisInfoDensFit, 
+					   molecule,
+					   integralInfo, 
+					   G_valence,
+					   densityMatrix_valence, 
+					   J_K_params,
+					   CAM_params,
+					   gridParams,
+					   scfopts.use_dft,
+					   &energy_2el_valence, no_of_valence_electrons,
+					   densfit_data,
+					   matOpts.size_block_info,
+					   matOpts.permutationHML,
+					   matOpts.inversePermutationHML,
+					   scfopts.do_sparsity_investigation,
+					   J_matrix_valence,
+					   K_matrix_valence,
+					   Fxc_matrix_valence,
+					   curr_cycle_stats_dummy) != 0) {
+	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "Error in get_2e_matrix_and_energy_sparse for densityMatrix_valence.");
+	  throw "Error in get_2e_matrix_and_energy_sparse for densityMatrix_valence.";
+	}
+      }
+      // Compute J_matrix_core
+      {
+	symmMatrix J_matrix_core;
+	symmMatrix K_matrix_core;
+	symmMatrix Fxc_matrix_core;
+	SCF_statistics curr_cycle_stats_dummy;
+	energy_2el_core = 0;
+	if(get_2e_matrix_and_energy_sparse(basisInfo, 
+					   basisInfoDensFit, 
+					   molecule,
+					   integralInfo, 
+					   twoel_matrix_core,
+					   densityMatrix_core, 
+					   J_K_params,
+					   CAM_params,
+					   gridParams,
+					   scfopts.use_dft,
+					   &energy_2el_core, scfopts.no_of_core_electrons,
+					   densfit_data,
+					   matOpts.size_block_info,
+					   matOpts.permutationHML,
+					   matOpts.inversePermutationHML,
+					   scfopts.do_sparsity_investigation,
+					   J_matrix_core,
+					   K_matrix_core,
+					   Fxc_matrix_core,
+					   curr_cycle_stats_dummy) != 0) {
+	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "Error in get_2e_matrix_and_energy_sparse for densityMatrix_core.");
+	  throw "Error in get_2e_matrix_and_energy_sparse for densityMatrix_core.";
+	}
+      }
+    }
+  }
+
   densityMatrix.writeToFile();
+  densityMatrix_core.writeToFile();
 
   // Check that G matrix is free from "inf", "nan" etc.
   if(check_if_matrix_contains_strange_elements(G, matOpts.inversePermutationHML))
@@ -599,6 +683,14 @@ void SCF_restricted::calculate_energy()
   H_core_Matrix.readFromFile();
   densityMatrix.readFromFile();
   energy = symmMatrix::trace_ab(densityMatrix, H_core_Matrix) + energy_2el;
+  if(scfopts.compute_core_density == 1) {
+    densityMatrix_core.readFromFile();
+    symmMatrix densityMatrix_valence(densityMatrix);
+    densityMatrix_valence += (ergo_real)-1 * densityMatrix_core;
+    energy_of_valence = symmMatrix::trace_ab(densityMatrix_valence, H_core_Matrix) + symmMatrix::trace_ab(densityMatrix_valence, twoel_matrix_core) + energy_2el_valence;
+    energy_reference = symmMatrix::trace_ab(densityMatrix_core, H_core_Matrix) + energy_2el_core + nuclearEnergy;
+    densityMatrix_core.writeToFile();
+  }
   densityMatrix.writeToFile();
   H_core_Matrix.writeToFile();
   energy += nuclearEnergy;
@@ -755,6 +847,8 @@ void SCF_restricted::get_new_density_matrix()
 				    scfopts.purification_create_m_files,
 				    scfopts.purification_ignore_failure,
 				    scfopts.purification_use_rand_perturbation_for_alleigsint,
+				    scfopts.use_stochastic_orbs,
+				    stochastic_orbitals,
 				    "investigation_"+mat::getNormTypeString(ntype[i])+"_",
 				    puri_stats,
 				    scfopts.do_sparsity_investigation,
@@ -826,6 +920,8 @@ void SCF_restricted::get_new_density_matrix()
 				scfopts.purification_create_m_files,
 				scfopts.purification_ignore_failure,
 				scfopts.purification_use_rand_perturbation_for_alleigsint,
+				scfopts.use_stochastic_orbs,
+				stochastic_orbitals,
 				"",
 				puri_stats,
 				scfopts.do_sparsity_investigation,
@@ -839,6 +935,53 @@ void SCF_restricted::get_new_density_matrix()
     }
   this->curr_cycle_stats->add_values(puri_stats);
 
+  if(scfopts.compute_core_density == 1) {
+    std::map<std::string, double> puri_stats_unused;
+    intervalType homoInterval_dummy1(-1e22,1e22);
+    intervalType lumoInterval_dummy1(-1e22,1e22);
+    intervalType homoInterval_dummy2(-1e22,1e22);
+    intervalType lumoInterval_dummy2(-1e22,1e22);
+    if(get_dens_from_fock_general(n,
+				  scfopts.no_of_core_electrons / 2,
+				  scfopts.use_diagonalization,
+				  scfopts.use_diag_on_error,
+				  scfopts.electronic_temperature,
+				  densityMatrix_core,
+				  2,
+				  electronicEntropyTerm,
+				  *F_effective, // should normally be same as Fprev now
+				  homoInterval_dummy1, // output
+				  lumoInterval_dummy1, // output
+				  S_symm,
+				  invCholFactor,
+				  invCholFactor_euclnorm,
+				  scfopts.gap_expected_lower_bound,
+				  matOpts.size_block_info,
+				  F_ort_prev,
+				  homoInterval_dummy2,
+				  lumoInterval_dummy2,
+				  scfopts.purification_eigvalue_err_limit,
+				  scfopts.purification_subspace_err_limit,
+				  scfopts.purification_truncation_norm,
+				  scfopts.purification_maxmul,
+				  scfopts.purification_create_m_files,
+				  scfopts.purification_ignore_failure,
+				  scfopts.purification_use_rand_perturbation_for_alleigsint,
+				  scfopts.use_stochastic_orbs,
+				  stochastic_orbitals,
+				  "",
+				  puri_stats_unused,
+				  0,
+				  0,
+				  0,
+				  0,
+				  NULL,
+				  NULL) != 0)
+      {
+	throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock_general for core density matrix.";
+      }
+  }
+
   // Report sparsity of D and trace(DS)
   S_symm.readFromFile();
   densityMatrix.readFromFile();
@@ -849,35 +992,37 @@ void SCF_restricted::get_new_density_matrix()
 }
 
 
-void SCF_restricted::write_density_to_file()
-{
-  densityMatrix.readFromFile();
+static int write_matrix_to_file(symmMatrix & M, const std::vector<int> & inversePermutationHML, const BasisInfoStruct & basisInfo, const char* fileName) {
+  M.readFromFile();
   matrix_description_struct matrixList[2];
-  int nvalues = densityMatrix.nvalues();
+  int nvalues = M.nvalues();
   std::vector<int> rowind; 
   rowind.reserve(nvalues);
   std::vector<int> colind; 
   colind.reserve(nvalues);
   std::vector<ergo_real> values; 
   values.reserve(nvalues);
-  densityMatrix.get_all_values(rowind,
-			       colind,
-			       values,
-			       matOpts.inversePermutationHML,
-			       matOpts.inversePermutationHML);
-  densityMatrix.writeToFile();
+  M.get_all_values(rowind, colind, values, inversePermutationHML, inversePermutationHML);
+  M.writeToFile();
   matrixList[0].nvalues = nvalues;
   matrixList[0].rowind = &rowind[0];
   matrixList[0].colind = &colind[0];
   matrixList[0].values = &values[0];
-  if(ddf_writeShellListAndDensityMatricesToFile_sparse(&basisInfo,
-						       1,
-						       matrixList,
-						       "density.bin") != 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ddf_writeShellListAndDensityMatricesToFile_sparse");
-      throw "error in ddf_writeShellListAndDensityMatricesToFile_sparse";
-    }
+  if(ddf_writeShellListAndDensityMatricesToFile_sparse(basisInfo, 1, matrixList, fileName) != 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "Error in ddf_writeShellListAndDensityMatricesToFile_sparse.");
+    return -1;
+  }
+  return 0;
+}
+
+void SCF_restricted::write_density_to_file()
+{
+  if(write_matrix_to_file(densityMatrix, matOpts.inversePermutationHML, basisInfo, "density.bin") != 0)
+    throw "error in SCF_restricted::write_density_to_file(): write_matrix_to_file failed for densityMatrix.";
+  if(scfopts.compute_core_density == 1) {
+    if(write_matrix_to_file(densityMatrix_core, matOpts.inversePermutationHML, basisInfo, "density_core.bin") != 0)
+      throw "error in SCF_restricted::write_density_to_file(): write_matrix_to_file failed for densityMatrix_core.";
+  }
 }
 
 
@@ -950,6 +1095,221 @@ void SCF_restricted::output_density_images()
 }
 
 
+static ergo_real rand_minus1_to_1(int rand_param) {
+  if(rand_param == 0) {
+    // ELIAS NOTE 2013-11-14: this is the old bad implementation that gives a slightly "unfair" distribution in the [-1,1] interval.
+    ergo_real randomNumber = (ergo_real)rand() / RAND_MAX;
+    // Now randomNumber is between 0 and 1
+    randomNumber *= 2;
+    // Now randomNumber is between 0 and 2
+    randomNumber -= 1;
+    // Now randomNumber is between -1 and 1
+    return randomNumber;
+  }
+  else if(rand_param == 1) {
+    ergo_real h = (ergo_real)1.0 / ((ergo_real)RAND_MAX+1);
+    ergo_real randomNumber = (ergo_real)rand() / ((ergo_real)RAND_MAX+1);
+    // Now randomNumber is between 0 and 1-h, and the spacing between possible values is h.
+    randomNumber *= 2;
+    // Now randomNumber is between 0 and 2-2*h, and the spacing between possible values is 2*h.
+    randomNumber -= 1;
+    // Now randomNumber is between -1 and 1-2*h, and the spacing between possible values is 2*h.
+    randomNumber += h;
+    // Now randomNumber is between -1+h and 1-h, and the spacing between possible values is 2*h.
+    return randomNumber;
+  }
+  else if(rand_param == 2) {
+    ergo_real randomNumber_a = ((ergo_real)RAND_MAX+1)*(ergo_real)rand() + (ergo_real)rand();
+    ergo_real randomNumber_a_max = ((ergo_real)RAND_MAX+1)*((ergo_real)RAND_MAX+1) - 1;
+    // Now randomNumber_a is between 0 and randomNumber_a_max, with spacing 1 between possible values.
+    ergo_real h = (ergo_real)1.0 / (randomNumber_a_max+1);
+    ergo_real randomNumber = randomNumber_a / (randomNumber_a_max+1);
+    // Now randomNumber is between 0 and 1-h, and the spacing between possible values is h.
+    randomNumber *= 2;
+    // Now randomNumber is between 0 and 2-2*h, and the spacing between possible values is 2*h.
+    randomNumber -= 1;
+    // Now randomNumber is between -1 and 1-2*h, and the spacing between possible values is 2*h.
+    randomNumber += h;
+    // Now randomNumber is between -1+h and 1-h, and the spacing between possible values is 2*h.
+    return randomNumber;
+  }
+  else
+    throw "Error in rand_minus1_to_1: unsupported rand_param value.";
+}
+
+
+static ergo_real get_random_vector_with_norm_leq_one_recursive(std::vector<ergo_real> & v, int n, int rand_param) {
+  if(n == 1) {
+    v.resize(1);
+    v[0] = rand_minus1_to_1(rand_param);
+    ergo_real norm = std::sqrt(v[0]*v[0]);
+    return norm;
+  }
+  while(1) {
+    // Consider two parts.
+    int n1 = n - n/2;
+    int n2 = n - n1;
+    std::vector<ergo_real> v1;
+    std::vector<ergo_real> v2;
+    ergo_real norm1 = get_random_vector_with_norm_leq_one_recursive(v1, n1, rand_param);
+    ergo_real norm2 = get_random_vector_with_norm_leq_one_recursive(v2, n2, rand_param);
+    ergo_real norm = std::sqrt(norm1*norm1+norm2*norm2);
+    if(norm <= 1) {
+      v.resize(n);
+      for(int i = 0; i < n1; i++)
+	v[i] = v1[i];
+      for(int i = 0; i < n2; i++)
+	v[n1+i] = v2[i];
+      // Done!
+      return norm;
+    }
+  }
+}
+
+static void get_normalized_random_vector_2(std::vector<ergo_real> & v, int n, int rand_param) {
+  ergo_real norm = get_random_vector_with_norm_leq_one_recursive(v, n, rand_param);
+  for(int i = 0; i < n; i++)
+    v[i] /= norm;
+}
+
+// ELIAS NOTE 2013-11-13: this routine seems wrong, does not seem to give a "fair" distribution of vectors.
+static void get_normalized_random_vector_recursive(std::vector<ergo_real> & v, int n, int rand_param) {
+  if(n == 1) {
+    v.resize(1);
+    if(rand() % 2 == 0)
+      v[0] = 1;
+    else
+      v[0] = -1;
+    return;
+  }
+  // Consider two parts.
+  int n1 = n - n/2;
+  int n2 = n - n1;
+  std::vector<ergo_real> v1;
+  std::vector<ergo_real> v2;
+  get_normalized_random_vector_recursive(v1, n1, rand_param);
+  get_normalized_random_vector_recursive(v2, n2, rand_param);
+  ergo_real x1 = 0;
+  ergo_real x2 = 0;
+  while(1) {
+    x1 = rand_minus1_to_1(rand_param);
+    x2 = rand_minus1_to_1(rand_param);
+    if(x1*x1+x2*x2 <= 1)
+      break;
+  }
+  // OK, now we have x1 and x2 selected randomly in a "fair" way.
+  ergo_real norm = std::sqrt(x1*x1+x2*x2);
+  x1 /= norm;
+  x2 /= norm;
+  // OK, now the coordinate pair (x1,x2) points in a random direction in the 2D space.
+  v.resize(n);
+  for(int i = 0; i < n1; i++)
+    v[i] = x1 * v1[i];
+  for(int i = 0; i < n2; i++)
+    v[n1+i] = x2 * v2[i];
+  // Done!
+}
+
+/* ELIAS NOTE 2014-07-13: Commented out this unused routine to silence compiler warning.
+static void get_normalized_random_vector_naive(std::vector<ergo_real> & v, int n, int rand_param) {
+  v.resize(n);
+  ergo_real sqSum = 0;
+  while(1) {
+    sqSum = 0;
+    for(int k = 0; k < n; k++) {
+      ergo_real x = rand_minus1_to_1(rand_param);
+      v[k] = x;
+      sqSum += x*x;
+    }
+    if(sqSum <= 1)
+      break;
+  } // end while
+  // Normalize vector
+  ergo_real norm = std::sqrt(sqSum);
+  for(int k = 0; k < n; k++)
+    v[k] /= norm;
+}
+*/
+
+/* ELIAS NOTE 2014-07-13: Commented out this unused routine to silence compiler warning.
+static void do_random_vector_tests(int rand_param) {
+  int N = 200000;
+  int n = 3;
+  ergo_real limit = 0.95;
+  ergo_real probability_naive_min = 1;
+  ergo_real probability_naive_max = 0;
+  ergo_real probability_recursive_min = 1;
+  ergo_real probability_recursive_max = 0;
+  ergo_real probability_new_min = 1;
+  ergo_real probability_new_max = 0;
+  for(int outerLoop = 0; outerLoop < 20; outerLoop++) {
+    int first_component_larger_than_limit_count_naive = 0;
+    int first_component_larger_than_limit_count_recursive = 0;
+    int first_component_larger_than_limit_count_new = 0;
+    for(int i = 0; i < N; i++) {
+      std::vector<ergo_real> v_naive;
+      get_normalized_random_vector_naive(v_naive, n, rand_param);
+      if(v_naive[0] > limit)
+	first_component_larger_than_limit_count_naive++;
+      std::vector<ergo_real> v_recursive;
+      get_normalized_random_vector_recursive(v_recursive, n, rand_param);
+      if(v_recursive[0] > limit)
+	first_component_larger_than_limit_count_recursive++;
+      std::vector<ergo_real> v_new;
+      get_normalized_random_vector_2(v_new, n, rand_param);
+      if(v_new[0] > limit)
+	first_component_larger_than_limit_count_new++;
+    }
+    ergo_real probability_naive = (ergo_real)first_component_larger_than_limit_count_naive / N;
+    ergo_real probability_recursive = (ergo_real)first_component_larger_than_limit_count_recursive / N;
+    ergo_real probability_new = (ergo_real)first_component_larger_than_limit_count_new / N;
+    if(probability_naive > probability_naive_max)
+      probability_naive_max = probability_naive;
+    if(probability_naive < probability_naive_min)
+      probability_naive_min = probability_naive;
+    if(probability_recursive > probability_recursive_max)
+      probability_recursive_max = probability_recursive;
+    if(probability_recursive < probability_recursive_min)
+      probability_recursive_min = probability_recursive;
+    if(probability_new > probability_new_max)
+      probability_new_max = probability_new;
+    if(probability_new < probability_new_min)
+      probability_new_min = probability_new;
+  }
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_random_vector_tests(), probability_naive     min max = %12.6f %12.6f.", probability_naive_min,     probability_naive_max    );
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_random_vector_tests(), probability_recursive min max = %12.6f %12.6f.", probability_recursive_min, probability_recursive_max);
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_random_vector_tests(), probability_new       min max = %12.6f %12.6f.", probability_new_min,       probability_new_max      );
+}
+*/
+
+void SCF_restricted::prepare_stochastic_orbitals() {
+  //  do_random_vector_tests();
+  Util::TimeMeter timeMeter;
+  int n = basisInfo.noOfBasisFuncs;
+  int nOrbs = scfopts.stochastic_orbs_no_of_vectors;
+  stochastic_orbitals.resize(nOrbs);
+  if(scfopts.stochastic_orbs_use_unit_vectors == 1) {
+    // In this case we simply set the "stochastic orbitals" to unit vectors, and continue in a round-robin way if nOrbs>n.
+    for(int i = 0; i < nOrbs; i++) {
+      stochastic_orbitals[i].resize(n);
+      for(int k = 0; k < n; k++)
+	stochastic_orbitals[i][k] = 0;
+      stochastic_orbitals[i][i%n] = 1;
+    }
+  }
+  else {
+    for(int i = 0; i < nOrbs; i++) {
+#if 1
+      get_normalized_random_vector_2(stochastic_orbitals[i], n, scfopts.stochastic_orbs_rand_param);
+#else
+      get_normalized_random_vector_naive(stochastic_orbitals[i], n);
+#endif
+    }
+  }
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_restricted::prepare_stochastic_orbitals finished OK.");
+  timeMeter.print(LOG_AREA_SCF, "SCF_restricted::prepare_stochastic_orbitals");
+}
+
 
 void SCF_restricted::output_csr_matrices_for_gao()
 {
@@ -1440,6 +1800,8 @@ void SCF_restricted::disturb_dens_matrix_exact_try(const symmMatrix & randomMatr
 				scfopts.purification_create_m_files,
 				scfopts.purification_ignore_failure,
 				scfopts.purification_use_rand_perturbation_for_alleigsint,
+				scfopts.use_stochastic_orbs,
+				stochastic_orbitals,
 				"",
 				puri_stats,
 				scfopts.do_sparsity_investigation,
@@ -1563,6 +1925,8 @@ void SCF_restricted::disturb_fock_matrix(ergo_real subspaceError)
 				scfopts.purification_create_m_files,
 				scfopts.purification_ignore_failure,
 				scfopts.purification_use_rand_perturbation_for_alleigsint,
+				scfopts.use_stochastic_orbs,
+				stochastic_orbitals,
 				"",
 				puri_stats,
 				scfopts.do_sparsity_investigation,
@@ -1625,45 +1989,80 @@ SCF_restricted::compute_gradient_fixeddens() {
   // interaction term.
   densityMatrix.readFromFile();
   int nAtoms = molecule.getNoOfAtoms();
-  std::vector<ergo_real> result(nAtoms*3);
-  for(int i = 0; i < nAtoms; i++) {
-    for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
-      const ergo_real h = 1e-3;
-      Molecule moleculeTmp = molecule;
-      Atom atomTmp = molecule.getAtom(i);
-      atomTmp.coords[coordIdx] += h;
-      moleculeTmp.replaceAtom(i, atomTmp);
-      ergo_real E1 = get_nucl_energy_for_given_mol_and_dens(integralInfo,
-							    moleculeTmp,
-							    basisInfo,
-							    densityMatrix,
-							    threshold_integrals_1el,
-							    matOpts.size_block_info,
-							    matOpts.permutationHML);
-      moleculeTmp = molecule;
-      atomTmp = molecule.getAtom(i);
-      atomTmp.coords[coordIdx] -= h;
-      moleculeTmp.replaceAtom(i, atomTmp);
-      ergo_real E2 = get_nucl_energy_for_given_mol_and_dens(integralInfo,
-							    moleculeTmp,
-							    basisInfo,
-							    densityMatrix,
-							    threshold_integrals_1el,
-							    matOpts.size_block_info,
-							    matOpts.permutationHML);
-      ergo_real gradientComponent = (E1 - E2) / (2 * h);
-      result[i*3+coordIdx] = gradientComponent;
-    } // END FOR coordIdx
-  } // END FOR i
+  std::vector<ergo_real> gradient(nAtoms*3);
+  get_gradient_for_given_mol_and_dens(integralInfo,
+				      molecule,
+				      basisInfo,
+				      densityMatrix,
+				      threshold_integrals_1el,
+				      matOpts.size_block_info,
+				      matOpts.permutationHML,
+				      &gradient[0]);
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Gradient of energy with respect to nuclear positions, for fixed electron density:");
   for(int i = 0; i < nAtoms; i++) {
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Atom %6d: %22.11f %22.11f %22.11f", 
 	      i,
-	      result[i*3+0],
-	      result[i*3+1],
-	      result[i*3+2]);
+	      (double)gradient[i*3+0],
+	      (double)gradient[i*3+1],
+	      (double)gradient[i*3+2]);
   }
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "(End of gradient)");
+
+  if(scfopts.verify_gradient_fixeddens == 1) {
+    std::vector<ergo_real> gradient_for_verification(nAtoms*3);
+    for(int i = 0; i < nAtoms; i++) {
+      for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
+	const ergo_real h = 1e-3;
+	Molecule moleculeTmp = molecule;
+	Atom atomTmp = molecule.getAtom(i);
+	atomTmp.coords[coordIdx] += h;
+	moleculeTmp.replaceAtom(i, atomTmp);
+	ergo_real E1 = get_nucl_energy_for_given_mol_and_dens(integralInfo,
+							      moleculeTmp,
+							      basisInfo,
+							      densityMatrix,
+							      threshold_integrals_1el,
+							      matOpts.size_block_info,
+							      matOpts.permutationHML);
+	moleculeTmp = molecule;
+	atomTmp = molecule.getAtom(i);
+	atomTmp.coords[coordIdx] -= h;
+	moleculeTmp.replaceAtom(i, atomTmp);
+	ergo_real E2 = get_nucl_energy_for_given_mol_and_dens(integralInfo,
+							      moleculeTmp,
+							      basisInfo,
+							      densityMatrix,
+							      threshold_integrals_1el,
+							      matOpts.size_block_info,
+							      matOpts.permutationHML);
+	ergo_real gradientComponent = (E1 - E2) / (2 * h);
+	gradient_for_verification[i*3+coordIdx] = gradientComponent;
+      } // END FOR coordIdx
+    } // END FOR i
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Gradient of energy with respect to nuclear positions, for fixed electron density (for verification, computed using finite differences):");
+    for(int i = 0; i < nAtoms; i++) {
+      do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Atom %6d: %22.11f %22.11f %22.11f", 
+		i,
+		(double)gradient_for_verification[i*3+0],
+		(double)gradient_for_verification[i*3+1],
+		(double)gradient_for_verification[i*3+2]);
+    }
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "(End of gradient)");
+    ergo_real maxAbsDiff = -1;
+    int atomIdx_saved = -1;
+    int coordIdx_saved = -1;
+    for(int atomIdx = 0; atomIdx < nAtoms; atomIdx++)
+      for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
+	ergo_real absdiff = std::fabs(gradient[atomIdx*3+coordIdx] - gradient_for_verification[atomIdx*3+coordIdx]);
+	if(absdiff > maxAbsDiff) {
+	  maxAbsDiff = absdiff;
+	  atomIdx_saved = atomIdx;
+	  coordIdx_saved = coordIdx;
+	}
+      }
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "maxAbsDiff %22.11f = %9.4g, for atomIdx %d and coordIdx %d", maxAbsDiff, maxAbsDiff, atomIdx_saved, coordIdx_saved);
+  }
+
   densityMatrix.writeToFile();
 }
 
