@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,6 +77,7 @@ class SCF_restricted : public SCF_general
   void save_final_potential();
   void add_random_disturbance_to_starting_guess();
   void output_density_images();
+  void prepare_stochastic_orbitals();
   void output_csr_matrices_for_gao();
   void write_diag_dens_to_file();
   void report_final_results();
@@ -108,6 +109,8 @@ class SCF_restricted : public SCF_general
 				     symmMatrix & resultDensMatrix);
 
   symmMatrix densityMatrix;
+  symmMatrix densityMatrix_core;
+  symmMatrix twoel_matrix_core;
   symmMatrix FockMatrix;
   symmMatrix Fprev;
   symmMatrix Dprev;
@@ -123,12 +126,13 @@ class SCF_restricted : public SCF_general
   generalVector eigVecLUMO;
   generalVector eigVecHOMO;
 
-
   intervalType homoInterval_F_ort_prev;
   intervalType lumoInterval_F_ort_prev;
   intervalType homoInterval_Fprev;
   intervalType lumoInterval_Fprev;
 
+  std::vector< std::vector<ergo_real> > stochastic_orbitals;
+
 };
 
 
