@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 #include "densfromf_sparse.h"
 #include "densfromf_full.h"
 #include "densfromf_general.h"
-#include "density_description_file_2.h"
+#include "density_description_file.h"
 #include "densitymanager.h"
 #include "matrix_utilities.h"
 #include "units.h"
@@ -231,6 +231,7 @@ void SCF_unrestricted::get_starting_guess_density()
 	  densityMatrix_alpha.writeToFile();
 	  std::map<std::string, double> puri_stats_alpha;
 	  ergo_real electronicEntropyTermDummy = 0;
+	  std::vector< std::vector<ergo_real> > stochasticOrbsDummy;
 	  if(get_dens_from_fock_general(n,
 					noOfElectrons_alpha,
 					scfopts.use_diagonalization,
@@ -257,6 +258,8 @@ void SCF_unrestricted::get_starting_guess_density()
 					scfopts.purification_create_m_files,
 					scfopts.purification_ignore_failure,
 					scfopts.purification_use_rand_perturbation_for_alleigsint,
+					0,
+					stochasticOrbsDummy,
 					"",
 					puri_stats_alpha,
 					scfopts.do_sparsity_investigation,
@@ -304,6 +307,8 @@ void SCF_unrestricted::get_starting_guess_density()
 					scfopts.purification_create_m_files,
 					scfopts.purification_ignore_failure,
 					scfopts.purification_use_rand_perturbation_for_alleigsint,
+					0,
+					stochasticOrbsDummy,
 					"",
 					puri_stats_beta,
 					scfopts.do_sparsity_investigation,
@@ -813,6 +818,7 @@ void SCF_unrestricted::get_new_density_matrix()
   // ALPHA
   std::map<std::string, double> puri_stats_alpha;
   ergo_real electronicEntropyTerm_alpha = 0;
+  std::vector< std::vector<ergo_real> > stochasticOrbsDummy;
   if(get_dens_from_fock_general(n,
 				noOfElectrons_alpha, 
 				scfopts.use_diagonalization,
@@ -839,6 +845,8 @@ void SCF_unrestricted::get_new_density_matrix()
 				scfopts.purification_create_m_files,
 				scfopts.purification_ignore_failure,
 				scfopts.purification_use_rand_perturbation_for_alleigsint,
+				0,
+				stochasticOrbsDummy,
 				"",
 				puri_stats_alpha,
 				scfopts.do_sparsity_investigation,
@@ -878,6 +886,8 @@ void SCF_unrestricted::get_new_density_matrix()
 				scfopts.purification_create_m_files,
 				scfopts.purification_ignore_failure,
 				scfopts.purification_use_rand_perturbation_for_alleigsint,
+				0,
+				stochasticOrbsDummy,
 				"",
 				puri_stats_beta,
 				scfopts.do_sparsity_investigation,
@@ -955,7 +965,7 @@ void SCF_unrestricted::write_density_to_file()
   matrixList[1].colind = &colind_beta[0];
   matrixList[1].values = &values_beta[0];
 
-  if(ddf_writeShellListAndDensityMatricesToFile_sparse(&basisInfo,
+  if(ddf_writeShellListAndDensityMatricesToFile_sparse(basisInfo,
 						       2,
 						       matrixList,
 						       "density.bin") != 0)
@@ -1019,6 +1029,10 @@ void SCF_unrestricted::output_density_images()
 }
 
 
+void SCF_unrestricted::prepare_stochastic_orbitals() {
+  throw "Error: SCF_unrestricted::prepare_stochastic_orbitals() not implemented.";
+}
+
 
 void SCF_unrestricted::do_spin_flip(int atomCount)
 {
