@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -101,6 +101,8 @@ struct Options {
   int output_density_images;
   int output_density_images_only;
   int write_guess_density_only;
+  int compute_core_density;
+  int no_of_core_electrons;
   ergo_real output_density_images_boxwidth;
   int image_view_axis;
   int save_final_potential;
@@ -127,6 +129,13 @@ struct Options {
   int output_homo_and_lumo_eigenvectors;
   int output_mulliken_pop;
   int compute_gradient_fixeddens;
+  int verify_gradient_fixeddens;
+
+  int use_stochastic_orbs;
+  int stochastic_orbs_no_of_vectors;
+  int stochastic_orbs_use_unit_vectors;
+  int stochastic_orbs_rand_param;
+
   /** Initializes all the fields to sane values. */
   Options() : calculation_identifier("N/A"),
        method_and_basis_set("N/A"),
@@ -187,6 +196,8 @@ struct Options {
        output_density_images(0),
        output_density_images_only(0),
        write_guess_density_only(0),
+       compute_core_density(0),
+       no_of_core_electrons(0),
        output_density_images_boxwidth(0.5),
        image_view_axis(),
        save_final_potential(0),
@@ -212,7 +223,12 @@ struct Options {
        create_basis_func_coord_file(0),
        output_homo_and_lumo_eigenvectors(0),
        output_mulliken_pop(0),
-       compute_gradient_fixeddens(0)
+       compute_gradient_fixeddens(0),
+       verify_gradient_fixeddens(0),
+       use_stochastic_orbs(0),
+       stochastic_orbs_no_of_vectors(100),
+       stochastic_orbs_use_unit_vectors(0),
+       stochastic_orbs_rand_param(1)
   { 
     memset(disturbedElementIndexVector, 0,
            sizeof(disturbedElementIndexVector));
