@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@
 #include "integrals_2el_boxed.h"
 #include "integrals_2el_explicit.h"
 #include "integrals_2el_layer.h"
-#include "density_description_file_2.h"
+#include "density_description_file.h"
 #include "mat_acc_extrapolate.h"
 #include "basis_func_pair_list_1el.h"
 #include "integral_matrix_wrappers.h"
@@ -455,73 +455,6 @@ assign_from_full_via_vectors_symm(int n,
 
 
 
-static int
-get_T(const IntegralInfo& integralInfo,
-      const BasisInfoStruct& basisInfo,
-      symmMatrix & T_symm,
-      ergo_real threshold,
-      std::vector<int> const & permutationHML)
-{
-  int n = basisInfo.noOfBasisFuncs;
-
-  int* nvaluesList = ergo_new(n, int);
-  int** colindList = ergo_new(n, int*);
-  ergo_real** valuesList = ergo_new(n, ergo_real*);
-  
-  if(compute_T_matrix_sparse(basisInfo,
-			     threshold,
-			     n,
-			     nvaluesList,
-			     colindList,
-			     valuesList) != 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_T_matrix_sparse");
-      return -1;
-    }
-  
-  // Now convert result to three vectors so prepare for HML storage.
-  int nvalues = 0;
-  for(int i = 0; i < n; i++)
-    nvalues += nvaluesList[i];
-  // allocate vectors
-  std::vector<int> rowind(nvalues);
-  std::vector<int> colind(nvalues);
-  std::vector<ergo_real> values(nvalues);
-  // populate vectors
-  int count = 0;
-  for(int i = 0; i < n; i++)
-    {
-      for(int j = 0; j < nvaluesList[i]; j++)
-	{
-	  rowind[count] = i;
-	  colind[count] = colindList[i][j];
-	  values[count] = valuesList[i][j];
-	  count++;
-	} // END FOR j
-    } // END FOR i
-
-  // Now the information is in rowind colind values.
-  // free memory allocated by compute_operator_matrix_sparse.
-  for(int i = 0; i < n; i++)
-    {
-      ergo_free(colindList[i]);
-      ergo_free(valuesList[i]);
-    }
-  ergo_free(nvaluesList);
-  ergo_free(colindList);
-  ergo_free(valuesList);
-
-  T_symm.assign_from_sparse(rowind,
-			    colind,
-			    values,
-			    permutationHML,
-			    permutationHML);
-
-  return 0;
-}
-
-
-
 int
 compute_h_core_matrix_simple_dense(const IntegralInfo& integralInfo,
 				   const Molecule& molecule,
@@ -598,17 +531,17 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 
   symmMatrix T(H_core_Matrix_sparse);
   T.clear();
-  if(get_T(integralInfo,
-	   basisInfo,
-	   T,
-	   threshold_integrals_1el,
-	   permutationHML) != 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_T");
+
+  if(compute_T_sparse(basisInfo,
+		      integralInfo,
+		      threshold_integrals_1el,
+		      T,
+		      permutationHML) != 0) {
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_T_sparse");
       return -1;
     }
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_T returned OK.");
-  timeMeterT.print(LOG_AREA_SCF, "get_T");
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "compute_T_sparse returned OK.");
+  timeMeterT.print(LOG_AREA_SCF, "compute_T_sparse");
 
   // Get V
   Util::TimeMeter timeMeterV;
@@ -808,31 +741,28 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 }
 
 
-ergo_real
-get_electron_nuclear_attraction_energy(const IntegralInfo& integralInfo,
-				       const Molecule& molecule,
-				       const BasisInfoStruct& basisInfo,
-				       const symmMatrix & D,
-				       ergo_real threshold_integrals_1el,
-				       mat::SizesAndBlocks const & matrix_size_block_info,
-				       std::vector<int> const & permutationHML)
+int
+get_gradient_for_given_mol_and_dens(const IntegralInfo& integralInfo,
+				    const Molecule& molecule,
+				    const BasisInfoStruct& basisInfo,
+				    const symmMatrix & D,
+				    ergo_real threshold_integrals_1el,
+				    mat::SizesAndBlocks const & matrix_size_block_info,
+				    std::vector<int> const & permutationHML,
+				    ergo_real* result_gradient_list)
 {
-  // Get V
-  symmMatrix V;
-  V.resetSizesAndBlocks(matrix_size_block_info,
-			matrix_size_block_info);
-  ergo_real boxSize = 10.0;
-  if(compute_V_sparse(basisInfo,
-		      integralInfo, 
-		      molecule,
-		      threshold_integrals_1el,
-		      boxSize,
-		      V,
-		      permutationHML) != 0)
-    {
-      throw "error in compute_V_sparse";
-    }
-  return symmMatrix::trace_ab(D, V);
+  ergo_real boxSize = 1.0;
+  if(compute_gradient_of_nucl_and_trDV(basisInfo,
+				       integralInfo, 
+				       molecule,
+				       threshold_integrals_1el,
+				       boxSize,
+				       D,
+				       permutationHML,
+				       result_gradient_list) != 0) {
+    throw "Error in compute_gradient_of_tr_D_V";
+  }
+  return 0;
 }
 
 
@@ -863,7 +793,7 @@ save_symmetric_matrix(symmMatrix& A, const BasisInfoStruct & basisInfo,
 		     inversePermutationHML);
   std::copy (fullTmp.begin(), fullTmp.end(), potentialMatrixFull);
   delete tmpMat;
-  int ret = ddf_writeShellListAndDensityMatricesToFile(&basisInfo,
+  int ret = ddf_writeShellListAndDensityMatricesToFile(basisInfo,
                                                        1,
                                                        &potentialMatrixFull,
                                                        fileName);
