@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -69,14 +69,15 @@ compute_h_core_matrix_simple_dense(const IntegralInfo& integralInfo,
 				   mat::SizesAndBlocks const & matrix_size_block_info,
 				   std::vector<int> const & permutationHML);
 
-ergo_real
-get_electron_nuclear_attraction_energy(const IntegralInfo& integralInfo,
-				       const Molecule& molecule,
-				       const BasisInfoStruct& basisInfo,
-				       const symmMatrix & D,
-				       ergo_real threshold_integrals_1el,
-				       mat::SizesAndBlocks const & matrix_size_block_info,
-				       std::vector<int> const & permutationHML);
+int
+get_gradient_for_given_mol_and_dens(const IntegralInfo& integralInfo,
+				    const Molecule& molecule,
+				    const BasisInfoStruct& basisInfo,
+				    const symmMatrix & D,
+				    ergo_real threshold_integrals_1el,
+				    mat::SizesAndBlocks const & matrix_size_block_info,
+				    std::vector<int> const & permutationHML,
+				    ergo_real* result_gradient_list);
 
 int save_symmetric_matrix(symmMatrix& A, 
                           const BasisInfoStruct & basisInfo,
