@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ Example usage is:
 #include "Matrix.h"
 #include "SCF_restricted.h"
 #include "basisinfo.h"
-#include "density_description_file_2.h"
+#include "density_description_file.h"
 #include "dft_common.h"
 #include "integral_info.h"
 #include "integrals_2el.h"
@@ -80,12 +80,12 @@ main(int argc, char *argv[])
   const char *basisFileName = argv[2];
 
 
-  std::auto_ptr<IntegralInfo> integralInfo(new IntegralInfo(true));
+  IntegralInfo integralInfo(true);
   BasisInfoStruct basisInfo;
   if(basisInfo.addBasisfuncsForMolecule(molecule, 
                                         basisFileName,
                                         0, NULL,
-                                        *integralInfo, 
+                                        integralInfo, 
                                         false,
                                         true, true) != 0) {
     fprintf(stderr, "Error in BasisInfoStruct::add_basisfuncs_for_molecule "
@@ -116,7 +116,7 @@ main(int argc, char *argv[])
 		       extraCharges,
                        basisInfo, 
                        basisInfoDensFit,
-                       *integralInfo,
+                       integralInfo,
                        NULL,
                        jkOptions,
 		       gridParams,
@@ -128,12 +128,12 @@ main(int argc, char *argv[])
     TDDFT::saveOverlap(basisInfo, f);
     TDDFT::saveDipole(basisInfo, f);
     TDDFT::saveKinetic(basisInfo, f);
-    TDDFT::savePotential(molecule, basisInfo, *integralInfo, f);
-    TDDFT::saveCoulomb(basisInfo, *integralInfo, f);
+    TDDFT::savePotential(molecule, basisInfo, integralInfo, f);
+    TDDFT::saveCoulomb(basisInfo, integralInfo, f);
     {
       ergo_real       *densMatrix_full = NULL;
       BasisInfoStruct *basis_read  = NULL;
-      if(ddf_load_density("density.bin", 1, *integralInfo,
+      if(ddf_load_density("density.bin", 1, integralInfo,
 			  &basis_read, &densMatrix_full) == 0) {
         TDDFT::saveXC(molecule, basisInfo, densMatrix_full, f);
 	size_t n = basis_read->noOfBasisFuncs;
