@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 
 int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
 
-  std::auto_ptr<Molecule> m(new Molecule); /* Don't allocate it on stack, it's too large. */
+  Molecule m;
 
   int nx = 2;
   int ny = 1;
@@ -62,11 +62,11 @@ int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
 	    charge = 6;
 	  //	  if(atomCount%9 == 0)
 	  //	    charge = 30;
-	  m->addAtom(charge, x, y, z);
+	  m.addAtom(charge, x, y, z);
 	  atomCount++;
 	}
 
-  ergo_real moleculeBaseCoord = m->getAtom(0).coords[0];
+  ergo_real moleculeBaseCoord = m.getAtom(0).coords[0];
 
   ergo_real machine_epsilon = std::numeric_limits<ergo_real>::epsilon();
 
@@ -78,20 +78,20 @@ int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
   int n = 0; // Will be set later.
   for(int caseIdx = 0; caseIdx < noOfCases; caseIdx++) {
     // Move first atom by small distance in x direction.
-    Atom atom = m->getAtom(0);
+    Atom atom = m.getAtom(0);
     atom.coords[0] = moleculeBaseCoord + caseIdx * h;
-    m->replaceAtom(0, atom);
+    m.replaceAtom(0, atom);
     // New scope here so that the BasisInfoStruct pointer is freed each time.
     {
-      std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct);
-      if(bis->addBasisfuncsForMolecule(*m, ERGO_SPREFIX "/basis/3-21G",
+      BasisInfoStruct bis;
+      if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/3-21G",
 				       0, NULL, integralInfo, 0, 0, 0) != 0) {
-	printf("bis->addBasisfuncsForMolecule failed.\n");
+	printf("bis.addBasisfuncsForMolecule failed.\n");
 	return -1;
       }
       // OK, now we have a basis set for the current version of the
       // molecule. Now compute integrals.
-      n = bis->noOfBasisFuncs;
+      n = bis.noOfBasisFuncs;
       integralList[caseIdx].resize(n);
       for(int i = 0; i < n; i++) {
 	integralList[caseIdx][i].resize(n);
@@ -100,7 +100,7 @@ int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
 	  for(int k = 0; k < n; k++) {
 	    integralList[caseIdx][i][j][k].resize(n);
 	    for(int l = k; l < n; l++) {
-	      integralList[caseIdx][i][j][k][l] = do_2e_integral(i, k, l, j, *bis, integralInfo);
+	      integralList[caseIdx][i][j][k][l] = do_2e_integral(i, k, l, j, bis, integralInfo);
 	    }
 	  }
 	}
@@ -149,14 +149,14 @@ int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
 
 
 int main(int argc, char *argv[]) {
-  std::auto_ptr<IntegralInfo> integralInfo(new IntegralInfo(true));
-  integralInfo->init();
+  IntegralInfo integralInfo(true);
+  integralInfo.init();
 
   ergo_real machine_epsilon = std::numeric_limits<ergo_real>::epsilon();
   ergo_real h = 10000*machine_epsilon;
   printf("machine_epsilon = %9.5g, using h = %9.5g\n", (double)machine_epsilon, (double)h);
 
-  if(try_integral_diffs(*integralInfo, h) != 0)
+  if(try_integral_diffs(integralInfo, h) != 0)
     return -1;
 
   return 0;
