@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -172,33 +172,33 @@ grid_test_scaling(const char *fName)
 {
   time_t tm; time(&tm);
 
-  std::auto_ptr<Molecule> mol(new Molecule);
-  int res = mol->setFromMoleculeFile(fName, 0, NULL);
+  Molecule mol;
+  int res = mol.setFromMoleculeFile(fName, 0, NULL);
 
   if(res != 0) {
     fprintf(stderr, "Loading molecule from %s failed.\n", fName);
     return false;
   }
 
-  std::auto_ptr<IntegralInfo> ii(new IntegralInfo(true));
-  std::auto_ptr<BasisInfoStruct> bisOrig(new BasisInfoStruct());
-  if(bisOrig->addBasisfuncsForMolecule(*mol, ERGO_SPREFIX "/basis/STO-1G",
-				       0, NULL, *ii, 0, 0, 0) != 0) {
+  IntegralInfo ii(true);
+  BasisInfoStruct bisOrig;
+  if(bisOrig.addBasisfuncsForMolecule(mol, ERGO_SPREFIX "/basis/STO-1G",
+				       0, NULL, ii, 0, 0, 0) != 0) {
     printf("bis->addBasisfuncsForMolecule failed.\n");
     throw "addBasisfuncs failed";
   }
   Dft::GridParams gridParams(1e-7, 6, 25);
   gridParams.radialGridScheme = Dft::GridParams::LMG;
   
-  int *shellMap = new int[bisOrig->noOfShells];
-  int *aoMap = new int[bisOrig->noOfBasisFuncs];
+  int *shellMap = new int[bisOrig.noOfShells];
+  int *aoMap = new int[bisOrig.noOfBasisFuncs];
 
-  Dft::setupShellMap(*bisOrig, shellMap, aoMap);
-  BasisInfoStruct * bisPermuted = bisOrig->permuteShells(shellMap, *ii);
+  Dft::setupShellMap(bisOrig, shellMap, aoMap);
+  BasisInfoStruct * bisPermuted = bisOrig.permuteShells(shellMap, ii);
   delete []shellMap;
   delete []aoMap;
 
-  ErgoMolInfo molInfo(*bisPermuted, *mol);
+  ErgoMolInfo molInfo(*bisPermuted, mol);
   /* The important structure of this test */
   Dft::SparsePattern pattern(*bisPermuted); 
 
@@ -226,20 +226,20 @@ grid_test_synchronisation()
   real weight[BATCH_LENGTH];
   Dft::GridParams gridParams(1e-5, 6, 7);
 
-  std::auto_ptr<Molecule> mol(new Molecule);
+  Molecule mol;
   for(int i = 0; i < NATOMS; i++)
-    mol->addAtom(Atoms[i].charge, Atoms[i].position[0], Atoms[i].position[1], Atoms[i].position[2]);
+    mol.addAtom(Atoms[i].charge, Atoms[i].position[0], Atoms[i].position[1], Atoms[i].position[2]);
 
-  std::auto_ptr<IntegralInfo> ii(new IntegralInfo(true));
-  std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct());
-  if(bis->addBasisfuncsForMolecule(*mol, ERGO_SPREFIX "/basis/STO-1G",
-				   0, NULL, *ii, 0, 0, 0) != 0) {
-    printf("bis->addBasisfuncsForMolecule failed.\n");
+  IntegralInfo ii(true);
+  BasisInfoStruct bis;
+  if(bis.addBasisfuncsForMolecule(mol, ERGO_SPREFIX "/basis/STO-1G",
+				   0, NULL, ii, 0, 0, 0) != 0) {
+    printf("bis.addBasisfuncsForMolecule failed.\n");
     throw "addBasisfuncs failed";
   }
   
   for(int i = 0; i < NREPEAT; i++) {
-    DftGridReader* g = grid_open_full(&MolInfo, gridParams, NULL, NULL, *bis);
+    DftGridReader* g = grid_open_full(&MolInfo, gridParams, NULL, NULL, bis);
     int np;
     unsigned cnt = 0;
     while ( (np = grid_getchunk_blocked(g, BATCH_LENGTH,
