@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -101,10 +101,10 @@ int main(int argc, char *argv[])
 	   <<mat::Params::getMatrixParallelLevel()<<"."<<std::endl;
 #endif
   
-  std::auto_ptr<IntegralInfo> biBasic(new IntegralInfo(true));
-  std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct);
+  IntegralInfo biBasic(true);
+  BasisInfoStruct bis;
 
-  static Molecule m; /* Don't allocate it on stack, it's too large. */
+  Molecule m;
 
   int nx, ny, nz;
   if(getenv("RUN_BENCHMARK")) 
@@ -141,17 +141,17 @@ int main(int argc, char *argv[])
 	  atomCount++;
 	}
 	
-  if(bis->addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/6-31Gss",
-                                   0, NULL, *biBasic, 0, 0, 0) != 0) {
-    printf("bis->addBasisfuncsForMolecule failed.\n");
+  if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/6-31Gss",
+                                   0, NULL, biBasic, 0, 0, 0) != 0) {
+    printf("bis.addBasisfuncsForMolecule failed.\n");
     return 1;
   }
 
   mat::SizesAndBlocks matrix_size_block_info =
-    prepareMatrixSizesAndBlocks(bis->noOfBasisFuncs,
+    prepareMatrixSizesAndBlocks(bis.noOfBasisFuncs,
 				20, 8, 8, 8);
-  std::vector<int> permutationHML(bis->noOfBasisFuncs);
-  getMatrixPermutation(*bis, 20, 8, 8, 8, permutationHML);
+  std::vector<int> permutationHML(bis.noOfBasisFuncs);
+  getMatrixPermutation(bis, 20, 8, 8, 8, permutationHML);
 
   symmMatrix D;
   D.resetSizesAndBlocks(matrix_size_block_info,
@@ -169,7 +169,7 @@ int main(int argc, char *argv[])
   {
     /* Add values to density matrix diagonal and one step 
        next to diagonal. */
-    const int nvalues1 = bis->noOfBasisFuncs;
+    const int nvalues1 = bis.noOfBasisFuncs;
     std::vector<int> idxrow(nvalues1);
     std::vector<int> idxcol(nvalues1);
     std::vector<ergo_real> values(nvalues1);
@@ -181,7 +181,7 @@ int main(int argc, char *argv[])
     D.add_values(idxrow, idxcol, values, 
 		 permutationHML,
 		 permutationHML);
-    const int nvalues2 = bis->noOfBasisFuncs-1;
+    const int nvalues2 = bis.noOfBasisFuncs-1;
     idxrow.resize(nvalues2); 
     idxcol.resize(nvalues2); 
     values.resize(nvalues2); 
@@ -200,7 +200,7 @@ int main(int argc, char *argv[])
 
 
 
-  Jworkertest worker(D, *biBasic, *bis, J_K_params, permutationHML);
+  Jworkertest worker(D, biBasic, bis, J_K_params, permutationHML);
   
 
   MatAccInvestigator<ergo_real, Jworkertest> 
@@ -234,14 +234,14 @@ int main(int argc, char *argv[])
   J_K_params.threshold = sqrt(EPS);
 
   J_K_params.fmm_box_size = 10;
-  if(compute_J_by_boxes_sparse(*bis, *biBasic, J_K_params, J_1, D) != 0)
+  if(compute_J_by_boxes_sparse(bis, biBasic, J_K_params, J_1, D) != 0)
     {
       printf("Error in compute_J_by_boxes_sparse\n");
       return -1;
     }
   
   J_K_params.fmm_box_size = 4;
-  if(compute_J_by_boxes_sparse(*bis, *biBasic, J_K_params, J_2, D) != 0)
+  if(compute_J_by_boxes_sparse(bis, biBasic, J_K_params, J_2, D) != 0)
     {
       printf("Error in compute_J_by_boxes_sparse\n");
       return -1;
