@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,10 +54,10 @@ int main(int argc, char *argv[])
 	   <<mat::Params::getMatrixParallelLevel()<<"."<<std::endl;
 #endif
 
-  std::auto_ptr<IntegralInfo> integralInfo(new IntegralInfo(true));
-  std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct);
+  IntegralInfo integralInfo(true);
+  BasisInfoStruct bis;
 
-  static Molecule m; /* Don't allocate it on stack, it's too large. */
+  Molecule m;
 
   int nx, ny, nz;
   if(getenv("RUN_BENCHMARK")) 
@@ -94,18 +94,18 @@ int main(int argc, char *argv[])
 	  atomCount++;
 	}
 	
-  if(bis->addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/3-21G",
-                                   0, NULL, *integralInfo, 0, 0, 0) != 0) {
-    printf("bis->addBasisfuncsForMolecule failed.\n");
+  if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/3-21G",
+                                   0, NULL, integralInfo, 0, 0, 0) != 0) {
+    printf("bis.addBasisfuncsForMolecule failed.\n");
     return -1;
   }
 
   mat::SizesAndBlocks matrix_size_block_info =
-    prepareMatrixSizesAndBlocks(bis->noOfBasisFuncs,
+    prepareMatrixSizesAndBlocks(bis.noOfBasisFuncs,
 				20, 8, 8, 8);
-  std::vector<int> permutationHML(bis->noOfBasisFuncs);
-  std::vector<int> inversePermutationHML(bis->noOfBasisFuncs);
-  getMatrixPermutation(*bis, 20, 8, 8, 8, 
+  std::vector<int> permutationHML(bis.noOfBasisFuncs);
+  std::vector<int> inversePermutationHML(bis.noOfBasisFuncs);
+  getMatrixPermutation(bis, 20, 8, 8, 8, 
 		       permutationHML,
 		       inversePermutationHML);
 
@@ -119,7 +119,7 @@ int main(int argc, char *argv[])
 
   {
     /* Add values to density matrix. */
-    int n = bis->noOfBasisFuncs;
+    int n = bis.noOfBasisFuncs;
     /* Put values on diagonal and next to diagonal. */
     int nvalues = n + 2 * (n-1);
     std::vector<int> rowind(nvalues);
@@ -163,7 +163,7 @@ int main(int argc, char *argv[])
 
 
   J_K_params.exchange_box_size = 4;
-  if(compute_K_by_boxes_sparse_nosymm(*bis, *integralInfo, CAM_params_not_used, J_K_params, K_nosymm, D_nosymm,
+  if(compute_K_by_boxes_sparse_nosymm(bis, integralInfo, CAM_params_not_used, J_K_params, K_nosymm, D_nosymm,
 				      permutationHML,
 				      inversePermutationHML) != 0)
     {
@@ -200,7 +200,7 @@ int main(int argc, char *argv[])
 			    inversePermutationHML,
 			    inversePermutationHML);
     int nvalues_K = values_K.size();
-    int n = bis->noOfBasisFuncs;
+    int n = bis.noOfBasisFuncs;
     // Create full matrix version of D.
     std::vector<ergo_real> D_full(n*n);
     for(int ii = 0; ii < n*n; ii++)
@@ -226,7 +226,7 @@ int main(int argc, char *argv[])
 	ergo_real sum = 0;
 	for(int k = 0; k < n; k++)
 	  for(int l = 0; l < n; l++) {
-	    ergo_real integral_iklj = do_2e_integral(i, k, l, j, *bis, *integralInfo);
+	    ergo_real integral_iklj = do_2e_integral(i, k, l, j, bis, integralInfo);
 	    ergo_real contrib = (-0.5) * D_full[k*n+l] * integral_iklj;
 	    sum += contrib;
 	  }
