@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,10 +52,10 @@ int main(int argc, char *argv[])
 	   <<mat::Params::getMatrixParallelLevel()<<"."<<std::endl;
 #endif
 
-  std::auto_ptr<IntegralInfo> biBasic(new IntegralInfo(true));
-  std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct);
+  IntegralInfo biBasic(true);
+  BasisInfoStruct bis;
 
-  static Molecule m; /* Don't allocate it on stack, it's too large. */
+  Molecule m;
 
   int nx, ny, nz;
   if(getenv("RUN_BENCHMARK")) 
@@ -92,18 +92,18 @@ int main(int argc, char *argv[])
 	  atomCount++;
 	}
 	
-  if(bis->addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/6-31Gss",
-                                   0, NULL, *biBasic, 0, 0, 0) != 0) {
-    printf("bis->addBasisfuncsForMolecule failed.\n");
+  if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/6-31Gss",
+                                   0, NULL, biBasic, 0, 0, 0) != 0) {
+    printf("bis.addBasisfuncsForMolecule failed.\n");
     return 1;
   }
 
   mat::SizesAndBlocks matrix_size_block_info =
-    prepareMatrixSizesAndBlocks(bis->noOfBasisFuncs,
+    prepareMatrixSizesAndBlocks(bis.noOfBasisFuncs,
 				20, 8, 8, 8);
-  std::vector<int> permutationHML(bis->noOfBasisFuncs);
-  std::vector<int> inversePermutationHML(bis->noOfBasisFuncs);
-  getMatrixPermutation(*bis, 20, 8, 8, 8, 
+  std::vector<int> permutationHML(bis.noOfBasisFuncs);
+  std::vector<int> inversePermutationHML(bis.noOfBasisFuncs);
+  getMatrixPermutation(bis, 20, 8, 8, 8, 
 		       permutationHML,
 		       inversePermutationHML);
 
@@ -124,7 +124,7 @@ int main(int argc, char *argv[])
   {
     /* Add values to density matrix diagonal and one step 
        next to diagonal. */
-    const int nvalues1 = bis->noOfBasisFuncs;
+    const int nvalues1 = bis.noOfBasisFuncs;
     std::vector<int> idxrow(nvalues1);
     std::vector<int> idxcol(nvalues1);
     std::vector<ergo_real> values(nvalues1);
@@ -136,7 +136,7 @@ int main(int argc, char *argv[])
     D.add_values(idxrow, idxcol, values, 
 		 permutationHML,
 		 permutationHML);
-    const int nvalues2 = bis->noOfBasisFuncs-1;
+    const int nvalues2 = bis.noOfBasisFuncs-1;
     for(int i=0; i<nvalues2; i++) {
       idxrow[i] = i;
       idxcol[i] = i+1;
@@ -158,7 +158,7 @@ int main(int argc, char *argv[])
   JK::ExchWeights CAM_params_not_used;
 
   J_K_params.exchange_box_size = 10;
-  if(compute_K_by_boxes_sparse(*bis, *biBasic, CAM_params_not_used, J_K_params, K_1, D,
+  if(compute_K_by_boxes_sparse(bis, biBasic, CAM_params_not_used, J_K_params, K_1, D,
 			       permutationHML,
 			       inversePermutationHML) != 0)
     {
@@ -167,7 +167,7 @@ int main(int argc, char *argv[])
     }
   
   J_K_params.exchange_box_size = 4;
-  if(compute_K_by_boxes_sparse(*bis, *biBasic, CAM_params_not_used, J_K_params, K_2, D,
+  if(compute_K_by_boxes_sparse(bis, biBasic, CAM_params_not_used, J_K_params, K_2, D,
 			       permutationHML,
 			       inversePermutationHML) != 0)
     {
