@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -124,11 +124,10 @@ int main(int argc, char *argv[])
 
 
   
-  std::auto_ptr<IntegralInfo> biBasic(new IntegralInfo(true));
-  std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct);
+  IntegralInfo biBasic(true);
+  BasisInfoStruct bis;
 
-  std::auto_ptr<Molecule> mol(new Molecule);
-  Molecule& m = *mol;
+  Molecule m;
 
   int nx, ny, nz;
   nx = 10;
@@ -157,17 +156,17 @@ int main(int argc, char *argv[])
 	  atomCount++;
 	}
 	
-  if(bis->addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/6-31Gss",
-                                   0, NULL, *biBasic, 0, 0, 0) != 0) {
-    printf("bis->addBasisfuncsForMolecule failed.\n");
+  if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/6-31Gss",
+                                   0, NULL, biBasic, 0, 0, 0) != 0) {
+    printf("bis.addBasisfuncsForMolecule failed.\n");
     return 1;
   }
   
   mat::SizesAndBlocks matrix_size_block_info =
-    prepareMatrixSizesAndBlocks(bis->noOfBasisFuncs,
+    prepareMatrixSizesAndBlocks(bis.noOfBasisFuncs,
 				20, 8, 8, 8);
-  std::vector<int> permutationHML(bis->noOfBasisFuncs);
-  getMatrixPermutation(*bis, 20, 8, 8, 8, permutationHML);
+  std::vector<int> permutationHML(bis.noOfBasisFuncs);
+  getMatrixPermutation(bis, 20, 8, 8, 8, permutationHML);
 
 
   symmMatrix Aempty;
@@ -179,7 +178,7 @@ int main(int argc, char *argv[])
   Adiag.resetSizesAndBlocks(matrix_size_block_info,
 			    matrix_size_block_info);
   {
-    const int n = bis->noOfBasisFuncs;
+    const int n = bis.noOfBasisFuncs;
     const int nvalues = n;
     std::vector<int> idxrow(nvalues);
     std::vector<int> idxcol(nvalues);
@@ -217,7 +216,7 @@ int main(int argc, char *argv[])
 			matrix_size_block_info);
   
   {
-    const int n = bis->noOfBasisFuncs;
+    const int n = bis.noOfBasisFuncs;
     const int nvalues = n*n;
     std::vector<int> idxrow(nvalues);
     std::vector<int> idxcol(nvalues);
