@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 #include <vector>
 #include "realtype.h"
 #include "utilities.h"
-#include "gblas.h"
+#include "mat_gblas.h"
 
 static void fill_matrix_with_random_numbers(int n, std::vector<ergo_real> & A) {
   for(int i = 0; i < n; i++)
