@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,9 +39,9 @@
 
 int main(int argc, char *argv[])
 {
-  std::auto_ptr<IntegralInfo> biBasic(new IntegralInfo(true));
-  std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct);
-  static Molecule m; /* Don't allocate it on stack, it's too large. */
+  IntegralInfo biBasic(true);
+  BasisInfoStruct bis;
+  Molecule m;
   static const long double T[2][2] = {
     {   0.760031854530854163621168L, 0.383253665253394198238115L }, 
     {   0.383253665253394198238115L, 0.760031854530854163621168L }
@@ -51,12 +51,12 @@ int main(int argc, char *argv[])
   m.addAtom(1, 0,0,0);
   m.addAtom(1, 0,0,1);
 
-  if(bis->addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/STO-3G",
-                                   0, NULL, *biBasic, 0, 0, 0) != 0) {
-    printf("bis->addBasisfuncsForMolecule failed.\n");
+  if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/STO-3G",
+                                   0, NULL, biBasic, 0, 0, 0) != 0) {
+    printf("bis.addBasisfuncsForMolecule failed.\n");
     return 1;
   }
-  int n = bis->noOfBasisFuncs;
+  int n = bis.noOfBasisFuncs;
   ergo_real *mat= ergo_new(n*n, ergo_real);
   ergo_real EPS = std::numeric_limits<ergo_real>::epsilon()*10;
   /* Ugly fix because the reference values are only accurate ato
@@ -65,7 +65,7 @@ int main(int argc, char *argv[])
   ergo_real EPS_double = std::numeric_limits<double>::epsilon()*10;
   if(EPS < EPS_double)
     EPS = EPS_double;
-  if (compute_T_matrix_full(*bis, EPS, mat)) {
+  if (compute_T_matrix_full(bis, EPS, mat)) {
     printf("compute_T_matrix_full failed.\n");
     return 1;
   }
