@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,19 +66,19 @@ preparePermutations(const BasisInfoStruct& basisInfo,
 static int 
 test_S_V_comparison(const IntegralInfo & integralInfo)
 {
-  std::auto_ptr<Molecule> moleculePtr1(new Molecule);
-  std::auto_ptr<Molecule> moleculePtr2(new Molecule);
+  Molecule molecule1;
+  Molecule molecule2;
   
   const ergo_real R = 22222.0;
 
-  moleculePtr1->addAtom(5.0, 0.0, 1.0, 0.0);
-  moleculePtr1->addAtom(9.0, 3.0, 0.0, 0.0);
+  molecule1.addAtom(5.0, 0.0, 1.0, 0.0);
+  molecule1.addAtom(9.0, 3.0, 0.0, 0.0);
 
-  moleculePtr2->addAtom(5.0, 0.0, 1.0, R  );
-  moleculePtr2->addAtom(9.0, 3.0, 0.0, R  );
+  molecule2.addAtom(5.0, 0.0, 1.0, R  );
+  molecule2.addAtom(9.0, 3.0, 0.0, R  );
   
-  std::auto_ptr<BasisInfoStruct> basisInfo(new BasisInfoStruct);
-  if(basisInfo->addBasisfuncsForMolecule(*moleculePtr1,
+  BasisInfoStruct basisInfo;
+  if(basisInfo.addBasisfuncsForMolecule(molecule1,
 					 "6-31Gss",
 					 0,
 					 NULL,
@@ -94,13 +94,13 @@ test_S_V_comparison(const IntegralInfo & integralInfo)
   // Get overlap matrix
   std::vector<int> permutationHML, inversePermutationHML;
   mat::SizesAndBlocks sizeBlockInfo;
-  preparePermutations(*basisInfo, sizeBlockInfo,
+  preparePermutations(basisInfo, sizeBlockInfo,
 		      permutationHML, inversePermutationHML);
 
   symmMatrix S;
   S.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
  
-   if(compute_overlap_matrix_sparse(*basisInfo, S, 
+   if(compute_overlap_matrix_sparse(basisInfo, S, 
 				   permutationHML) != 0)
     {
       puts("error in compute_overlap_matrix_sparse");
@@ -112,9 +112,9 @@ test_S_V_comparison(const IntegralInfo & integralInfo)
   ergo_real boxSize = 3.3; // Use small box size to provoke errors.
   symmMatrix V;
   V.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
-  if(compute_V_sparse(*basisInfo,
+  if(compute_V_sparse(basisInfo,
 		      integralInfo,
-		      *moleculePtr2,
+		      molecule2,
 		      threshold,
 		      boxSize,
 		      V,
@@ -147,24 +147,24 @@ test_S_V_comparison(const IntegralInfo & integralInfo)
 static int
 test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
 {
-  std::auto_ptr<Molecule> moleculePtr(new Molecule);
+  Molecule molecule;
   
   // Put some atoms far away to make sure multipoles are used, but
   // still some of them close together so that significant overlaps
   // exist.
-  moleculePtr->addAtom(5.0, 0.0, 1.6, 0.4);
-  moleculePtr->addAtom(9.0, 3.7, 0.0, 0.3);
-  moleculePtr->addAtom(4.0, 1.7, 2.0, 0.8);
-  moleculePtr->addAtom(6.0, 10.0, 21.6, 88.4);
-  moleculePtr->addAtom(7.0, 13.7, 20.0, 88.3);
-  moleculePtr->addAtom(3.0, 11.7, 22.0, 88.8);
-  moleculePtr->addAtom(6.0, 10.0, 31.6, 77.4);
-  moleculePtr->addAtom(9.0, 13.7, 30.0, 77.3);
-  moleculePtr->addAtom(5.0, 11.7, 32.0, 77.8);
-  moleculePtr->addAtom(8.0, 44.7, 44.0, 44.8);  
+  molecule.addAtom(5.0, 0.0, 1.6, 0.4);
+  molecule.addAtom(9.0, 3.7, 0.0, 0.3);
+  molecule.addAtom(4.0, 1.7, 2.0, 0.8);
+  molecule.addAtom(6.0, 10.0, 21.6, 88.4);
+  molecule.addAtom(7.0, 13.7, 20.0, 88.3);
+  molecule.addAtom(3.0, 11.7, 22.0, 88.8);
+  molecule.addAtom(6.0, 10.0, 31.6, 77.4);
+  molecule.addAtom(9.0, 13.7, 30.0, 77.3);
+  molecule.addAtom(5.0, 11.7, 32.0, 77.8);
+  molecule.addAtom(8.0, 44.7, 44.0, 44.8);  
 
-  std::auto_ptr<BasisInfoStruct> basisInfo(new BasisInfoStruct);
-  if(basisInfo->addBasisfuncsForMolecule(*moleculePtr,
+  BasisInfoStruct basisInfo;
+  if(basisInfo.addBasisfuncsForMolecule(molecule,
 					 "6-31Gss",
 					 0,
 					 NULL,
@@ -180,15 +180,15 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
   // Get V matrix
   std::vector<int> permutationHML, inversePermutationHML;
   mat::SizesAndBlocks sizeBlockInfo;
-  preparePermutations(*basisInfo, sizeBlockInfo,
+  preparePermutations(basisInfo, sizeBlockInfo,
 		      permutationHML, inversePermutationHML);
   ergo_real threshold = 1e-12;
   ergo_real boxSize = 3.3; // Use small box size to provoke errors.
   symmMatrix V;
   V.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
-  if(compute_V_sparse(*basisInfo,
+  if(compute_V_sparse(basisInfo,
 		      integralInfo,
-		      *moleculePtr,
+		      molecule,
 		      threshold,
 		      boxSize,
 		      V,
@@ -199,7 +199,7 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
     }
 
   // Convert V to full matrix format
-  int n = basisInfo->noOfBasisFuncs;
+  int n = basisInfo.noOfBasisFuncs;
   std::vector<ergo_real> V_full(n*n);
   V.fullMatrix(V_full, 
 	       inversePermutationHML,
@@ -213,14 +213,14 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
 	// Compute list of simpleprimitives for product of basis funcs i and j.
 	const int maxCount = 888;
 	DistributionSpecStruct list[maxCount];
-	int nPrims = get_product_simple_primitives(*basisInfo, i, *basisInfo, j, list, maxCount, 0);
+	int nPrims = get_product_simple_primitives(basisInfo, i, basisInfo, j, list, maxCount, 0);
 	if(nPrims < 0)
 	  return -1;
 	ergo_real sum = 0;
 	for(int m = 0; m < nPrims; m++)
-	  for(int k = 0; k < moleculePtr->getNoOfAtoms(); k++)
-	    sum += -1 * do_1e_repulsion_integral_using_symb_info(&list[m], moleculePtr->getAtom(k).charge,
-								 moleculePtr->getAtom(k).coords,
+	  for(int k = 0; k < molecule.getNoOfAtoms(); k++)
+	    sum += -1 * do_1e_repulsion_integral_using_symb_info(&list[m], molecule.getAtom(k).charge,
+								 molecule.getAtom(k).coords,
 								 integralInfo);
 	ergo_real absDiff = fabs(V_full[i*n+j] - sum);
 	if(absDiff > maxAbsDiff)
@@ -235,8 +235,8 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
 
   // Also test the compute_h_core_matrix_full routine.
   std::vector<ergo_real> V_full_other(n*n);
-  if(compute_V_matrix_full(*basisInfo, integralInfo, moleculePtr->getNoOfAtoms(),
-			   moleculePtr->getAtomListPtr(), threshold, &V_full_other[0]) != 0)
+  if(compute_V_matrix_full(basisInfo, integralInfo, molecule.getNoOfAtoms(),
+			   molecule.getAtomListPtr(), threshold, &V_full_other[0]) != 0)
     {
       puts("error in compute_V_matrix_full");
       return -1;
@@ -265,28 +265,28 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
 static int
 test_V_by_explicit_comparison_tight(const IntegralInfo & integralInfo)
 {
-  std::auto_ptr<Molecule> moleculePtr(new Molecule);
+  Molecule molecule;
 
   // Put atoms close together so that basis funcs overlap with most
   // other basis funcs.
-  moleculePtr->addAtom(1.0, 0.0, 1.6, 0.4);
-  moleculePtr->addAtom(1.0, 3.7, 0.0, 0.3);
-  moleculePtr->addAtom(1.0, 1.7, 2.0, 0.8);
-  moleculePtr->addAtom(1.0, 0.1, 1.3, 0.4);
-  moleculePtr->addAtom(1.0, 3.8, 0.3, 0.3);
-  moleculePtr->addAtom(1.0, 1.8, 2.3, 0.8);
-  moleculePtr->addAtom(1.0, 0.3, 1.7, 0.4);
-  moleculePtr->addAtom(1.0, 3.5, 0.7, 0.3);
-  moleculePtr->addAtom(1.0, 1.2, 2.7, 0.8);
-  moleculePtr->addAtom(1.0, 0.3, 1.1, 0.4);
-  moleculePtr->addAtom(1.0, 3.6, 0.2, 0.3);
-  moleculePtr->addAtom(1.0, 1.9, 2.3, 0.8);
-  moleculePtr->addAtom(1.0, 0.3, 1.1, 1.1);
-  moleculePtr->addAtom(1.0, 3.6, 0.2, 1.2);
-  moleculePtr->addAtom(1.0, 1.9, 2.3, 1.4);
+  molecule.addAtom(1.0, 0.0, 1.6, 0.4);
+  molecule.addAtom(1.0, 3.7, 0.0, 0.3);
+  molecule.addAtom(1.0, 1.7, 2.0, 0.8);
+  molecule.addAtom(1.0, 0.1, 1.3, 0.4);
+  molecule.addAtom(1.0, 3.8, 0.3, 0.3);
+  molecule.addAtom(1.0, 1.8, 2.3, 0.8);
+  molecule.addAtom(1.0, 0.3, 1.7, 0.4);
+  molecule.addAtom(1.0, 3.5, 0.7, 0.3);
+  molecule.addAtom(1.0, 1.2, 2.7, 0.8);
+  molecule.addAtom(1.0, 0.3, 1.1, 0.4);
+  molecule.addAtom(1.0, 3.6, 0.2, 0.3);
+  molecule.addAtom(1.0, 1.9, 2.3, 0.8);
+  molecule.addAtom(1.0, 0.3, 1.1, 1.1);
+  molecule.addAtom(1.0, 3.6, 0.2, 1.2);
+  molecule.addAtom(1.0, 1.9, 2.3, 1.4);
 
-  std::auto_ptr<BasisInfoStruct> basisInfo(new BasisInfoStruct);
-  if(basisInfo->addBasisfuncsForMolecule(*moleculePtr,
+  BasisInfoStruct basisInfo;
+  if(basisInfo.addBasisfuncsForMolecule(molecule,
 					 "STO-3G",
 					 0,
 					 NULL,
@@ -302,15 +302,15 @@ test_V_by_explicit_comparison_tight(const IntegralInfo & integralInfo)
   // Get V matrix
   std::vector<int> permutationHML, inversePermutationHML;
   mat::SizesAndBlocks sizeBlockInfo;
-  preparePermutations(*basisInfo, sizeBlockInfo,
+  preparePermutations(basisInfo, sizeBlockInfo,
 		      permutationHML, inversePermutationHML);
   ergo_real threshold = 1e-12;
   ergo_real boxSize = 3.3; // Use small box size to provoke errors.
   symmMatrix V;
   V.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
-  if(compute_V_sparse(*basisInfo,
+  if(compute_V_sparse(basisInfo,
 		      integralInfo,
-		      *moleculePtr,
+		      molecule,
 		      threshold,
 		      boxSize,
 		      V,
@@ -321,7 +321,7 @@ test_V_by_explicit_comparison_tight(const IntegralInfo & integralInfo)
     }
 
   // Convert V to full matrix format
-  int n = basisInfo->noOfBasisFuncs;
+  int n = basisInfo.noOfBasisFuncs;
   std::vector<ergo_real> V_full(n*n);
   V.fullMatrix(V_full, inversePermutationHML, inversePermutationHML);
 
@@ -333,14 +333,14 @@ test_V_by_explicit_comparison_tight(const IntegralInfo & integralInfo)
 	// Compute list of simpleprimitives for product of basis funcs i and j.
 	const int maxCount = 888;
 	DistributionSpecStruct list[maxCount];
-	int nPrims = get_product_simple_primitives(*basisInfo, i, *basisInfo, j, list, maxCount, 0);
+	int nPrims = get_product_simple_primitives(basisInfo, i, basisInfo, j, list, maxCount, 0);
 	if(nPrims < 0)
 	  return -1;
 	ergo_real sum = 0;
 	for(int m = 0; m < nPrims; m++)
-	  for(int k = 0; k < moleculePtr->getNoOfAtoms(); k++)
-	    sum += -1 * do_1e_repulsion_integral_using_symb_info(&list[m], moleculePtr->getAtom(k).charge,
-								 moleculePtr->getAtom(k).coords,
+	  for(int k = 0; k < molecule.getNoOfAtoms(); k++)
+	    sum += -1 * do_1e_repulsion_integral_using_symb_info(&list[m], molecule.getAtom(k).charge,
+								 molecule.getAtom(k).coords,
 								 integralInfo);
 	ergo_real absDiff = fabs(V_full[i*n+j] - sum);
 	if(absDiff > maxAbsDiff)
@@ -359,15 +359,15 @@ test_V_by_explicit_comparison_tight(const IntegralInfo & integralInfo)
 
 int main(int argc, char *argv[])
 {
-  std::auto_ptr<IntegralInfo> integralInfo(new IntegralInfo(true));
+  IntegralInfo integralInfo(true);
   int errorCount = 0;
-  if(test_S_V_comparison(*integralInfo) != 0)
+  if(test_S_V_comparison(integralInfo) != 0)
     {
       puts("error in test_S_V_comparison.");
       errorCount++;
     }
 
-  if(test_V_by_explicit_comparison(*integralInfo) != 0)
+  if(test_V_by_explicit_comparison(integralInfo) != 0)
     {
       puts("error in test_V_by_explicit_comparison.");
       errorCount++;
@@ -378,7 +378,7 @@ int main(int argc, char *argv[])
     N = atoi(argv[1]);
   for(int i = 0; i < N; i++)
     {
-      if(test_V_by_explicit_comparison_tight(*integralInfo) != 0)
+      if(test_V_by_explicit_comparison_tight(integralInfo) != 0)
 	errorCount++;
     }
   printf("test_V_by_explicit_comparison_tight test repeated, %i times, errorCount = %i\n", N, errorCount);
