@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,10 +52,10 @@ calculation_shared(const IntegralInfo& ii, const Molecule& mol,
 		   bool useHiCu)
 {
   time_t tm; time(&tm);
-  std::auto_ptr<BasisInfoStruct> bis(new BasisInfoStruct());
-  if(bis->addBasisfuncsForMolecule(mol, ERGO_SPREFIX "/basis/4-31G",
+  BasisInfoStruct bis;
+  if(bis.addBasisfuncsForMolecule(mol, ERGO_SPREFIX "/basis/4-31G",
                                    0, NULL, ii, 0, 0, 0) != 0) {
-    printf("bis->addBasisfuncsForMolecule failed.\n");
+    printf("bis.addBasisfuncsForMolecule failed.\n");
     throw "addBasisfuncs failed";
   }
   if(dft_setfunc(funcName) == 0)
@@ -70,9 +70,9 @@ calculation_shared(const IntegralInfo& ii, const Molecule& mol,
   int nElectrons = mol.getNumberOfElectrons();
 
   mat::SizesAndBlocks matrix_size_block_info =
-    prepareMatrixSizesAndBlocks(bis->noOfBasisFuncs, blSize,
+    prepareMatrixSizesAndBlocks(bis.noOfBasisFuncs, blSize,
 				blFactor, blFactor, blFactor);
-  getMatrixPermutation(*bis, blSize,
+  getMatrixPermutation(bis, blSize,
 		       blFactor, blFactor, blFactor, 
 		       permutationHML);
   
@@ -83,16 +83,16 @@ calculation_shared(const IntegralInfo& ii, const Molecule& mol,
 				   matrix_size_block_info);
 
   {
-    std::vector<int> idx(bis->noOfBasisFuncs);
-    std::vector<ergo_real> values(bis->noOfBasisFuncs);
-    for(int i=0; i<bis->noOfBasisFuncs; i++) {
+    std::vector<int> idx(bis.noOfBasisFuncs);
+    std::vector<ergo_real> values(bis.noOfBasisFuncs);
+    for(int i=0; i<bis.noOfBasisFuncs; i++) {
       idx[i] = i;
       values[i] = 1.0;
     }
     dmat.add_values(idx, idx, values, permutationHML, permutationHML);
   }
 
-  Dft::getXC_mt(*bis, ii, mol, gridParams, nElectrons, dmat,
+  Dft::getXC_mt(bis, ii, mol, gridParams, nElectrons, dmat,
 		xcMat, energy, permutationHML);
 
   if(PRINT_TIME)
@@ -107,11 +107,11 @@ small_calculation_core(const IntegralInfo& ii,
 {
   bool failed = false;
   
-  std::auto_ptr<Molecule> m(new Molecule);
+  Molecule m;
   /* The code later will change the order of atoms, this is why the
      reference table may seem strange at the first sight. */
-  m->addAtom(2, 0,0,0);
-  m->addAtom(1, 0,0,1.5);
+  m.addAtom(2, 0,0,0);
+  m.addAtom(1, 0,0,1.5);
 
   static const int BL_SIZE = 4;
   static const int BL_FACTOR = 2;
@@ -119,7 +119,7 @@ small_calculation_core(const IntegralInfo& ii,
   symmMatrix xcMat;
   ergo_real dftEnergy;
   std::vector<int> permutationHML;
-  calculation_shared(ii, *m, functionalName, BL_SIZE, BL_FACTOR,
+  calculation_shared(ii, m, functionalName, BL_SIZE, BL_FACTOR,
 		     xcMat, &dftEnergy, permutationHML, useHiCu);
 
   /* We give some room to accumulation error. For long double calculation,
@@ -238,10 +238,10 @@ benchmark_calculation(const IntegralInfo& ii, int sideLength)
   bool failed = false;
   int nElectrons = 0;
   
-  std::auto_ptr<Molecule> mol(new Molecule);
+  Molecule mol;
   for(int i=0; i<sideLength; i++) {
     for(int j=0; j<sideLength; j++)
-      mol->addAtom(CHARGE, 0,
+      mol.addAtom(CHARGE, 0,
                    i*DISTANCE_BETWEEN_ATOMS,j*DISTANCE_BETWEEN_ATOMS);
     nElectrons += CHARGE*sideLength;
   }
@@ -253,7 +253,7 @@ benchmark_calculation(const IntegralInfo& ii, int sideLength)
 
   ergo_real dftEnergy;
   std::vector<int> permutationHML;
-  calculation_shared(ii, *mol, "SVWN5", BL_SIZE, BL_FACTOR,
+  calculation_shared(ii, mol, "SVWN5", BL_SIZE, BL_FACTOR,
 		     xcMatrix, &dftEnergy, permutationHML, false);
 #if 1
   /* We give some room to accumulation error. */
@@ -302,7 +302,7 @@ mol_calculation(const IntegralInfo& ii, const char *fname)
 int main(int argc, char *argv[])
 {
   static const int PROBLEM_SQUARE_SIDE_LENGTH = 60;
-  std::auto_ptr<IntegralInfo> ii(new IntegralInfo(true));
+  IntegralInfo ii(true);
   dft_init();
   const char *tmpdir = getenv("TMPDIR");
   tmpdir = tmpdir ? tmpdir : "/tmp";
@@ -310,16 +310,16 @@ int main(int argc, char *argv[])
   bool success;
   if(getenv("RUN_BENCHMARK")) {
     printf("Running an XC benchmark, tmpdir=%s.\n", tmpdir);
-    success = benchmark_calculation(*ii, PROBLEM_SQUARE_SIDE_LENGTH);
+    success = benchmark_calculation(ii, PROBLEM_SQUARE_SIDE_LENGTH);
   } else {
     if(argc>1) {
       int side = strtol(argv[1], NULL, 10);
       if(side)
-        success = benchmark_calculation(*ii, side);
+        success = benchmark_calculation(ii, side);
       else
-        success = mol_calculation(*ii, argv[1]);
+        success = mol_calculation(ii, argv[1]);
     } else 
-      success = small_calculation(*ii);
+      success = small_calculation(ii);
   }
 
   printf("Success: %s\n", success ? "YES" : "NO");
