@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -74,8 +74,7 @@ test_small(const IntegralInfo& ii, const char *functional,
            const long double (*XCRef)[2])
 {
   BasisInfoStruct* bis = new BasisInfoStruct();
-  std::auto_ptr<Molecule> mol(new Molecule);
-  Molecule& m = *mol;
+  Molecule m;
   /* The code later will change the order of atoms, this is why the
      reference table may seem strange at the first sight. */
   for(int i=0; i<2; i++) {
@@ -180,7 +179,7 @@ static int
 test_small_both()
 {
   int res = 0;
-  std::auto_ptr<IntegralInfo> ii(new IntegralInfo(true));
+  IntegralInfo ii(true);
 
   static const int sys1Z[2] = { 2, 1 };
   static const ergo_real sys1C[2][3] = { { 0, 0, 0 }, { 0, 0, 1.5 } };
@@ -216,13 +215,13 @@ test_small_both()
   };
 #endif
 
-  res += test_small(*ii, "SVWN5", Dft::GridParams::GC2, "GC2  ",
+  res += test_small(ii, "SVWN5", Dft::GridParams::GC2, "GC2  ",
                     sys1Z, sys1C, &XCRefSys1Svwn5_GC2[0]);
 
-  res += test_small(*ii, "SVWN5", Dft::GridParams::TURBO, "Turbo",
+  res += test_small(ii, "SVWN5", Dft::GridParams::TURBO, "Turbo",
                     sys1Z, sys1C, &XCRefSys1Svwn5_Turbo[0]);
 
-  res += test_small(*ii, "SVWN5", Dft::GridParams::LMG, "LMG  ",
+  res += test_small(ii, "SVWN5", Dft::GridParams::LMG, "LMG  ",
                     sys1Z, sys1C, &XCRefSys1Svwn5_LMG[0]);
 #if 0
   /* these used to work at some point in time */
@@ -248,9 +247,9 @@ test_small_both()
     { -0.2847608922553022361067940L, -0.6584790455338922763911698L }
   };
 #endif
-  res += test_small(*ii, "BP86", Dft::GridParams::LMG, "LMG  ",
+  res += test_small(ii, "BP86", Dft::GridParams::LMG, "LMG  ",
                     sys1Z, sys1C, &XCRefSys1BP86_LMG[0]);
-  res += test_small(*ii, "BP86", Dft::GridParams::TURBO, "Turbo",
+  res += test_small(ii, "BP86", Dft::GridParams::TURBO, "Turbo",
                     sys1Z, sys1C, &XCRefSys1BP86_TURBO[0]);
 
 
@@ -282,11 +281,11 @@ test_small_both()
     {  0.0,                          -0.6158981042254555427991648L }
   };
 #endif
-  res += test_small(*ii, "Combine Slater=1 PZ81=1",
+  res += test_small(ii, "Combine Slater=1 PZ81=1",
                     Dft::GridParams::LMG, "LMG  ",
 		    sys2Z, sys2C, &XCRefSys2Combine[0]);
 
-  res += test_small(*ii, "BLYP", Dft::GridParams::LMG, "LMG  ",
+  res += test_small(ii, "BLYP", Dft::GridParams::LMG, "LMG  ",
                     sys2Z, sys2C, &XCRefSys2Blyp[0]);
   return res;
 }
@@ -297,19 +296,18 @@ test_mol(const char *mol_fname, const char *basisSet, const char *xcFunc)
   unlink("ergoscf.out");
   dft_init();
 
-  std::auto_ptr<Molecule> mol(new Molecule);
-  Molecule& m = *mol;
+  Molecule m;
 
   if(m.setFromMoleculeFile(mol_fname, 0, NULL) != 0) {
     printf("Molecule::setFromMoleculeFile failed.\n");
     return 1;
   }
 
-  std::auto_ptr<IntegralInfo> biBasic(new IntegralInfo(true));
+  IntegralInfo biBasic(true);
   BasisInfoStruct *bis = new BasisInfoStruct();
 
   if(bis->addBasisfuncsForMolecule(m, basisSet, 0, NULL,
-                                   *biBasic, 0, 1, 0) != 0) {
+                                   biBasic, 0, 1, 0) != 0) {
     printf("bis->addBasisfuncsForMolecule failed.\n");
     delete bis;
     return 1;
