@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 
-#include "density_description_file_2.h"
+#include "density_description_file.h"
 #include "density_projection.h"
 #include "densfromf_full.h"
 #include "densfromf_sparse.h"
@@ -436,6 +436,7 @@ load_density_and_project_sparse(const char *densityFileName,
 
       std::map<std::string, double> puri_stats;
       ergo_real electronicEntropyTerm_dummy = 0;
+      std::vector< std::vector<ergo_real> > stochasticOrbsDummy;
       if(get_dens_from_fock_general(n,
 				    noOfOccupiedOrbs,
 				    use_diagonalization,
@@ -462,6 +463,8 @@ load_density_and_project_sparse(const char *densityFileName,
 				    purification_create_m_files,
 				    purification_ignore_failure,
 				    purification_use_rand_perturbation_for_alleigsint,
+				    0,
+				    stochasticOrbsDummy,
 				    "",
 				    puri_stats,
 				    0, // no sparsity investigation
