@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "integral_matrix_wrappers.h"
 #include "output.h"
 #include "basis_func_pair_list_1el.h"
+#include "integrals_1el_kinetic.h"
 #include "integrals_1el_potential.h"
 #include "operator_matrix.h"
 #include "basis_func_pair_list.h"
@@ -38,6 +39,18 @@
 #include "integrals_2el_exchange.h"
 #include "integrals_2el_exchange_prep.h"
 #include "utilities.h"
+#include "memorymanag.h"
+
+
+static ergo_real get_max_charge(const Molecule& molecule) {
+  ergo_real maxCharge = 0;
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
+    ergo_real currCharge = molecule.getAtom(i).charge;
+    if(currCharge > maxCharge)
+      maxCharge = currCharge;
+  }
+  return maxCharge;
+}
 
 
 int
@@ -51,14 +64,7 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 {
   int n = basisInfo.noOfBasisFuncs;
 
-  // Get maxCharge
-  ergo_real maxCharge = 0;
-  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
-    {
-      ergo_real currCharge = molecule.getAtom(i).charge;
-      if(currCharge > maxCharge)
-	maxCharge = currCharge;
-    }
+  ergo_real maxCharge = get_max_charge(molecule);
 
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
 	    "compute_V_sparse, no of atoms = %5i, maxCharge = %6.2f", 
@@ -91,16 +97,17 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 	    "noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
 	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
 
-  if(compute_V_linear(basisInfo,
-		      integralInfo,
-		      molecule,
-		      threshold,
-		      boxSize,
-		      &basisFuncIndexPairList[0],
-		      &V_list[0],
-		      noOfBasisFuncIndexPairs) != 0)
+  if(compute_V_and_gradient_linear(basisInfo,
+				   integralInfo,
+				   molecule,
+				   threshold,
+				   boxSize,
+				   &basisFuncIndexPairList[0],
+				   &V_list[0],
+				   noOfBasisFuncIndexPairs,
+				   false, NULL, NULL) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_V_linear");
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_V_and_gradient_linear");
       return -1;
     }
   
@@ -125,6 +132,150 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 }
 
 
+int
+compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
+				  const IntegralInfo& integralInfo,
+				  const Molecule& molecule,
+				  ergo_real threshold,
+				  ergo_real boxSize,
+				  const symmMatrix & densityMatrix_sparse,
+				  std::vector<int> const & permutationHML,
+				  ergo_real* result_gradient_list) {
+  int n = basisInfo.noOfBasisFuncs;
+
+  ergo_real maxCharge = get_max_charge(molecule);
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+	    "compute_gradient_of_tr_D_V, no of atoms = %5i, maxCharge = %6.2f", 
+	    molecule.getNoOfAtoms(), (double)maxCharge);
+
+  int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo, threshold, maxCharge, NULL, 2000000000);
+  if(noOfBasisFuncIndexPairs <= 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    return -1;
+  }
+  std::vector<basis_func_index_pair_struct_1el> basisFuncIndexPairList(noOfBasisFuncIndexPairs);
+  std::vector<ergo_real> V_list(noOfBasisFuncIndexPairs);
+  noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo, threshold, maxCharge, &basisFuncIndexPairList[0], noOfBasisFuncIndexPairs);
+  if(noOfBasisFuncIndexPairs <= 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    return -1;
+  }
+  
+  // Get density matrix elements
+  std::vector<ergo_real> D_list(noOfBasisFuncIndexPairs);
+  {
+    std::vector<int> rowind(noOfBasisFuncIndexPairs);
+    std::vector<int> colind(noOfBasisFuncIndexPairs);
+    for(int i = 0; i < noOfBasisFuncIndexPairs; i++) {
+      rowind[i] = basisFuncIndexPairList[i].index_1;
+      colind[i] = basisFuncIndexPairList[i].index_2;
+    }
+    densityMatrix_sparse.get_values(rowind, colind, D_list, permutationHML, permutationHML);
+  }
+  
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+	    "noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
+	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
+
+  if(compute_V_and_gradient_linear(basisInfo,
+				   integralInfo,
+				   molecule,
+				   threshold,
+				   boxSize,
+				   &basisFuncIndexPairList[0],
+				   &V_list[0],
+				   noOfBasisFuncIndexPairs,
+				   true, &D_list[0],
+				   result_gradient_list) != 0)
+    {
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_V_and_gradient_linear");
+      return -1;
+    }
+  
+  return 0;
+}
+
+
+ergo_real
+get_electron_nuclear_attraction_energy(const IntegralInfo& integralInfo,
+				       const Molecule& molecule,
+				       const BasisInfoStruct& basisInfo,
+				       const symmMatrix & D,
+				       ergo_real threshold_integrals_1el,
+				       mat::SizesAndBlocks const & matrix_size_block_info,
+				       std::vector<int> const & permutationHML)
+{
+  // Get V
+  symmMatrix V;
+  V.resetSizesAndBlocks(matrix_size_block_info, matrix_size_block_info);
+  ergo_real boxSize = 10.0;
+  if(compute_V_sparse(basisInfo,
+		      integralInfo, 
+		      molecule,
+		      threshold_integrals_1el,
+		      boxSize,
+		      V,
+		      permutationHML) != 0)
+    throw "error in compute_V_sparse";
+  return symmMatrix::trace_ab(D, V);
+}
+
+
+int
+compute_T_sparse(const BasisInfoStruct& basisInfo,
+		 const IntegralInfo& integralInfo,
+		 ergo_real threshold,
+		 symmMatrix & T,
+		 std::vector<int> const & permutationHML) {
+  int n = basisInfo.noOfBasisFuncs;
+  std::vector<int> nvaluesList(n);
+  std::vector<int*> colindList(n);
+  std::vector<ergo_real*> valuesList(n);
+  if(compute_T_matrix_sparse(basisInfo,
+			     threshold,
+			     n,
+			     &nvaluesList[0],
+			     &colindList[0],
+			     &valuesList[0]) != 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_T_matrix_sparse");
+    return -1;
+  }
+  // Now convert result to three vectors so prepare for HML storage.
+  int nvalues = 0;
+  for(int i = 0; i < n; i++)
+    nvalues += nvaluesList[i];
+  // allocate vectors
+  std::vector<int> rowind(nvalues);
+  std::vector<int> colind(nvalues);
+  std::vector<ergo_real> values(nvalues);
+  // populate vectors
+  int count = 0;
+  for(int i = 0; i < n; i++)
+    {
+      for(int j = 0; j < nvaluesList[i]; j++)
+	{
+	  rowind[count] = i;
+	  colind[count] = colindList[i][j];
+	  values[count] = valuesList[i][j];
+	  count++;
+	} // END FOR j
+    } // END FOR i
+  // Now the information is in rowind colind values.
+  // free memory allocated by compute_operator_matrix_sparse.
+  for(int i = 0; i < n; i++) {
+    ergo_free(colindList[i]);
+    ergo_free(valuesList[i]);
+  }
+  T.assign_from_sparse(rowind,
+		       colind,
+		       values,
+		       permutationHML,
+		       permutationHML);
+  return 0;
+}
+
+
 
 static int
 check_diagonal_elements_of_overlap_matrix(int n, const symmMatrix & S_symm)
