@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,35 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 		 std::vector<int> const & permutationHML);
 
 
+int
+compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
+				  const IntegralInfo& integralInfo,
+				  const Molecule& molecule,
+				  ergo_real threshold,
+				  ergo_real boxSize,
+				  const symmMatrix & densityMatrix_sparse,
+				  std::vector<int> const & permutationHML,
+				  ergo_real* result_gradient_list);
+
+
+ergo_real
+get_electron_nuclear_attraction_energy(const IntegralInfo& integralInfo,
+				       const Molecule& molecule,
+				       const BasisInfoStruct& basisInfo,
+				       const symmMatrix & D,
+				       ergo_real threshold_integrals_1el,
+				       mat::SizesAndBlocks const & matrix_size_block_info,
+				       std::vector<int> const & permutationHML);
+
+
+int
+compute_T_sparse(const BasisInfoStruct& basisInfo,
+		 const IntegralInfo& integralInfo,
+		 ergo_real threshold,
+		 symmMatrix & T,
+		 std::vector<int> const & permutationHML);
+
+
 int
 compute_overlap_matrix_sparse(const BasisInfoStruct& basisInfo,
 			      symmMatrix & S_symm,
