@@ -1,6 +1,6 @@
-/* Ergo, version 3.3, a program for linear scaling electronic structure
+/* Ergo, version 3.4, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2013 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include "memorymanag.h"
 #include "output.h"
 
-#include "../matrix/gblas.h"
+#include "../matrix/mat_gblas.h"
 
 
 #define USE_BLAS_MM
@@ -51,6 +51,36 @@ void dgemm_(const char *ta,const char *tb,
 #endif
 
 
+void multiply_matrices_general_2(int An1, int An2, int Bn1, int Bn2, const ergo_real* A, const ergo_real* B, ergo_real* AB, bool initToZero) {
+  if(An2 != Bn1) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_LOWLEVEL, "error in multiply_matrices_general_2: (An2 != Bn1)");
+    exit(0);
+  }
+#ifdef USE_BLAS_MM
+  if(An1 == 0 || An2 == 0 || Bn1 == 0 || Bn2 == 0)
+    return;
+  /*  call gemm */
+  ergo_real alpha = 1;
+  ergo_real beta = 1;
+  if(initToZero)
+    beta = 0;
+  mat::gemm("N", "N", &Bn2, &An1, &Bn1, &alpha, 
+	    B, &Bn2, 
+	    A, &An2, 
+	    &beta, 
+	    AB, &Bn2);
+#else
+  if(initToZero)
+    memset(AB, 0, An1*Bn2*sizeof(ergo_real));
+  for(int i = 0; i < An1; i++)
+    for(int k = 0; k < An2; k++)
+      for(int j = 0; j < Bn2; j++)
+	AB[i*Bn2+j] += A[i*An2+k] * B[k*Bn2+j];
+#endif
+}
+
+
+
 /*
   Standard matrix multiplication.
 */
@@ -89,7 +119,7 @@ multiply_matrices_general(int An1, int An2, int Bn1, int Bn2, const ergo_real* A
     for(j = 0; j < Bn2; j++)
       {
 	ergo_real sum = 0;
-	for(k = 0; k < An2; k++)
+	for(int k = 0; k < An2; k++)
 	  sum += A[i*An2+k] * B[k*Bn2+j];
 	AB[i*Bn2+j] = sum;
       }
